package mpi.eudico.client.annotator.lexica;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;

import mpi.eudico.client.annotator.gui.ClosableDialog;
import mpi.eudico.client.annotator.lexica.record.LXRecordQueries;
import mpi.eudico.client.annotator.viewer.AbstractViewer;
import mpi.eudico.client.annotator.viewer.LexiconViewer;

public class ChooseReferenceDialog extends ClosableDialog implements MouseListener, ActionListener {

	private List<LexiconRef> lexiconRefList;
	private List<JTable> tableList;
	private JButton apply;
	private ParsingMorpheme parsingMorpheme;
	private AbstractViewer viewer;
	
	public ChooseReferenceDialog(AbstractViewer viewer, ParsingMorpheme parsingMorpheme, List<LexiconRef> lexiconRefList){
		this.viewer=viewer;
		this.lexiconRefList=lexiconRefList;
		this.parsingMorpheme=parsingMorpheme;
		tableList=new ArrayList<JTable>();
		apply=new JButton();
		init();
	}
	
	private void init(){
		setTitle("Choose appropriate segments");
		this.setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
        Insets insets = new Insets(2, 2, 12, 2);
        gbc.insets = insets;
        gbc.fill = GridBagConstraints.HORIZONTAL;
       
        boolean error=false;
        int j=0;
        for(int i=0; i<lexiconRefList.size(); i++){
        	LexiconRef lexiconRef=lexiconRefList.get(i);
        	if(lexiconRef!=null){
        		ChooseReferenceTableModel model=new ChooseReferenceTableModel(lexiconRef);
        		JTable table=new JTable(model);
        		table.setDefaultRenderer(Object.class, new ChooseReferenceTableRenderer());
        		table.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        		table.setRowSelectionInterval(0, 0);
        		table.setAutoscrolls(true);
        		table.addMouseListener(this);
        		gbc.gridx=0;
            	gbc.gridy=j*4;
            	gbc.weighty=1.0;
                gbc.weightx=1.0;
            	gbc.gridheight=4;
            	add(table, gbc);
                j++;
                tableList.add(table);
        		
        	}
        	else{
        		error=true;
        		gbc.gridx=0;
            	gbc.gridy=j*4;
            	gbc.gridheight=4;
        		JLabel label=new JLabel("<html>The segment '"+parsingMorpheme.getLexiconMorphemesList().get(i)+"' doesn't exist,<br/> Insert it before interlinearize operation</html>");
        		add(label,gbc);
        		j++;
        	}
		}
//        int i=0;
//        for(JTable table:tableList){
//        	gbc.gridx=0;
//        	gbc.gridy=i*4;
//        	gbc.weighty=1.0;
//            gbc.weightx=1.0;
//        	gbc.gridheight=4;
//        	add(table, gbc);
//            i++;
//        }
        if(!error){
        	j=j*4;
        	gbc.gridx=0;
        	gbc.gridy=j;
        	gbc.gridheight=1;
        	gbc.anchor=GridBagConstraints.CENTER;
        	JLabel label=new JLabel("Choose appropriate Segment and Validate");
        	add(label, gbc);

        	gbc.gridx=0;
        	gbc.gridy=j+1;
        	gbc.gridheight=1;
        	apply.setText("OK");
        	apply.addActionListener(this);
        	add(apply,gbc);
        }
        
		pack();
		setLocationRelativeTo(getParent());
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==apply){
			List<String> idList=new ArrayList<String>();
			for(JTable table :tableList){
				int row=table.getSelectedRow();
				if(table.getValueAt(row, 0) instanceof LexiconTableContent){
					LexiconTableContent lexTableContent=(LexiconTableContent)table.getValueAt(row, 0);
					idList.add(lexTableContent.getAssociatedLexicalEntryId());
					//LexiconRef lexRef=new LexiconRef(lexTableContent.getLx());
					//lexRef.addLexicon(lexTableContent);
					
				} 
			}
			if(parsingMorpheme.getLexiconMorphemesList().size()==idList.size()){
				LXRecordQueries query=new LXRecordQueries(viewer, parsingMorpheme.getAssociatedId());
				boolean isInserted=query.actionEditParseReferences(parsingMorpheme,idList);
				if(isInserted){
					this.setVisible(false);
					this.dispose();
					((LexiconViewer) viewer).updateLexiconChange();
					//InterlinearizationProcess.interlinearize(viewer, false);
				} else{
					JOptionPane.showMessageDialog(this, "Modification failed\n Please check your datas", "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
			else{
				JOptionPane.showMessageDialog(this, "Modification failed\n Please check your datas", "Error", JOptionPane.ERROR_MESSAGE);
			}

//				System.out.println("element selected="+table.getValueAt(row, ChooseReferenceTableModel.ID));
//				LexiconRef lexRef=new LexiconRef((String)table.getValueAt(row, ChooseReferenceTableModel.LX));
				//lexRef.addLexicon(lexicon)
			//parsingMorpheme.setLexiconRefList(lexiconRefList);
		}
	}
}
