package mpi.eudico.client.annotator.lexica;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import mpi.eudico.client.annotator.commands.NewAnnotationWithValueCommand;	// CC 310811
import mpi.eudico.client.annotator.commands.AnnotationValAfterCommand;	// CC 310811
import mpi.eudico.client.annotator.commands.DeleteSelectedAnnotationsCommand;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.viewer.AbstractViewer;
import mpi.eudico.client.annotator.viewer.LexiconViewer;
import mpi.eudico.server.corpora.clom.Annotation;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.AbstractAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;

import org.jdom.Element;

/**
 * Algorithme to parse the selected morpheme and init the data from the downloaded lexicon
 * @author Coralie
 *
 */
public class InterlinearizationProcess {
	private Transcription transcription;		// CC 031011
	//list of entries to interlinearize
	private List<ParseRef> parseForms;
	
	private List<ParsingMorpheme> rootUnderlyingForms;
	private List<ParsingMorpheme> stemUnderlyingForms;
	private List<ParsingMorpheme> suffixUnderlyingForms;
	private List<ParsingMorpheme> prefixUnderlyingForms;
	
	private List<LexiconRef> rootForms;
	private List<LexiconRef> stemForms;
	private List<LexiconRef> suffixForms;
	private List<LexiconRef> prefixForms;

	private List<LexiconMorphemes2> parsedEntries;
	private List<LexiconTableContent> entries;
	
	private String stemChar;
	
	//list of patterns
	private Pattern inBrackets=Pattern.compile(".*.*\u2022.*");// 2022
	public static Pattern stemPattern;
	public static Pattern endingPattern;
	public static Pattern beginningPattern;
	public static Pattern specialCharPattern=Pattern.compile("[\\/\\*\\.\\^\\$\\-\\(\\)\\?\\{\\}\\+\\[\\]]");
	
	//tier to annotate
	private TierImpl ct=null;
	private TierImpl mbTier=null;
	private TierImpl geTier=null;
	private TierImpl rxTier=null;
	
	private AbstractViewer viewer;
	private DeleteSelectedAnnotationsCommand comDel;		// CC 310811
	private NewAnnotationWithValueCommand comNewMb;
	private NewAnnotationWithValueCommand comNewGe;
	private NewAnnotationWithValueCommand comNewRx;
	private AnnotationValAfterCommand comNewAfterMb;
	private AnnotationValAfterCommand comNewAfterGe;
	private AnnotationValAfterCommand comNewAfterRx;

	/**constructor
	 * 
	 * @param model
	 */
	public InterlinearizationProcess(LexiconTableModel model, List<ParsingMorpheme> parsingMorphemeList, LexiconViewer viewer){
		this.viewer=viewer;
		this.entries=model.getEntries();
		initPatterns();
		initEntries2(parsingMorphemeList);
	}
	
	/**
	 * Init the data from the downloaded lexicon
	 */
	private void initEntries2(List<ParsingMorpheme> parsingMorphemeList){
		parsedEntries=new ArrayList<LexiconMorphemes2>();
		
		parseForms=new ArrayList<ParseRef>();
		if(ParseDocumentData.DOCUMENT!=null){
			initParseEntries();
		}
		rootUnderlyingForms=new ArrayList<ParsingMorpheme>();
		suffixUnderlyingForms=new ArrayList<ParsingMorpheme>();
		prefixUnderlyingForms=new ArrayList<ParsingMorpheme>();
		stemUnderlyingForms=new ArrayList<ParsingMorpheme>();
		rootForms=new ArrayList<LexiconRef>();
		prefixForms=new ArrayList<LexiconRef>();
		suffixForms=new ArrayList<LexiconRef>();
		stemForms=new ArrayList<LexiconRef>();
		if(entries!=null){
			for(int i=0; i<entries.size(); i++){
				LexiconTableContent entry=entries.get(i);
				String lx=entry.getLx();
				if(endingPattern.matcher(lx).matches()){
					initMorphemes(suffixForms, entry, lx);					
				} else if(beginningPattern.matcher(lx).matches()){
					initMorphemes(prefixForms, entry, lx);
				} else if(stemPattern.matcher(lx).matches()){
					initMorphemes(stemForms, entry, lx);
				}else{
					if(entry.getSegmentList().isEmpty()){
						initMorphemes(rootForms, entry, lx);
					}
				}
				
				List<LexiconVariantContent> variantList=entry.getLexiconVariantList();
				for(LexiconVariantContent variant:variantList){
					if(variant.getVariantSegmentList().isEmpty()){
						if(endingPattern.matcher(variant.getVariant()).matches()){
							initMorphemes(suffixForms, entry, variant.getVariant());						
						} else if(beginningPattern.matcher(variant.getVariant()).matches()){
							initMorphemes(prefixForms, entry, variant.getVariant());
						} else if(stemPattern.matcher(variant.getVariant()).matches()){
							initMorphemes(stemForms, entry, variant.getVariant());
						} else{
							initMorphemes(rootForms, entry, variant.getVariant());
						}
					} 
				}
			}
		}
		if(parsingMorphemeList!=null){
			if(!parsingMorphemeList.isEmpty()){
				for(ParsingMorpheme parsingMorpheme:parsingMorphemeList){
					if(endingPattern.matcher(parsingMorpheme.getMorphemeToParse()).matches()){
						suffixUnderlyingForms.add(parsingMorpheme);			
					} else if(beginningPattern.matcher(parsingMorpheme.getMorphemeToParse()).matches()){
						prefixUnderlyingForms.add(parsingMorpheme);
					} else if(stemPattern.matcher(parsingMorpheme.getMorphemeToParse()).matches()){
						stemUnderlyingForms.add(parsingMorpheme);
					}
					else {
						rootUnderlyingForms.add(parsingMorpheme);
					}
				}
			}
		}
	}
	
	private void initParseEntries(){
		Element parseRootElmt=ParseDocumentData.DOCUMENT.getRootElement();
		List<Element> parseElmtList=parseRootElmt.getChildren(ParseDocumentData.PARSE);
		for(Element parseElmt: parseElmtList){
			Element wFormElmt=parseElmt.getChild(LexiconDocumentData.WRITTENFORM);
			ParseRef parseRef=new ParseRef(wFormElmt.getText());	
			List<Element> segmentElmtList=parseElmt.getChild(LexiconDocumentData.MORPHOLOGY).getChildren(LexiconDocumentData.SEGMENT);
			for(Element segmentElmt:segmentElmtList){
				Segment parseSegment=new Segment(segmentElmt.getText(), segmentElmt.getAttributeValue(ParseDocumentData.GE), segmentElmt.getAttributeValue(ParseDocumentData.RX));
				parseRef.addSegment(parseSegment);
			}
			parseForms.add(parseRef);
		}
	}
	
	/**
	 * init parsing morphemes
	 * @param morphemes
	 * @param form
	 * @param lxTableContentList
	 */
	private void initParsingMorphemes(List<ParsingMorpheme> morphemesList, String id, String morphemeToParse, List<String> lxMorphemeList, List<LexiconRef> lxRefList){
		ParsingMorpheme parsingMorpheme=new ParsingMorpheme(morphemeToParse, id, lxMorphemeList, lxRefList);
		morphemesList.add(parsingMorpheme);
	}
	
	/**
	 * init the suffix, prefix and root list to make the interlinearization
	 * @param morphemes
	 * @param entry
	 */
	private void initMorphemes(List<LexiconRef> morphemes, LexiconTableContent entry, String form){
		boolean found=false;
		for(LexiconRef morphemeForm:morphemes){
			if(morphemeForm.getMorphemeForm().equals(form)){
				morphemeForm.addLexicon(entry);
				found=true;
				break;
			}
		}
		if(!found){
			LexiconRef morpheme=new LexiconRef(form);
			morpheme.addLexicon(entry);
			morphemes.add(morpheme);
		}
	}
	
	/**
	 * Look for the lexiconTableContents with the index of the LexicalElement
	 * @param id
	 * @return
	 */
	private List<LexiconTableContent> lookForLexiconTableContent(String index){
		List<LexiconTableContent> lexiconTableContentList=new ArrayList<LexiconTableContent>();
		for(LexiconTableContent lxTableContent:entries){
			if (lxTableContent.getAssociatedLexicalEntryId().equals(index)){
				lexiconTableContentList.add(lxTableContent);
			}
		}
		return lexiconTableContentList;
	}
	
	private boolean addToUFormList(Element lexicalEntryElmt, String id, String lx, List<ParsingMorpheme> uFormList){
		boolean hasUFormList=false;
		if(lexicalEntryElmt.getChild(LexiconDocumentData.MORPHOLOGY)!=null){
			Element morphElmt=lexicalEntryElmt.getChild(LexiconDocumentData.MORPHOLOGY);
			List<Element> segmentElmtList=morphElmt.getChildren(LexiconDocumentData.SEGMENT);
			List<LexiconRef> lexiconRefList=new ArrayList<LexiconRef>();
			List<String> morphemesList=new ArrayList<String>();
			for(Element segment:segmentElmtList){
				String morpheme=segment.getValue();
				String ref=segment.getAttributeValue(LexiconDocumentData.REFERENCE);
				if(ref!=null){
					LexiconRef lexiconRef=new LexiconRef(segment.getText());
					lexiconRef.addLexiconList(lookForLexiconTableContent(ref));
					lexiconRefList.add(lexiconRef);
				} if(!morpheme.isEmpty()){
					morphemesList.add(morpheme);
				}
			}
			initParsingMorphemes(uFormList, id, lx, morphemesList, lexiconRefList);
			hasUFormList=true;
		}
		return hasUFormList;
	}
	
	private boolean addVariantToUFormList(Element altElmt, String id, String a, List<ParsingMorpheme> uFormList){
		boolean hasUFormList=false;
		if(altElmt.getChild(LexiconDocumentData.MORPHOLOGY)!=null){
			Element morphElmt=altElmt.getChild(LexiconDocumentData.MORPHOLOGY);
			List<Element> segmentElmtList=morphElmt.getChildren(LexiconDocumentData.SEGMENT);

			String uFormLexicon="";
			List<LexiconTableContent> lexiconContentList=new ArrayList<LexiconTableContent>();
			List<LexiconRef> lexiconRefList=new ArrayList<LexiconRef>();
			List<String> morphemesList=new ArrayList<String>();
			for(Element segment:segmentElmtList){
				String ref=segment.getAttributeValue(LexiconDocumentData.REFERENCE);
				String morpheme=segment.getValue();
				if(ref!=null){
					LexiconRef lexiconRef=new LexiconRef(segment.getText());
					lexiconRef.addLexiconList(lookForLexiconTableContent(ref));
					lexiconRefList.add(lexiconRef);
					uFormLexicon+=segment.getText();						
				}
				if(!morpheme.isEmpty()){
					morphemesList.add(morpheme);
				}
			}
			Element wFormElmt=altElmt.getChild(LexiconDocumentData.WRITTENFORM);
			if(wFormElmt.getText().equals(a)){
				initParsingMorphemes(uFormList, id, a, morphemesList, lexiconRefList);
			}
			hasUFormList=true;
		}
		return hasUFormList;
	}
	
	/**
	 * Option to make a manual interlinearization
	 * @param viewer
	 */
	public void interlinearize(AbstractViewer viewer, boolean parsePreference){
		this.viewer=viewer;
		parsedEntries.clear();
		if(viewer.getActiveAnnotation()!=null){
			if(parsePreference){
				getParse(viewer, viewer.getActiveAnnotation().getValue());
			} else{
				rootParser(""+viewer.getActiveAnnotation().getValue()+"\u2022", new LexiconMorphemes2());
				//sort parsed entries
				List<LexiconMorphemes2> unguessedEntries=new ArrayList<LexiconMorphemes2>();
				List<LexiconMorphemes2> guessedEntries=new ArrayList<LexiconMorphemes2>();
				for(LexiconMorphemes2 morpheme:parsedEntries){
					if(morpheme.isGuessed()){
						guessedEntries.add(morpheme);
					} else{
						unguessedEntries.add(morpheme);
					}
				}
				parsedEntries.clear();
				Collections.sort(unguessedEntries);
				parsedEntries.addAll(unguessedEntries);
				Collections.sort(guessedEntries);
				parsedEntries.addAll(guessedEntries);
			}
		}
	}
	
	/**
	 * Option to make an automatic interlinearization (when there is no ambiguity, the program fill in every parse automatically)
	 * @param viewer
	 */
	public void autoInterlinearize(AbstractViewer viewer, boolean parsePreference){
		this.viewer=viewer;
		interlinearize(viewer, parsePreference);
		if(!parsePreference){
			List<String> mbStringList=new ArrayList<String>();
			mbStringList.clear();
			List<String> geStringList=new ArrayList<String>();
			geStringList.clear();
			List<String> rxStringList=new ArrayList<String>();
			rxStringList.clear();
			if(parsedEntries.size()!=1){
				List<LexiconMorphemes2> parsedEntriesWithoutGuessed=new ArrayList<LexiconMorphemes2>();
				int parse=0;
				for(int i=0; i<parsedEntries.size(); i++){
					if(parse>=2){
						break;
					}
					LexiconMorphemes2 morphemes=parsedEntries.get(i);
					if(!morphemes.isGuessed()){
						parsedEntriesWithoutGuessed.add(morphemes);
						parse++;
					}
				}
				if(parsedEntriesWithoutGuessed.size()==1){
					parsedEntries=parsedEntriesWithoutGuessed;
				}
			}
			if(parsedEntries.size()==1){
				LexiconMorphemes2 morphemes=parsedEntries.get(0);
				if(morphemes.getPrefixes().size()>=0){
					for(LexiconRef lxRef:morphemes.getPrefixes()){
						if(lxRef.getLexicons().size()!=1){
							return;
						}
						else{
							LexiconTableContent content=lxRef.getLexicons().get(0);
							mbStringList.add(content.getLx());
							geStringList.add(content.getGe());
							rxStringList.add(content.getRx());
						}
					}
				}
				if(morphemes.getRoots().size()>0){
					for(LexiconRef lxRef:morphemes.getRoots()){
						if(lxRef.getLexicons().size()!=1){
							return;
						}
						else{
							LexiconTableContent content=lxRef.getLexicons().get(0);
							mbStringList.add(content.getLx());
							geStringList.add(content.getGe());
							rxStringList.add(content.getRx());
						}
					}
				}
				if(morphemes.getSuffixes().size()>=0){
					for(LexiconRef lxRef:morphemes.getSuffixes()){
						if(lxRef.getLexicons().size()!=1){
							return;
						}
						else{
							LexiconTableContent content=lxRef.getLexicons().get(0);
							mbStringList.add(content.getLx());
							geStringList.add(content.getGe());
							rxStringList.add(content.getRx());
						}
					}
				}
				annotateTier(viewer, morphemes, mbStringList, geStringList, rxStringList, parsePreference);
			} 
		}
	}
	
	private void initAnnotationTiers(AbstractViewer viewer){
		//delete the pre-existing mb and children
		Annotation annMot=viewer.getActiveAnnotation();
		Transcription transcription=((LexiconViewer)viewer).getTranscription();
		boolean mbOK=false;
		boolean geOK=false;
		boolean rxOK=false;
		
		//Undoable deleting of (pre-existing)) mb annotations
		ArrayList<Annotation> annDeleteList = new ArrayList<Annotation>(); 	
		TierImpl tier=(TierImpl) viewer.getActiveAnnotation().getTier();
		Vector childTiers = tier.getChildTiers();
		int numCh=childTiers.size();
		if (numCh==0) {
			return;
		}
		TierImpl ct=null;
		
		for(int j=0; j<numCh; j++) {
			ct=(TierImpl) childTiers.get(j);
			if(ct.getName().equals(((LexiconViewer)viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_PARSE))){		// CC 300811
				mbTier=(TierImpl)childTiers.get(j);
				mbOK=true;
				Vector mbChildren=mbTier.getChildTiers();
				if(mbChildren.size()==2){
					for(int k=0; k<mbChildren.size(); k++){
						TierImpl currentMbChidren=(TierImpl)mbChildren.get(k);
						//if(currentMbChildren.getName().equals((String) ((LexiconViewer) viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_GLOSS))){
						if(currentMbChidren.getName().startsWith((String) ((LexiconViewer) viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_GLOSS))){
							geTier=currentMbChidren;
							geOK=true;

						}
						//else if(currentMbChildren.getName().equals((String) ((LexiconViewer) viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_POS))){
						else if(currentMbChidren.getName().startsWith((String) ((LexiconViewer) viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_POS))){
							rxTier=currentMbChidren;
							rxOK=true;
						}
					}
				}
				if (mbOK && geOK && rxOK){
					Vector<Annotation> annotations=annMot.getChildrenOnTier(mbTier);
					for(Annotation annotation:annotations){
						annDeleteList.add(annotation);			// CC 310811 build a list of MB annotations to be deleted
					}

					comDel = (DeleteSelectedAnnotationsCommand) ELANCommandFactory.createCommand(transcription,
			                ELANCommandFactory.DELETE_MULTIPLE_ANNOS);
					Object receiver = transcription;		
					Object[] args = new Object[1];
					args[0]=annDeleteList;
					comDel.execute(receiver,args);
					if(ct==null||mbTier==null||geTier==null||rxTier==null){
						System.out.println("Problem in the hierarchy !");
					}
					int transMorphBreakChar=(Integer)((LexiconViewer) viewer).getMorphemeBreakPreferences(LexiconViewer.TRANS_BREAK_CHAR);
				}
			}
		}

	}
	
	public void annotateTier(AbstractViewer viewer, List<Segment> segmentList){
		// CC 020911 Revu pour Undo
		initAnnotationTiers(viewer);

		int transMorphBreakChar=(Integer)((LexiconViewer) viewer).getMorphemeBreakPreferences(LexiconViewer.TRANS_BREAK_CHAR);
		Annotation annMot=viewer.getActiveAnnotation();
		TierImpl tierMot=(TierImpl) annMot.getTier();	// mot tier
		Transcription transcription=((LexiconViewer)viewer).getTranscription();

		Annotation currentMbAnn=null;
		Annotation currentGeAnn=null;
		Annotation currentRxAnn=null;
		
		// insert all the parse annotation mb, ge, rx
 		long BT=annMot.getBeginTimeBoundary();
 		long ET=annMot.getEndTimeBoundary();
 
		Vector motChildren;
    	for(int i=0; i<segmentList.size(); i++){
			Object[] annAdd = new Object[3];
			Object[] annAddAfter = new Object[2];
			
			//String[] data=(String[])parsingTable.getValueAt(selectedParsingRow, i);
			String data1=segmentList.get(i).getLexeme();
			String data2=segmentList.get(i).getGloss();
			String data3=segmentList.get(i).getRx();
			if(data1!=""){					// test  revoir pas ncessaire ?
				String mb=""+data1;
				String ge=""+data2;
				String rx=""+data3;
				
				// transmission of morphemeBreakCharacter to ge and rx
				if(transMorphBreakChar==1){	
					if(InterlinearizationProcess.endingPattern.matcher(mb).matches()){
						char endChar=mb.charAt(0);
						ge=endChar+ge;
						rx=endChar+rx;
					} else if(InterlinearizationProcess.beginningPattern.matcher(mb).matches()){
						char beginningChar=mb.charAt(mb.length()-1);
						ge=ge+beginningChar;
						rx=rx+beginningChar;
					}
				}
				
				if(i>0){	// Other segments				
					motChildren=annMot.getChildrenOnTier(mbTier);
					Annotation lastAnnMb = (AbstractAnnotation) motChildren.lastElement();
					annAddAfter[0]=lastAnnMb;
					annAddAfter[1] = mb;
					//create a new annotation after the preceding
					comNewAfterMb=(AnnotationValAfterCommand) ELANCommandFactory.createCommand(transcription,
			                ELANCommandFactory.NEW_ANNOTATION_VAL_AFTER);
					comNewAfterMb.execute(mbTier,annAddAfter);
					//the beginTime of the new created annotation
					motChildren=annMot.getChildrenOnTier(mbTier);
					lastAnnMb = (AbstractAnnotation) motChildren.lastElement();
					annAdd[0]=lastAnnMb.getBeginTimeBoundary();
					annAdd[1]=annAdd[0];

				}
				else{		// First segment
					annAdd[0]=BT;
					annAdd[1]=BT;
					annAdd[2]=mb;
					//create a new annotation
					comNewMb= (NewAnnotationWithValueCommand) ELANCommandFactory.createCommand(transcription,
			                ELANCommandFactory.NEW_ANNOTATION_VAL);
					comNewMb.execute(mbTier,annAdd);
				}
				// GE and RX
				annAdd[2]=ge;
				comNewGe= (NewAnnotationWithValueCommand) ELANCommandFactory.createCommand(transcription,
		                ELANCommandFactory.NEW_ANNOTATION_VAL);
				comNewGe.execute(geTier,annAdd);
				annAdd[2]=rx;
				comNewRx= (NewAnnotationWithValueCommand) ELANCommandFactory.createCommand(transcription,
		                ELANCommandFactory.NEW_ANNOTATION_VAL);	
				comNewRx.execute(rxTier,annAdd);
			}
		}
			viewer.setActiveAnnotation(tierMot.getAnnotationAtTime(ET));
			autoInterlinearize(viewer, true);
			//removeSegmentationInformation();
		
	}
	
	private void annotateTier(AbstractViewer viewer, LexiconMorphemes2 morphemes, List<String> mbList, List<String> geList, List<String> rxList, boolean parsePreference){
		//delete the pre-existing mb and children
		Transcription transcription=((LexiconViewer)viewer).getTranscription();
		initAnnotationTiers(viewer); 
		Vector motChildren;
		int transMorphBreakChar=(Integer)((LexiconViewer) viewer).getMorphemeBreakPreferences(LexiconViewer.TRANS_BREAK_CHAR);

		Annotation annMot=viewer.getActiveAnnotation();	// CC 031011
		TierImpl tierMot=(TierImpl) annMot.getTier();	// mot tier	
 		long BT=annMot.getBeginTimeBoundary();
 		long ET=annMot.getEndTimeBoundary();
		
		Annotation currentMbAnn=null;
		Annotation currentGeAnn=null;
		Annotation currentRxAnn=null;
		if(mbList.size()==geList.size()&&geList.size()==rxList.size()){
			int size=mbList.size();
			for(int i=0; i<size; i++){
				Object[] annAdd = new Object[3];
				Object[] annAddAfter = new Object[2];
				if(transMorphBreakChar==1){
					if(endingPattern.matcher(mbList.get(i)).matches()){
						char endChar=mbList.get(i).charAt(0);
						geList.set(i, endChar+geList.get(i));
						rxList.set(i, endChar+rxList.get(i));
					} else if(beginningPattern.matcher(mbList.get(i)).matches()){
						char beginningChar=mbList.get(i).charAt(mbList.get(i).length()-1);
						geList.set(i, geList.get(i)+beginningChar);
						rxList.set(i, rxList.get(i)+beginningChar);
					}
				}
				/*			
				if(i>0){
					Annotation mbAnn=mbTier.createAnnotationAfter(currentMbAnn);
					mbAnn.setValue(mbList.get(i));
					currentMbAnn=mbAnn;
					Annotation geAnn=geTier.createAnnotation(currentGeAnn.getEndTimeBoundary(), currentGeAnn.getEndTimeBoundary());
					geAnn.setValue(geList.get(i));
					currentGeAnn=geAnn;
					Annotation rxAnn=rxTier.createAnnotation(currentRxAnn.getEndTimeBoundary(),currentRxAnn.getEndTimeBoundary());
					rxAnn.setValue(rxList.get(i));
					currentRxAnn=rxAnn;
				}
				else{
					currentMbAnn=mbTier.createAnnotation(viewer.getActiveAnnotation().getBeginTimeBoundary(), viewer.getActiveAnnotation().getBeginTimeBoundary());
					currentMbAnn.setValue(mbList.get(i));
					currentGeAnn=geTier.createAnnotation(viewer.getActiveAnnotation().getBeginTimeBoundary(), viewer.getActiveAnnotation().getBeginTimeBoundary());
					currentGeAnn.setValue(geList.get(i));
					currentRxAnn=rxTier.createAnnotation(viewer.getActiveAnnotation().getBeginTimeBoundary(), viewer.getActiveAnnotation().getBeginTimeBoundary());
					currentRxAnn.setValue(rxList.get(i));
				} 
				*/
				// CC 031011
				String mb=mbList.get(i);
				String ge=geList.get(i);
				String rx=rxList.get(i);
				if(i>0){	// Other segments				
					motChildren=annMot.getChildrenOnTier(mbTier);
					Annotation lastAnnMb = (AbstractAnnotation) motChildren.lastElement();
					annAddAfter[0]=lastAnnMb;
					annAddAfter[1] = mb;
					comNewAfterMb=(AnnotationValAfterCommand) ELANCommandFactory.createCommand(transcription,
			                ELANCommandFactory.NEW_ANNOTATION_VAL_AFTER);
					comNewAfterMb.execute(mbTier,annAddAfter);
					//the beginTime of the new created annotation
					motChildren=annMot.getChildrenOnTier(mbTier);
					lastAnnMb = (AbstractAnnotation) motChildren.lastElement();
					annAdd[0]=lastAnnMb.getBeginTimeBoundary();
					annAdd[1]=annAdd[0];

				}
				else{		// First segment
					annAdd[0]=BT;
					annAdd[1]=BT;
					annAdd[2]=mb;
					comNewMb= (NewAnnotationWithValueCommand) ELANCommandFactory.createCommand(transcription,
			                ELANCommandFactory.NEW_ANNOTATION_VAL);
					comNewMb.execute(mbTier,annAdd);
				}
				annAdd[2]=ge;
				comNewGe= (NewAnnotationWithValueCommand) ELANCommandFactory.createCommand(transcription,
		                ELANCommandFactory.NEW_ANNOTATION_VAL);
				comNewGe.execute(geTier,annAdd);
				annAdd[2]=rx;
				comNewRx= (NewAnnotationWithValueCommand) ELANCommandFactory.createCommand(transcription,
		                ELANCommandFactory.NEW_ANNOTATION_VAL);	
				comNewRx.execute(rxTier,annAdd);
			}
			//viewer.setActiveAnnotation(((TierImpl)viewer.getActiveAnnotation().getTier()).getAnnotationAtTime(viewer.getActiveAnnotation().getEndTimeBoundary()));
			viewer.setActiveAnnotation(tierMot.getAnnotationAtTime(ET));
			autoInterlinearize(viewer, parsePreference);
		}
	}
	
	private boolean controlCurrentHierarchy(AbstractViewer viewer){
		initAnnotationTiers(viewer);
		if(ct==null||mbTier==null||geTier==null||rxTier==null){
			System.out.println("Problem in the hierarchy !");
			return false;
		}
		return true;
		
	}
	
	private void getParse(AbstractViewer viewer, String annotation){
		List<ParseRef> parseRefList=new ArrayList<ParseRef>();
		parseRefList.clear();
		for(ParseRef parseRef:parseForms){
			if(parseRef.getLexeme().equals(annotation)){
				parseRefList.add(parseRef);
			}
		}
		if (parseRefList.size()>1){
			ParseDataChoiceDialog dialog=new ParseDataChoiceDialog(viewer, parseRefList, this);
			dialog.setVisible(true);
			dialog.setFocusableWindowState(true);
			dialog.setAlwaysOnTop(true);
		} else{
			if(!parseRefList.isEmpty()){
				annotateTier(viewer, parseRefList.get(0).getSegment());
			}
		}
	}
	
	/**
	 * lookup an annotation
	 * @param annotation
	 * @param entries
	 */
	private void rootParser(String annotation, LexiconMorphemes2 morphemes){
		boolean find=false;
		if (inBrackets.matcher(annotation).matches()){
			int begin=annotation.indexOf("");
			int end=annotation.indexOf("\u2022");
			String sequenceToMatch=(String) annotation.subSequence(begin+1, end);
			if(!sequenceToMatch.isEmpty()){
				List<LexiconRef> rootToAdd=new ArrayList<LexiconRef>();
				rootToAdd.clear();
				for(ParsingMorpheme parsingMorpheme:rootUnderlyingForms){
					String ulx=parsingMorpheme.getMorphemeToParse();
					if(ulx.equals(sequenceToMatch)){
						LexiconMorphemes2 currentMorphemes=new LexiconMorphemes2(morphemes);
						List<LexiconRef> roots=new ArrayList<LexiconRef>();
						if(!parsingMorpheme.getLexiconRefList().isEmpty()){
							for(LexiconRef lxRef:parsingMorpheme.getLexiconRefList()){
								roots.add(lxRef);
							}
						}
						else{
							List<LexiconRef> lexiconRefList=new ArrayList<LexiconRef>();
							for(String morpheme:parsingMorpheme.getLexiconMorphemesList()){
								lexiconRefList.add(findMorpheme(morpheme));
							}
							ChooseReferenceDialog dialog=new ChooseReferenceDialog(viewer, parsingMorpheme, lexiconRefList);
							dialog.setVisible(true);
						}
						currentMorphemes.setRoots(roots);
						addMorphemeToList(currentMorphemes.getMorphemes(), currentMorphemes);
						find=true;
					}
				}
				//find root Form or stem Form
				for(LexiconRef rootForm:rootForms){
					String lx=rootForm.getMorphemeForm();
					if(lx.equals(sequenceToMatch)){
						rootToAdd.add(rootForm);
					}
				}
				for(LexiconRef stemForm:stemForms){
					String stem=stemForm.getMorphemeForm().replaceAll("["+stemChar+"]", "");
					if(stem.equals(sequenceToMatch)){
						rootToAdd.add(stemForm);
					}
				}
				if(!rootToAdd.isEmpty()){
					LexiconMorphemes2 currentMorphemes=new LexiconMorphemes2(morphemes);
					for(int i=0; i<rootToAdd.size(); i++){
						LexiconRef root=rootToAdd.get(i);
						List<LexiconRef> roots=new ArrayList<LexiconRef>();
						roots.add(root);
						currentMorphemes.setRoots(roots);
					}
					addMorphemeToList(currentMorphemes.getMorphemes(), currentMorphemes);
				}
				else{
					if(!find){
						suffixParser(annotation, morphemes, false, true);
						prefixParser(annotation, morphemes, true, false);
					}
				}
			} 
			else{
				addMorphemeToList(annotation, morphemes);
			}
		}
		else{
			addMorphemeToList(annotation, morphemes);
		}
	}
	
	/**
	 * Find the different morphemes if the references of the underlying form missed
	 * @param morpheme
	 * @return
	 */
	private LexiconRef findMorpheme(String morpheme){
		String sequenceToMatch=morpheme;
		if(beginningPattern.matcher(morpheme).matches()){
			for(LexiconRef lexRef:prefixForms){
				if(lexRef.getMorphemeForm().equals(sequenceToMatch)){
					return lexRef;
				}
			}
		} 
		else if(endingPattern.matcher(morpheme).matches()){
			for(LexiconRef lexRef:suffixForms){
				if(lexRef.getMorphemeForm().equals(sequenceToMatch)){
					return lexRef;
				}
			}
		} 
		else if(stemPattern.matcher(morpheme).matches()){
			for(LexiconRef lexRef:stemForms){
				if(lexRef.getMorphemeForm().equals(sequenceToMatch)){
					return lexRef;
				}
			}			
		}
		else{
			for(LexiconRef lexRef:rootForms){
				if(lexRef.getMorphemeForm().equals(sequenceToMatch)){
					return lexRef;
				}
			}
		}
		return null;
	}

	/**
	 * Segment an annotation with a suffix
	 * @param annotation
	 * @param entries
	 */
	private void suffixParser(String annotation, LexiconMorphemes2 morphemes, boolean foundSuffix, boolean foundPrefix){
		if (inBrackets.matcher(annotation).matches()){
			int begin=annotation.indexOf("");
			int end=annotation.indexOf("\u2022");
			String sequenceToMatch=(String) annotation.subSequence(begin+1, end);
			if(!sequenceToMatch.isEmpty()){
				LinkedList<LexiconRef> suffixesToAdd=new LinkedList<LexiconRef>();
				suffixesToAdd.clear();
				List<String> segmentation=new ArrayList<String>();
				segmentation.clear();
				for(ParsingMorpheme parsingMorpheme:suffixUnderlyingForms){
					String ulx=parsingMorpheme.getMorphemeToParse().substring(1);
					if(sequenceToMatch.endsWith(ulx)){
						if(sequenceToMatch.split(ulx+"$").length>0){
							LexiconMorphemes2 currentMorphemes=new LexiconMorphemes2(morphemes);
							String sequence=""+sequenceToMatch.split(ulx+"$")[0]+"\u2022 "+ulx;
							String segment=annotation.replaceAll(""+sequenceToMatch+"\u2022", sequence);
							segmentation.add(segment);
							for(LexiconRef lxRef:parsingMorpheme.getLexiconRefList()){
								suffixesToAdd.addFirst(lxRef);		// UTD 27/09
							}
							for(LexiconRef suffixRef:suffixesToAdd){
								currentMorphemes.addSuffix(suffixRef);
							}
								rootParser(segment, currentMorphemes);
						}
					}
				}
				for(LexiconRef suffixForm:suffixForms){
					String lx=suffixForm.getMorphemeForm().substring(1);
					if(sequenceToMatch.endsWith(lx)){
						if(sequenceToMatch.split(lx+"$").length>0){
							foundSuffix=true;
							suffixesToAdd.add(suffixForm);
							String sequence=""+sequenceToMatch.split(lx+"$")[0]+"\u2022 "+suffixForm.getMorphemeForm();
							String segment=annotation.replaceAll(""+sequenceToMatch+"\u2022", sequence);
							segmentation.add(segment);
						}
					}
				}
				if (foundSuffix){
					if(segmentation.size()==suffixesToAdd.size()){
						for(int i=0; i<segmentation.size(); i++){
							LexiconMorphemes2 currentMorphemes=new LexiconMorphemes2(morphemes);
							String segment=segmentation.get(i);
							LexiconRef suffix=suffixesToAdd.get(i);
							currentMorphemes.addSuffix(suffix);
							rootParser(segment, currentMorphemes);
						}
					}
				}
				else{
					if(foundPrefix){
						prefixParser(annotation, morphemes, foundSuffix, false);
					}
					//add a guessed morpheme
					else{
						LexiconMorphemes2 currentMorphemes=new LexiconMorphemes2(morphemes);
						currentMorphemes.setGuessed(true);
						String sequence=annotation.replaceAll(""+sequenceToMatch+"\u2022", "*"+sequenceToMatch);

						LexiconRef guessed= new LexiconRef("*"+sequenceToMatch);
						List<LexiconRef> roots=new ArrayList<LexiconRef>();
						roots.add(guessed);
						currentMorphemes.setRoots(roots);

						addMorphemeToList(sequence, currentMorphemes);
					}
				}
			}
			else{
				addMorphemeToList(annotation, morphemes);
			}
		}
		else{
			addMorphemeToList(annotation, morphemes);
		}
	}

	/**
	 * Segment an annotation with a prefix
	 * @param annotation
	 * @param entries
	 */
	private void prefixParser(String annotation, LexiconMorphemes2 morphemes, boolean foundSuffix, boolean foundPrefix){
		if (inBrackets.matcher(annotation).matches()){
			int begin=annotation.indexOf("");
			int end=annotation.indexOf("\u2022");
			String sequenceToMatch=(String) annotation.subSequence(begin+1, end);
			if(!sequenceToMatch.isEmpty()){
				List<String> segmentation=new ArrayList<String>();
				List<LexiconRef> prefixesToAdd=new ArrayList<LexiconRef>();
				prefixesToAdd.clear();
				for(ParsingMorpheme parsingMorpheme:prefixUnderlyingForms){
					String ulx=parsingMorpheme.getMorphemeToParse().substring(0, parsingMorpheme.getMorphemeToParse().length()-1);
					if(sequenceToMatch.startsWith(ulx)){
						if(sequenceToMatch.split("^"+ulx).length>0){
							LexiconMorphemes2 currentMorphemes=new LexiconMorphemes2(morphemes);
							String sequence=ulx+" "+sequenceToMatch.split("^"+ulx)[1]+"\u2022";
							String segment=annotation.replaceAll(""+sequenceToMatch+"\u2022", sequence);
							segmentation.add(segment);
							for(LexiconRef lxRef:parsingMorpheme.getLexiconRefList()){
								currentMorphemes.addPrefix(lxRef);
							}
							rootParser(segment, currentMorphemes);
						}
					}
				}
				for(LexiconRef prefixForm:prefixForms){
					String lx=prefixForm.getMorphemeForm().substring(0, prefixForm.getMorphemeForm().length()-1);
					if(sequenceToMatch.startsWith(lx)){
						if(sequenceToMatch.split("^"+lx).length>0){
							foundPrefix=true;
							prefixesToAdd.add(prefixForm);
							String sequence=prefixForm.getMorphemeForm()+" "+sequenceToMatch.split("^"+lx)[1]+"\u2022";
							String segment=annotation.replaceAll(""+sequenceToMatch+"\u2022", sequence);
							segmentation.add(segment);
						}
					}
				}
				if (foundPrefix){
					if(segmentation.size()==prefixesToAdd.size()){
						for(int i=0; i<segmentation.size(); i++){
							LexiconMorphemes2 currentMorphemes=new LexiconMorphemes2(morphemes);
							String segment=segmentation.get(i);
							LexiconRef prefix=prefixesToAdd.get(i);
							currentMorphemes.addPrefix(prefix);
							rootParser(segment, currentMorphemes);
						}
					}
				}
				else{
					if(foundSuffix){
						suffixParser(annotation, morphemes, false, foundPrefix);
					}
					//add a guessed morpheme
					else{
						LexiconMorphemes2 currentMorphemes=new LexiconMorphemes2(morphemes);
						currentMorphemes.setGuessed(true);
						String sequence=annotation.replaceAll(""+sequenceToMatch+"\u2022", "*"+sequenceToMatch);
						LexiconRef guessed= new LexiconRef("*"+sequenceToMatch);
						List<LexiconRef> roots=new ArrayList<LexiconRef>();
						roots.add(guessed);
						currentMorphemes.setRoots(roots);

						addMorphemeToList(sequence, currentMorphemes);
					}
				}
			}
			else{
				addMorphemeToList(annotation, morphemes);
			}
		}
		else{
			addMorphemeToList(annotation, morphemes);
		}
	}

	/**
	 * Add a morphemeParse if and only if this parse is not already in the list
	 * @param annotation
	 * @param morphemes
	 */
	private void addMorphemeToList(String annotation, LexiconMorphemes2 morphemes){
		boolean newParse=true;
		if(!parsedEntries.isEmpty()){
			for(LexiconMorphemes2 entry:parsedEntries){
				if(entry.equalsMorphemes(morphemes)){
					newParse=false;
					break;
				}
			}
		}
		if(newParse){
			parsedEntries.add(morphemes);
		}
	}
	
	/**
	 * Take the morpheme break characters define in the preference file or take the default value
	 */
	private void initPatterns(){
		String suffixChar=(String)((LexiconViewer) viewer).getMorphemeBreakPreferences(LexiconViewer.SUFFIX_CHAR);
		if (suffixChar==null){
			suffixChar=LexiconViewer.DEFAULT_SUFFIX_VALUE;
		}
		Matcher suffixMatcher=specialCharPattern.matcher(suffixChar); 
		if (suffixMatcher.find()){
			for (int i=0; i<=suffixMatcher.groupCount(); i++){
				String specialChar=suffixMatcher.group(i);
				suffixChar=suffixChar.replace(specialChar, "\\"+specialChar);
			}
		}
		endingPattern=Pattern.compile("^["+suffixChar+"].*$");
		beginningPattern=Pattern.compile("^.*["+suffixChar+"]$");
		
		stemChar=(String)((LexiconViewer) viewer).getMorphemeBreakPreferences(LexiconViewer.STEM_CHAR);
		if (stemChar==null){
			stemChar=LexiconViewer.DEFAULT_STEM_VALUE;
		}
		Matcher stemMatcher=specialCharPattern.matcher(stemChar); 
		if (stemMatcher.find()){
			for (int i=0; i<=stemMatcher.groupCount(); i++){
				String specialChar=stemMatcher.group(i);
				stemChar=stemChar.replace(specialChar, "\\"+specialChar);
			}
		}
		stemPattern=Pattern.compile("(^.*["+stemChar+"]$|^["+stemChar+"].*$)");
	}
	
	public List<LexiconMorphemes2> getParsedEntries(){
		return parsedEntries;
	}
	
	public List<LexiconRef> getRootForms(){
		return rootForms;
	}
	
	public List<LexiconRef> getSuffixForms(){
		return suffixForms;
	}
	
	public List<LexiconRef> getPrefixForms(){
		return prefixForms;
	}
}
