package mpi.eudico.client.annotator.lexica;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.Set;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.gui.ClosableDialog;

/**
 * Configure fields to create a new Elan dictionary
 * @author Coralie
 *
 */
public class LXConfigurationDialog extends ClosableDialog implements ActionListener{
    public static final int MEDIA_TEMPLATE = 1;
    
    private JList fieldList;
    
    private JList selectedField;
    private DefaultListModel model;
    private JButton okButton;
    private JButton cancelButton;
    private JButton copyButton;
    private JButton removeButton;
    private JButton upButton;
    private JButton downButton;
	
	private File file;
	private Set<String> fields;
	private Pattern fieldsPattern;
	
	//fields to configure
	private String[] fieldsToSelect={ElanLocale.getString("LexiconViewer.ConfigurationDlg.Label.Lexeme"), ElanLocale.getString("LexiconViewer.ConfigurationDlg.Label.Variant"), ElanLocale.getString("LexiconViewer.ConfigurationDlg.Label.UForm"), ElanLocale.getString("LexiconViewer.ConfigurationDlg.Label.Glose"), ElanLocale.getString("LexiconViewer.ConfigurationDlg.Label.Date"), ElanLocale.getString("LexiconViewer.ConfigurationDlg.Label.Pos"), ElanLocale.getString("LexiconViewer.ConfigurationDlg.Label.TierX")};
	
	private String lexemeConf;
	private String variantConf;
	private String uFormConf;
	private String gloseConf;
	private String dateConf;
	private String partOfSpeechConf;
	private String tierXConf;
	
	private LexiconFieldConfiguration lexiconConfig=null;
	
	private boolean configOK=false;
	
	public LXConfigurationDialog(File f){
		file=f;
		extractFields(file);
        setModal(true);
        setTitle(ElanLocale.getString("MetadataViewer.Configure"));
        initComponents();
	}
	
	 private void initComponents() {
	        okButton = new JButton();
	        cancelButton = new JButton();
	        removeButton = new JButton();
	        upButton = new JButton();
	        downButton = new JButton();
	        copyButton = new JButton();

	        ImageIcon REMOVE_ICON = new ImageIcon(this.getClass().getResource("/mpi/eudico/client/annotator/resources/Remove.gif"));
	        ImageIcon UP_ICON = new ImageIcon(this.getClass().getResource("/mpi/eudico/client/annotator/resources/Up.gif"));
	        ImageIcon DOWN_ICON = new ImageIcon(this.getClass().getResource("/mpi/eudico/client/annotator/resources/Down.gif"));
	        removeButton.setIcon(REMOVE_ICON);
	        upButton.setIcon(UP_ICON);
	        downButton.setIcon(DOWN_ICON);

	        this.setLayout(new GridBagLayout());

	        GridBagConstraints gbc = new GridBagConstraints();
	        Insets insets = new Insets(2, 2, 2, 2);

	        //leftPanel
	        JPanel leftPanel=new JPanel(new GridBagLayout());
	        JPanel fieldPanel=new JPanel(new GridBagLayout());
	        Dimension dimf = new Dimension(110, 40);
	        fieldPanel.setPreferredSize(dimf);
	        fieldPanel.setMinimumSize(dimf);
	        gbc = new GridBagConstraints();
	        gbc.gridx = 0;
	        gbc.gridy = 0;
	        gbc.anchor = GridBagConstraints.CENTER;
	        gbc.insets = insets;
	        gbc.fill = GridBagConstraints.HORIZONTAL;
	        gbc.weighty = 1.0;
	        fieldPanel.add(new JLabel(ElanLocale.getString("LexiconViewer.ConfigurationDlg.Label.ShoeboxField")));

	        gbc = new GridBagConstraints();
	        gbc.gridx = 0;
	        gbc.gridy = 0;
	        gbc.insets = insets;
	        gbc.anchor = GridBagConstraints.NORTH;
	        gbc.fill = GridBagConstraints.HORIZONTAL;
	        gbc.weightx = 1.0;
	        leftPanel.add(fieldPanel, gbc);
	        
	        fieldList = new JList(fields.toArray());
	        fieldList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	        
	        JScrollPane jsp = new JScrollPane(fieldList);
	        jsp.setPreferredSize(new Dimension(jsp.getPreferredSize().getSize().width -30, jsp.getPreferredSize().getSize().height));

	        gbc = new GridBagConstraints();
	        gbc.gridx = 0;
	        gbc.gridy = 1;
	        gbc.insets = insets;
	        gbc.anchor = GridBagConstraints.NORTH;
	        gbc.fill = GridBagConstraints.BOTH;
	        gbc.weightx = 1.0;
	        gbc.weighty = 1.0;
	        leftPanel.add(jsp, gbc);
	        
	        gbc = new GridBagConstraints();
	        gbc.gridx = 0;
	        gbc.gridy = 0;
	        gbc.anchor = GridBagConstraints.NORTH;
	        gbc.insets = insets;
	        gbc.fill = GridBagConstraints.BOTH;
	        gbc.weightx = 1.0;
	        gbc.weighty = 1.0;
	        this.add(leftPanel, gbc);
	        
	        JPanel midPanel = new JPanel(new GridBagLayout());
	        copyButton.setText(" >> ");
	        gbc = new GridBagConstraints();
	        gbc.gridx = 0;
	        gbc.gridy = 0;
	        gbc.anchor = GridBagConstraints.CENTER;
	        gbc.insets = new Insets(4, 6, 4, 6);
	        midPanel.add(copyButton, gbc);

	        gbc = new GridBagConstraints();
	        gbc.gridx = 1;
	        gbc.gridy = 0;
	        gbc.anchor = GridBagConstraints.NORTH;

	        gbc.fill = GridBagConstraints.VERTICAL;
	        gbc.weighty = 1.0;
	        this.add(midPanel, gbc);
	        
	        //right panel
	        JPanel rightPanel = new JPanel(new GridBagLayout());
	        JPanel infoPanel = new JPanel(new GridBagLayout());
	        Dimension dim = new Dimension(70, 40);
	        infoPanel.setPreferredSize(dim);
	        infoPanel.setMinimumSize(dim);
	        gbc = new GridBagConstraints();
	        gbc.gridx = 0;
	        gbc.gridy = 0;
	        gbc.anchor = GridBagConstraints.CENTER;
	        gbc.insets = insets;
	        gbc.fill = GridBagConstraints.HORIZONTAL;
	        gbc.weighty = 1.0;
	        infoPanel.add(new JLabel(ElanLocale.getString("LexiconViewer.ConfigurationDlg.Label.ELANFields")));

	        gbc = new GridBagConstraints();
	        gbc.gridx = 0;
	        gbc.gridy = 0;
	        gbc.insets = insets;
	        gbc.anchor = GridBagConstraints.NORTH;
	        gbc.fill = GridBagConstraints.HORIZONTAL;
	        gbc.weightx = 1.0;
	        rightPanel.add(infoPanel, gbc);

	        model=new DefaultListModel();
	        //init model
	       
	        for(int i=0; i<fieldsToSelect.length; i++){
		        model.add(i, fieldsToSelect[i]);
	        }
	        selectedField = new JList(model);
	        selectedField.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

	        jsp = new JScrollPane(selectedField);
	        jsp.setPreferredSize(new Dimension(jsp.getPreferredSize().getSize().width -
	                30, jsp.getPreferredSize().getSize().height));

	        gbc = new GridBagConstraints();
	        gbc.gridx = 0;
	        gbc.gridy = 1;
	        gbc.insets = insets;
	        gbc.anchor = GridBagConstraints.NORTH;
	        gbc.fill = GridBagConstraints.BOTH;
	        gbc.weightx = 1.0;
	        gbc.weighty = 1.0;
	        rightPanel.add(jsp, gbc);

	        JPanel controlPanel = new JPanel(new GridLayout(1, 3, 6, 6));
	        controlPanel.add(removeButton);
	        controlPanel.add(upButton);
	        controlPanel.add(downButton);

	        gbc = new GridBagConstraints();
	        gbc.gridx = 0;
	        gbc.gridy = 2;
	        gbc.insets = insets;
	        gbc.anchor = GridBagConstraints.NORTH;
	        gbc.fill = GridBagConstraints.HORIZONTAL;
	        gbc.weightx = 1.0;
	        rightPanel.add(controlPanel, gbc);

	        gbc = new GridBagConstraints();
	        gbc.gridx = 2;
	        gbc.gridy = 0;
	        gbc.anchor = GridBagConstraints.NORTH;
	        gbc.insets = insets;
	        gbc.fill = GridBagConstraints.BOTH;
	        gbc.weightx = 1.0;
	        gbc.weighty = 1.0;
	        this.add(rightPanel, gbc);

	        JPanel buttonPanel = new JPanel(new GridLayout(1, 2, 6, 6));
	        okButton.setText(ElanLocale.getString("Button.OK"));
	        cancelButton.setText(ElanLocale.getString("Button.Cancel"));
	        buttonPanel.add(okButton);
	        buttonPanel.add(cancelButton);

	        JPanel butBorderPanel = new JPanel(new GridBagLayout());
	        gbc = new GridBagConstraints();
	        gbc.gridx = 0;
	        gbc.gridy = 0;
	        gbc.anchor = GridBagConstraints.WEST;
	        gbc.fill = GridBagConstraints.HORIZONTAL;
	        gbc.weightx = 1.0;
	        butBorderPanel.add(new JPanel(), gbc);
	        gbc = new GridBagConstraints();
	        gbc.gridx = 1;
	        gbc.gridy = 0;
	        gbc.anchor = GridBagConstraints.EAST;
	        gbc.insets = insets;
	        butBorderPanel.add(buttonPanel, gbc);

	        butBorderPanel.setBorder(new CompoundBorder(
	                new LineBorder(Color.GRAY, 1), new EmptyBorder(6, 6, 6, 6)));
	        gbc = new GridBagConstraints();
	        gbc.gridx = 0;
	        gbc.gridy = 1;
	        gbc.gridwidth = 3;
	        gbc.anchor = GridBagConstraints.CENTER;
	        gbc.insets = insets;
	        gbc.fill = GridBagConstraints.HORIZONTAL;
	        gbc.weightx = 1.0;
	        this.add(butBorderPanel, gbc);

	        copyButton.addActionListener(this);
	        removeButton.addActionListener(this);
	        upButton.addActionListener(this);
	        downButton.addActionListener(this);
	        okButton.addActionListener(this);
	        cancelButton.addActionListener(this);
	        
	        pack();
	        setLocationRelativeTo(getParent());
	    }
	
	public Set<String> extractFields(File f){
		fields=new TreeSet<String>();
		if(f.canRead()){
			BufferedReader bufferedReader;
			InputStreamReader reader;
			try {
				reader = new InputStreamReader(new FileInputStream(f),"UTF-8");
				bufferedReader=new BufferedReader(reader);
				String line=null;
				fieldsPattern=Pattern.compile("^\\\\[a-z0-9]+");
				
				while((line=bufferedReader.readLine())!=null){
					Matcher m = fieldsPattern.matcher(line);
					if(m.find()){
						String field=line.substring(m.start(), m.end());
						//System.out.println(line.substring(m.start(), m.end()));
						fields.add(field);
					}
				}
			} catch (UnsupportedEncodingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return fields;
	}
	
	  /**
     * Removes the selected file from the selected files list.
     */
    private void removeField() {
        int index = selectedField.getSelectedIndex();
        if (index != -1) {
            model.setElementAt(((String)model.getElementAt(index)).split(" -> ")[0], index);
        }
    }
	
	 /**
     * Moves the selected field up in the list.
     */
    private void moveUp() {
        int index = selectedField.getSelectedIndex();

        if (index > 0) {
        	String element=((String)model.getElementAt(index));
        	if(element.contains(" -> ")){
        		String moveUpElement=element.split(" -> ")[1];
        		element=element.split(" -> ")[0];
        		model.set(index, element);
        		String previousElement=(String)model.getElementAt(index-1);
        		if(!previousElement.contains(" -> ")){
        			model.set(index-1, previousElement+" -> "+moveUpElement);
        		}
        		else{
        			model.set(index-1, previousElement.split(" -> ")[0]+" -> "+moveUpElement);
        		}
        	}
           selectedField.setSelectedIndex(index - 1);
           selectedField.ensureIndexIsVisible(index - 1);
        }
    }
	
    /**
     * Moves the selected file down in the list.
     */
    private void moveDown() {
        int index = selectedField.getSelectedIndex();

        if (!(index==-1)&&!(index==(model.getSize()-1))) {
        	String element=((String)model.getElementAt(index));
        	if(element.contains(" -> ")){
        		String moveDownElement=element.split(" -> ")[1];
        		element=element.split(" -> ")[0];
        		model.set(index, element);
        		String nextElement=(String)model.getElementAt(index+1);
        		if(!nextElement.contains(" -> ")){
        			model.set(index+1, nextElement+" -> "+moveDownElement);
        		}
        		else{
        			model.set(index+1, nextElement.split(" -> ")[0]+" -> "+moveDownElement);
        		}
        	}
           selectedField.setSelectedIndex(index+1);
           selectedField.ensureIndexIsVisible(index+1);
        }
    	
    }
    
    private void configOK(){
    	configurateFieldValue();
    	configOK=true;
    }
    
	/**
	 * Copy the fields found in the current file in the selected field list
	 */
	private void copyField() {
		Object[] fields=fieldList.getSelectedValues();
		int curIndex;
        for (int i = 0; i < fields.length; i++) {
            if (!model.contains(fields[i])) {
                curIndex = selectedField.getSelectedIndex();
                if (!((String)model.getElementAt(curIndex)).contains(" -> ")) {
                	model.setElementAt(model.getElementAt(curIndex)+" -> "+fields[i], curIndex);
                }
                else{
                	model.setElementAt(((String)model.getElementAt(curIndex)).split(" -> ")[0]+" -> "+fields[i], curIndex);
                }

            }
        }
	}
	
	private void configurateFieldValue(){
		for(int i=0; i<model.size(); i++){
			if(((String)model.getElementAt(i)).contains(" -> ")){
				String[] splittedElement=((String)model.getElementAt(i)).split("-> \\\\");
				switch (i) {
				case 0:
					lexemeConf=splittedElement[1];
					break;
				case 1:
					variantConf=splittedElement[1];
					break;
				case 2:
					uFormConf=splittedElement[1];
					break;
				case 3:
					gloseConf=splittedElement[1];
					break;
				case 4:
					dateConf=splittedElement[1];
					break;
				case 5:
					partOfSpeechConf=splittedElement[1];
					break;
				case 6:
					tierXConf=splittedElement[1];
					break;
				}
			}
		}
		lexiconConfig=new LexiconFieldConfiguration(lexemeConf, variantConf, uFormConf, gloseConf, tierXConf, dateConf, partOfSpeechConf);
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		Object src = e.getSource();
		
		if (src==cancelButton) {
			setVisible(false);
			dispose();
		}else if(src==copyButton){
			copyField();
		}else if (src == upButton) {
            moveUp();
        } else if (src == downButton) {
            moveDown();
        }else if (src == removeButton) {
            removeField();
        } else if(src==okButton){
        	configOK();
        	setVisible(false);
			dispose();
			//new LXComponentViewer(file.getAbsolutePath(), lexiconConfig);
        }
	}
	
	/**
	 * Configuration values so as to parse the shoebox/toolbox lexicon
	 * @return lexiconConfig
	 */
	public LexiconFieldConfiguration getLexiconFieldConfig(){
		return lexiconConfig;
	}

	/**
	 * return true if the user clicked on ok in the JPanel Config
	 * @return boolean
	 */
	public boolean isConfigOK(){
		return configOK;
	}
}