package mpi.eudico.client.annotator.lexica;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;

/**
 * Document information of the imported (or created) lexicon
 * 
 * @author Coralie
 *
 */
public class LexiconDocumentData {
	static public Document DOCUMENT;
	
	static public File fileReference;
	static public boolean documentModified=false;
	
	//tags for xml file
	static public String ROOT="lexicon";
	static public String LEXICAL_ENTRY="lexicalEntry";
	static public String FORM="form";
	static public String SENSE="sense";
	static public String LEXEME="Lexeme";
	static public String MORPHOLOGY="morph";
	static public String SEGMENT="Segment";
	static public String ALTERNATE_FORM="altForm";
	static public String WRITTENFORM="WForm";
	static public String POS="Pos";
	static public String GLOSE="Gloss";
	static public String OTHER="stuff";
	
	//parameters value
	static public String ID="id";
	static public String DATE="dt";
	static public String LANGUAGE="lang";
	static public String MAIN_ENTRY="mn";
	static public String TAM="tam";
	static public String RX="tierX";
	static public String DERIVATION="der";
	static public String TYPE="typ";
	static public String REFERENCE="ref";
	
	//type values
	static public String LEM="lem";
	static public String STEM="stem";
	static public String SUFF="suff";
	static public String PREF="pref";
	static public String WF="wf";
	
	static public int MAX_ID;
	
	static public List<Element> NON_PARSING_LEXEME_LIST=new ArrayList<Element>();
	static public List<Element> LEXEME_LIST=new ArrayList<Element>();
	static public List<Element> VARIANT_LIST=new ArrayList<Element>();
	
	static public int modifiedElementId;
	static public int deletedElementId;
	static public int addedElementId;
	
	/**
	 * Construct an object that refers to the xml structure of this document 
	 * @param document
	 * @param f
	 */
	static public void setDocument(Document document, File f){
		DOCUMENT=document;
		fileReference=f;
		modifiedElementId=-1;
		deletedElementId=-1;
		addedElementId=-1;
		//documentModified=false;
		clearAllLists();
		initDocumentId();
	}
	
	/**
	 * Saves the current lexicon to file.
	 */
	static public void saveLexicon() throws IOException {
		LexiconSaver.saveLexicon(DOCUMENT, fileReference);
	}
	
	/**init the max id of a record to insert a new element
	 * **/
	static private void initDocumentId(){
		if(DOCUMENT!=null){
			List<Element> entries=DOCUMENT.getRootElement().getChildren(LexiconDocumentData.LEXICAL_ENTRY);
			for(Element entry:entries){
				String id=entry.getAttributeValue(ID);
				try{
					int i=Integer.parseInt(id); 
					if(i>MAX_ID){
						MAX_ID=new Integer(id);
					}
				}
				catch(NumberFormatException e){
					System.out.println("invalid id");
				}
			}
			MAX_ID++;
		}
	}
		
	static public void incrementMaxId(){
		MAX_ID++;
	}
	
	static public Element getElement(String id){
		int intId=Integer.parseInt(id);
		List<Element> entries=DOCUMENT.getRootElement().getChildren(LexiconDocumentData.LEXICAL_ENTRY);
		for(Element entry:entries){
			int currentId=Integer.parseInt(entry.getAttributeValue(ID));
			if(currentId==intId){
				Element lexeme=entry.getChild(LEXEME);
				return lexeme;
			}
		}
		return null;
	}
	
	static public void clearAllLists(){
		NON_PARSING_LEXEME_LIST.clear();
		LEXEME_LIST.clear();
		VARIANT_LIST.clear();
	}
	 /**
	  * @return a list of element to fill a comboBox
	  */
	 static public List<Element> getMorphemesToSelect(boolean isAVariant){
		 if(isAVariant){
			 if(VARIANT_LIST.isEmpty()){
				 VARIANT_LIST=initMorphemesToSelect(isAVariant);
				 Collections.sort(VARIANT_LIST, new Comparator<Element>() {	
						@Override
						public int compare(Element o1, Element o2) {
							if(o1!=null&&o2!=null){
								return o1.getText().compareTo(o2.getText());
							}
							else{
								return -1;
							}
						}
					});
			 }
			 return VARIANT_LIST;
		 }
		 else if(!isAVariant){
			 if(LEXEME_LIST.isEmpty()){
				 LEXEME_LIST=initMorphemesToSelect(isAVariant);
				 Collections.sort(LEXEME_LIST, new Comparator<Element>() {	
						@Override
						public int compare(Element o1, Element o2) {
							if(o1!=null&&o2!=null){
								return o1.getText().compareTo(o2.getText());
							}
							else{
								return -1;
							}
						}
					});
			 }
			 return LEXEME_LIST;
		 }
		 else{
			 return null;
		 }
	 }
	 
	 static public List<Element> getMorphemesToSelect(boolean isAVariant, boolean isAParsingForm){
		 if((!isAVariant)&&(!isAParsingForm)){
			 if(NON_PARSING_LEXEME_LIST.isEmpty()){
				 NON_PARSING_LEXEME_LIST=initNonParsingLexemeList();
				 Collections.sort(NON_PARSING_LEXEME_LIST, new Comparator<Element>() {	
						@Override
						public int compare(Element o1, Element o2) {
							if(o1!=null&&o2!=null){
								return o1.getText().compareTo(o2.getText());
							}
							else{
								return -1;
							}
						}
					});
			 }
			 return NON_PARSING_LEXEME_LIST;
		 }
		 else{
			 return null;
		 }
	 }
	 
	 private static List<Element> initMorphemesToSelect(boolean isAVariant){
		 List<Element> morphemesToSelect=new ArrayList<Element>();
		 Element nullElement=null;
		 morphemesToSelect.add(nullElement);
		 if(DOCUMENT!=null){
			 List<Element> lexicalEntries=DOCUMENT.getRootElement().getChildren();
			 for(Element lexicalEntry: lexicalEntries){
				 if(isAVariant){
					Element formElmt=lexicalEntry.getChild(FORM);
					if(formElmt!=null&&formElmt.getChildren(ALTERNATE_FORM)!=null){
						List<Element> altFormList=formElmt.getChildren(ALTERNATE_FORM);
						for(Element altFormElmt:altFormList){
							morphemesToSelect.add(altFormElmt.getChild(LexiconDocumentData.WRITTENFORM));
						}
					}
				 } else{
					morphemesToSelect.add(lexicalEntry.getChild(LEXEME)); 
				 }
			 }
		 }
		 return morphemesToSelect;
	 }
	 
	 private static List<Element> initNonParsingLexemeList(){
		 List<Element> morphemesToSelect=new ArrayList<Element>();
		 Element nullElement=null;
		 morphemesToSelect.add(nullElement);
		 if(DOCUMENT!=null){
			 List<Element> lexicalEntries=DOCUMENT.getRootElement().getChildren();
			 for(Element lexicalEntry: lexicalEntries){
				 if(lexicalEntry.getChild(MORPHOLOGY)==null){
					 morphemesToSelect.add(lexicalEntry.getChild(LEXEME));
				 }
			 }
		 }
		 return morphemesToSelect;
	 }
	 
	 //boolean to know if the user changes lexicon data or not
//	 public static void isModified(){
//		 documentModified=true;
//	 }
	 
	 public static void isModified(boolean modified){
		 documentModified=modified;
	 }
	 
	 public static void setDeletedElementId(int id){
		 deletedElementId=id;
	 }
	 
	 public static void setModifiedElementId(int id){
		 modifiedElementId=id;
	 }
	 
	 public static void setAddedElementId(int id){
		 addedElementId=id;
	 }
//	 private static List<Element> initMorphemesToSelect(boolean isAVariant){
//		 List<Element> morphemesToSelect=new ArrayList<Element>();
//		 Element nullElement=null;
//		 morphemesToSelect.add(nullElement);
//		 if(DOCUMENT!=null){
//			 List<Element> lexicalEntries=DOCUMENT.getRootElement().getChildren();
//			 for(Element lexicalEntry: lexicalEntries){
//				 if(isAVariant){
//					 if(lexicalEntry.getAttribute(LexiconDocumentDatas.MAIN_ENTRY)!=null){
//						 Element form=lexicalEntry.getChild(LexiconDocumentDatas.FORM);
//						 Element lexeme=form.getChild(LexiconDocumentDatas.LEXEME);
//						 morphemesToSelect.add(lexeme);
//						 System.out.println(lexeme);
//					 }
//					 else{
//						 Element form=lexicalEntry.getChild(LexiconDocumentDatas.FORM);
//						 if(form.getChildren(LexiconDocumentDatas.VARIANT)!=null){
//							 morphemesToSelect.addAll(form.getChildren(LexiconDocumentDatas.VARIANT)); 
//						 }
//					 }
//				 }
//				 else{
//					 if(lexicalEntry.getAttribute(LexiconDocumentDatas.MAIN_ENTRY)==null){
//						 Element form=lexicalEntry.getChild(LexiconDocumentDatas.FORM);
//						 Element lexeme=form.getChild(LexiconDocumentDatas.LEXEME);
//						 morphemesToSelect.add(lexeme);
//					 }
//				 }
//			 }
//		 }
//		 return morphemesToSelect;
//	 }
	 
//	 private static List<Element> initMorphemesToSelect(boolean isAVariant, boolean isAParsingForm){
//		 List<Element> morphemesToSelect=new ArrayList<Element>();
//		 Element nullElement=null;
//		 morphemesToSelect.add(nullElement);
//		 if(DOCUMENT!=null){
//			 List<Element> lexicalEntries=DOCUMENT.getRootElement().getChildren();
//			 for(Element lexicalEntry: lexicalEntries){
//				 if(isAVariant){
//					 if(lexicalEntry.getAttribute(LexiconDocumentDatas.MAIN_ENTRY)!=null){
//						 Element form=lexicalEntry.getChild(LexiconDocumentDatas.FORM);
//						 Element lexeme=form.getChild(LexiconDocumentDatas.LEXEME);
//						 if(isAParsingForm){
//							 if(lexeme.getAttributeValue(LexiconDocumentDatas.UNDERLYING_FORM)!=null){
//								 morphemesToSelect.add(lexeme);
//							 }
//						 }else{
//							 if(lexeme.getAttributeValue(LexiconDocumentDatas.UNDERLYING_FORM)==null){
//								 morphemesToSelect.add(lexeme);
//							 }
//						 }
//					 }
//				 }
//				 else{
//					 if(lexicalEntry.getAttribute(LexiconDocumentDatas.MAIN_ENTRY)==null){
//						 Element form=lexicalEntry.getChild(LexiconDocumentDatas.FORM);
//						 Element lexeme=form.getChild(LexiconDocumentDatas.LEXEME);
//						 if(isAParsingForm){
//							 if(lexeme.getAttributeValue(LexiconDocumentDatas.UNDERLYING_FORM)!=null){
//								 morphemesToSelect.add(lexeme);
//							 }
//						 }
//						 else{
//							 if(lexeme.getAttributeValue(LexiconDocumentDatas.UNDERLYING_FORM)==null){
//								 morphemesToSelect.add(lexeme);
//							 }
//						 }
//					 }
//				 }
//			 }
//		 }
//		 return morphemesToSelect;
//	 }
 }
