package mpi.eudico.client.annotator.lexica;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * represents the word morpheme's parse found in the current ELAN file
 * @author Coralie
 *
 */
public class LexiconMorphemes2 implements Comparable{
	private LinkedList<LexiconRef> suffixes;
	private List<LexiconRef> prefixes;
	private List<LexiconRef> roots;
	private boolean guessed=false;

	/**
	 * Constructs an empty LexiconMorphemes
	 */
	public LexiconMorphemes2(){
		roots=new ArrayList<LexiconRef>();
		suffixes=new LinkedList<LexiconRef>();
		prefixes=new ArrayList<LexiconRef>();
	}

	/**
	 * Constructs a LexiconMorphemes with another one
	 * @param morphemes
	 */
	public LexiconMorphemes2(LexiconMorphemes2 morphemes){
		roots=new ArrayList<LexiconRef>();
		roots=morphemes.getRoots();
		suffixes=new LinkedList<LexiconRef>();
		prefixes=new ArrayList<LexiconRef>();
		for(LexiconRef morphemesPrefix:morphemes.getPrefixes()){
			this.addPrefix(morphemesPrefix);
		}
		for(LexiconRef morphemesSuffix:morphemes.getSuffixes()){
			this.addLastSuffix(morphemesSuffix);
		}
	}

	public void addPrefix(LexiconRef lexicon){
		prefixes.add(lexicon);
	}

	public void addSuffix(LexiconRef lexicon){
		suffixes.addFirst(lexicon);
	}

	/**
	 * to add a suffix if you already know the segmentation and the order of each suffix
	 * @param lexicon
	 */
	public void addLastSuffix(LexiconRef lexicon){
		suffixes.addLast(lexicon);
	}

	/**
	 * return a String representation of the morpheme parse
	 * @return
	 */
	public String getMorphemes(){
		String morphemes="";
		if (!prefixes.isEmpty()){
			for(LexiconRef prefix:prefixes){
				morphemes+=prefix.getMorphemeForm();
			}
		}
		if (!roots.isEmpty()){
			for(LexiconRef root:roots){
				morphemes+=root.getMorphemeForm();
			}
		}
		if (!suffixes.isEmpty()){
			for(LexiconRef suffix: suffixes){
				morphemes+=suffix.getMorphemeForm();
			}
		}
		return morphemes;
	}

	/**
	 * Make an array representation of the lexicon
	 * @return
	 */
	public LexiconRef[] toArray(){
		LexiconRef[] lexiconArray=new LexiconRef[this.getSize()];
		int i=-1;
		if (!prefixes.isEmpty()){
			for(int x=0; x<prefixes.size(); x++){
				i++;
				lexiconArray[i]=prefixes.get(x);
			}
		}
		if (!roots.isEmpty()){
			for(int y=0; y<roots.size(); y++){
				i++;
				lexiconArray[i]=roots.get(y);
			}
		}
		if (!suffixes.isEmpty()){
			for(int z=0; z<suffixes.size(); z++){
				i++;
				lexiconArray[i]=suffixes.get(z);
			}
		}
		return lexiconArray;
	}

	public LexiconRef getMorpheme(int i){
		LexiconRef morpheme=null;
		int j=-1;
		if (!prefixes.isEmpty()){
			for(int y=0; y<prefixes.size(); y++){
				j++;
				if(j==i){
					return prefixes.get(y);
				}
			}
		}
		if (!roots.isEmpty()){
			for(int y=0; y<roots.size(); y++){
				j++;
				if(j==i){
					return roots.get(y);
				}
			}
		}
		if(!suffixes.isEmpty()){
			for(int y=0; y<suffixes.size(); y++){
				j++;
				if(j==i){
					return suffixes.get(y);
				}
			}
		}
		return morpheme;
	}

	/**
	 * Two LexiconMorphemes are equals if all prefixes, suffixes and root are equals
	 * @param m
	 * @return boolean
	 */
	public boolean equalsMorphemes(LexiconMorphemes2 m){
		if(prefixes.size()==m.getPrefixes().size()){
			for(int i=0; i<prefixes.size(); i++){
				if(!prefixes.get(i).equals(m.getPrefixes().get(i))){
					return false;
				}
			}
		}
		else{
			return false;
		}
		if(suffixes.size()==m.getSuffixes().size()){
			for(int i=0; i<suffixes.size();i++){
				if(!suffixes.get(i).equals(m.getSuffixes().get(i))){
					return false;
				}
			}
		}
		else{
			return false;
		}
		if(roots.size()==m.getRoots().size()){
			for(int i=0; i<roots.size(); i++){
				if(!roots.get(i).equals(m.getRoots().get(i))){
					return false;
				}
			}
		}
		else{
			return false;
		}
		return true;
	}

	public int getSize(){
		return prefixes.size()+suffixes.size()+roots.size();
	}

	public void setSuffixes(LinkedList<LexiconRef> suffixes) {
		this.suffixes=suffixes;
	}

	public LinkedList<LexiconRef> getSuffixes() {
		return suffixes;
	}

	public void setPrefixes(List<LexiconRef> prefixes) {
		this.prefixes=prefixes;
	}

	public List<LexiconRef> getPrefixes() {
		return prefixes;
	}
	
	public void addRoot(LexiconRef root){
		roots.add(root);
	}

	public void setRoots(List<LexiconRef> roots) {
		this.roots=roots;
	}

	public List<LexiconRef> getRoots() {
		return roots;
	}
	
	public boolean isGuessed(){
		return guessed;
	}
	
	public void setGuessed(boolean guessed){
		this.guessed=guessed;
	}

	@Override
	public int compareTo(Object lxMorph2) {
		int sizeMorph1=this.getSize();
		int sizeMorph2 = ((LexiconMorphemes2) lxMorph2).getSize();
		if (sizeMorph1>sizeMorph2){
			return 1; 
		}
		else if(sizeMorph1==sizeMorph2){
			return 0; 
		}
		else{
			return -1;
		} 
	}
			
}