package mpi.eudico.client.annotator.lexica;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jdom.Attribute;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
/***
 * Open a Toolbox Lexicon, make the list of field labels, 
 * make a list of Toolbox records, parse the Toolbox records
 * create an XML document, save the document as eafl file
 * 
 * @author Coralie
 *
 */

public class LexiconParser {
	//use fields to allow another configuration where rx=ps and so on
	private LexiconFieldConfiguration lexiconConfig;
	
	private SimpleDateFormat dateFormat=new SimpleDateFormat("dd/MMM/yyyy", Locale.ENGLISH);
	
	static private Element root;
	private Element lexicalEntryElmt;
	private Element formElmt;
	private Element senseElmt;

	private Pattern field_pattern;
	private Pattern empty_line;
//	private Pattern prefixPattern;
//	private Pattern cliticBPattern;
//	private Pattern suffixPattern;
//	private Pattern cliticEPattern;
	
	//private Pattern stemPattern;
	
	private List<Element> unreferencedMorphList;
	
	public LexiconParser(LexiconFieldConfiguration lexiconConfig){
		this.lexiconConfig=lexiconConfig;
	}
	
	public LexiconParser(){
		
	}
	
	/**
	 * Get a jdom Document from an Xml Lexicon 
	 * @param path
	 * @return
	 */
	public static void parseXmlLexicon(String path){
		SAXBuilder sxb=new SAXBuilder();
		try {
			File f=new File(path);
			LexiconDocumentData.setDocument(sxb.build(f), f);
		}
		catch(Exception e){
			System.out.println("document build failed");
		}
		root=LexiconDocumentData.DOCUMENT.getRootElement();
	}
	
	/**
	 * get each field/value pair in each shoebox/toolbox dictionary record 
	 * @param path
	 */
	public Element parseShoeboxDictionary(String path){
		compilePatterns();
		unreferencedMorphList=new ArrayList<Element>();
		
		BufferedReader bufferedReader;
		File file=new File(path);
		String xmlPath=null;
		try {
			InputStreamReader reader=new InputStreamReader(new FileInputStream(file),"UTF-8");
			bufferedReader=new BufferedReader(reader);
			String line=null;
			List<ShoeboxLexiconField> record=new ArrayList<ShoeboxLexiconField>();
			ShoeboxLexiconField currentField;
			
			root=new Element(LexiconDocumentData.ROOT);
			//LexiconDocumentDatas.setDocument(new Document(root));
			int id=-1;
			while((line=bufferedReader.readLine())!=null){
				Matcher m=empty_line.matcher(line);
				if(m.matches()){
					id++;
					id=parseRecord(record, id);
					record.clear();
				}
				else if(!line.contains("\\_")){
					m=field_pattern.matcher(line);
					if(m.find()){
						String fieldName="";
						for(int i= 0; i<= m.groupCount(); i++){
							fieldName=m.group(i);
						}	
						String value=line.substring(m.end());
						fieldName=fieldName.replaceFirst("\\\\", "");
						value=value.replaceFirst(" ", "");
						currentField=new ShoeboxLexiconField(fieldName, value);
						record.add(currentField);
					}
				}
			}
			id++;
			id=parseRecord(record, id);
			record.clear();
			bufferedReader.close();
			
			//createElanLexicon(root);
			//lookForParsingReferences();
			//xmlPath=path.replace(".txt", ".eafl");
			//LexiconDocumentData.setDocument(new Document(root), new File(xmlPath));
			//print(xmlPath);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch(IOException ioE){
			ioE.printStackTrace();
		}
		return root;
	}
	
//	private void createElanLexicon(Element root){
//		JFileChooser chooser = new JFileChooser();
//		chooser.setFileFilter(ElanFileFilter.createFileFilter(ElanFileFilter.ELAN_LEX_TYPE));
//		chooser.setDialogTitle("Create an Elan Lexicon");
//		int option = chooser.showSaveDialog(ELANCommandFactory.getRootFrame(transcription));
//		if (option == JFileChooser.APPROVE_OPTION) {
//			File f=chooser.getSelectedFile();
//
//			if (f != null) {
//				// make sure pathname finishes with .eafl
//				String pathName=f.getAbsolutePath();
//				String lowerPathName=pathName.toLowerCase();
//
//				if (!lowerPathName.endsWith(EAFL_EXT)) {
//					pathName+=EAFL_EXT;
//				}
//				if ((new File(pathName)).exists()) {
//					int answer = JOptionPane.showConfirmDialog(ELANCommandFactory.getRootFrame(transcription),
//							ElanLocale.getString("Message.Overwrite"),
//							ElanLocale.getString("SaveDialog.Message.Title"),
//							JOptionPane.YES_NO_OPTION);
//
//					if (answer==JOptionPane.NO_OPTION) {
//						return;
//					}
//				}
//				setPreference(LX_SOURCE, pathName, transcription);
//				lxPathLabel.setText(pathName);
//				lexicalPath=pathName;
//				print(pathName);
//				updateLinkedFile();
//			}
//		}
//	}
	
	/**
	 * parse record from shoebox/toolbox Lexicon
	 * @param record
	 * @param id
	 */
	private int parseRecord(List<ShoeboxLexiconField> record, int id){
		boolean rx=false;
		int refMnId=id;
		for(int i=0; i<record.size(); i++){
			ShoeboxLexiconField shoeboxField=record.get(i);
			//create new LexicalEntry and Lexeme
			if(shoeboxField.getFieldName().equals(lexiconConfig.getLx())){
				lexicalEntryElmt=new Element(LexiconDocumentData.LEXICAL_ENTRY);
				Attribute idAtt=new Attribute(LexiconDocumentData.ID, new Integer(id).toString());	
				lexicalEntryElmt.setAttribute(idAtt);
				root.addContent(lexicalEntryElmt);
				Element lexemeElmt=new Element(LexiconDocumentData.LEXEME);
				String lexValue=shoeboxField.getValue();
				lexemeElmt.setText(lexValue);

				//init type value
				Attribute typAtt;
				//if(InterlinearizationProcess.suffixPattern.matcher(lexValue).matches()||InterlinearizationProcess.cliticEPattern.matcher(lexValue).matches()||InterlinearizationProcess.redupPattern.matcher(lexValue).matches()){
				if(InterlinearizationProcess.endingPattern.matcher(lexValue).matches()){
					typAtt=new Attribute(LexiconDocumentData.TYPE, LexiconDocumentData.SUFF);
				}
				else if(InterlinearizationProcess.beginningPattern.matcher(lexValue).matches()){
				//else if(InterlinearizationProcess.prefixPattern.matcher(lexValue).matches()||InterlinearizationProcess.cliticBPattern.matcher(lexValue).matches()){
					typAtt=new Attribute(LexiconDocumentData.TYPE, LexiconDocumentData.PREF);
				} else if(InterlinearizationProcess.stemPattern.matcher(lexValue).matches()){
					typAtt=new Attribute(LexiconDocumentData.TYPE, LexiconDocumentData.STEM);
				} else{
					typAtt=new Attribute(LexiconDocumentData.TYPE, LexiconDocumentData.LEM);
				}
				lexemeElmt.setAttribute(typAtt);
				
				lexicalEntryElmt.addContent(lexemeElmt);
				formElmt=new Element(LexiconDocumentData.FORM);
				lexicalEntryElmt.addContent(formElmt);
				senseElmt=new Element(LexiconDocumentData.SENSE);
				lexicalEntryElmt.addContent(senseElmt);
			}
			//add date parameter to the lexeme
			else if(shoeboxField.getFieldName().equals(lexiconConfig.getDt())){
				Attribute dtAtt=new Attribute(LexiconDocumentData.DATE, shoeboxField.getValue());
				lexicalEntryElmt.setAttribute(dtAtt);
				if(lexicalEntryElmt.getAttribute(LexiconDocumentData.MAIN_ENTRY)!=null){
					String refId=lexicalEntryElmt.getAttributeValue(LexiconDocumentData.MAIN_ENTRY);
					List<Element>children= root.getChildren(LexiconDocumentData.LEXICAL_ENTRY);
					for(Element child: children){
						if(child.getAttributeValue(LexiconDocumentData.ID).equals(refId)){
							child.setAttribute(new Attribute(LexiconDocumentData.DATE, shoeboxField.getValue()));
						}
					}
				}
			}
			//add an alternateForm or a variant
			else if(shoeboxField.getFieldName().equals(lexiconConfig.getA())){
				Element altFormElmt=new Element(LexiconDocumentData.ALTERNATE_FORM);
				formElmt.addContent(altFormElmt);
				Element wFormElmt=new Element(LexiconDocumentData.WRITTENFORM);
				wFormElmt.setText(shoeboxField.getValue());
				altFormElmt.addContent(wFormElmt);

				//check if the next field is a parameter
				if(i+1<record.size()){
					ShoeboxLexiconField nextField=record.get(i+1);
					//add uForm parameter to the alternate form as a new record
					if(nextField.getFieldName().equals(lexiconConfig.getU())){
						Element morphElmt=new Element(LexiconDocumentData.MORPHOLOGY);
						String uForm=nextField.getValue();
						String[] uFormArray=uForm.split(" ");
						for(int j=0; j<uFormArray.length; j++){
							Element segmentElmt=new Element(LexiconDocumentData.SEGMENT);
							segmentElmt.setText(uFormArray[j]);
							morphElmt.addContent(segmentElmt);
						}
						unreferencedMorphList.add(morphElmt);
						altFormElmt.addContent(morphElmt);
						i++;
					}
				}
			}
			//add uForm parameter to the lexicalEntry
			else if(shoeboxField.getFieldName().equals(lexiconConfig.getU())){
				Element morphElmt=new Element(LexiconDocumentData.MORPHOLOGY);
				String uForm=shoeboxField.getValue();
				String[] uFormArray=uForm.split(" ");
				for(int j=0; j<uFormArray.length; j++){
					Element segmentElmt=new Element(LexiconDocumentData.SEGMENT);
					segmentElmt.setText(uFormArray[j]);
					morphElmt.addContent(segmentElmt);
				}
				unreferencedMorphList.add(morphElmt);
				lexicalEntryElmt.addContent(morphElmt);

				Element lexemeElmt=lexicalEntryElmt.getChild(LexiconDocumentData.LEXEME);
				if(lexemeElmt.getAttributeValue(LexiconDocumentData.TYPE).equals(LexiconDocumentData.LEM)){
					lexemeElmt.setAttribute(new Attribute(LexiconDocumentData.TYPE, LexiconDocumentData.WF));
				}
			}
			//add part of speech tag
			else if(shoeboxField.getFieldName().equals(lexiconConfig.getPos())){
				//if the record contains two different part of speech information, split the record in two
				if(!lexicalEntryElmt.getChildren(LexiconDocumentData.POS).isEmpty()){
					Element newLexicalEntryElmt=new Element(LexiconDocumentData.LEXICAL_ENTRY);
					Attribute mnAtt=new Attribute(LexiconDocumentData.MAIN_ENTRY, new Integer(refMnId).toString());
					newLexicalEntryElmt.setAttribute(mnAtt);
					id++;
					Attribute idAtt=new Attribute(LexiconDocumentData.ID, new Integer(id).toString());
					newLexicalEntryElmt.setAttribute(idAtt);
					Element lexemeElmt=lexicalEntryElmt.getChild(LexiconDocumentData.LEXEME);
					Element newLexemeElmt=new Element(LexiconDocumentData.LEXEME);
					newLexemeElmt.setText(lexemeElmt.getText());
					newLexemeElmt.setAttribute(LexiconDocumentData.TYPE, lexemeElmt.getAttributeValue(LexiconDocumentData.TYPE));
					newLexicalEntryElmt.addContent(newLexemeElmt);
					Element posElmt=new Element(LexiconDocumentData.POS);
					posElmt.setText(shoeboxField.getValue());
					newLexicalEntryElmt.addContent(posElmt);
					if(!rx){
						initRXWithPos(lexicalEntryElmt);
					}
					root.addContent(newLexicalEntryElmt);
					lexicalEntryElmt=newLexicalEntryElmt;
					rx=false;
				} else{
					Element posElmt=new Element(LexiconDocumentData.POS);
					posElmt.setText(shoeboxField.getValue());
					lexicalEntryElmt.addContent(posElmt);
				}
			}
			//add glose tag
			else if(shoeboxField.getFieldName().equals(lexiconConfig.getGe())){
				Element glossElmt=new Element(LexiconDocumentData.GLOSE);
				senseElmt.addContent(glossElmt);
				Attribute langAtt=new Attribute(LexiconDocumentData.LANGUAGE, "en");
				glossElmt.setAttribute(langAtt);
				String ge=shoeboxField.getValue();
				if(ge.contains("\\")){
					int limit=ge.indexOf("\\");
					String der=ge.substring(limit+1);		// CC 260811
					ge=ge.substring(0, limit);
					Attribute tamAtt=new Attribute(LexiconDocumentData.DERIVATION, der);
					glossElmt.setAttribute(tamAtt);
				}
				glossElmt.setText(ge);
			}
			else if(shoeboxField.getFieldName().equals(lexiconConfig.getRx())){
				if(!senseElmt.getChildren(LexiconDocumentData.GLOSE).isEmpty()){
					rx=true;
					List<Element> glossList=senseElmt.getChildren(LexiconDocumentData.GLOSE);
					for(Element glossElmt:glossList){
						if(glossElmt.getAttribute(LexiconDocumentData.RX)==null){
							Attribute tierXAtt=new Attribute(LexiconDocumentData.RX, shoeboxField.getValue());
							glossElmt.setAttribute(tierXAtt);
						}
					}
				}
				//if the rx element is associated with nothing keep the field in stuff tag
				else{
					addUndefinedElement(shoeboxField);
				}
			}
			//for other fields, keep the informations in stuff tag
			else{
				addUndefinedElement(shoeboxField);
			}
		}
		if(!rx){
			if(lexicalEntryElmt!=null){
				initRXWithPos(lexicalEntryElmt);
			}
		}
		return id;
	}

	/**
	 * make a new tag with others informations contained in the shoebox lexicon
	 * @param shoeboxField
	 */
	private void addUndefinedElement(ShoeboxLexiconField shoeboxField){
		Element undefinedField=new Element(shoeboxField.getFieldName());
		undefinedField.setText(shoeboxField.getValue());
		Element otherElmt;
		if(senseElmt.getChild(LexiconDocumentData.OTHER)!=null){
			otherElmt=senseElmt.getChild(LexiconDocumentData.OTHER);
		}
		else{
			otherElmt=new Element(LexiconDocumentData.OTHER);
			senseElmt.addContent(otherElmt);
		}
		otherElmt.addContent(undefinedField);
	}
	
	/**
	 * by default if rx is empty, take the pos value in rx value
	 * @param lexicalEntry
	 */
	private void initRXWithPos(Element lexicalEntry){
		if(lexicalEntry.getChild(LexiconDocumentData.POS)!=null){
			String posValue=lexicalEntry.getChild(LexiconDocumentData.POS).getValue();
			List<Element> glossElmtList=senseElmt.getChildren(LexiconDocumentData.GLOSE);
			for(Element glossElmt:glossElmtList){
				glossElmt.setAttribute(new Attribute(LexiconDocumentData.RX, posValue));
			}
		}
	}
	
	private void lookForParsingReferences(){
		if(!unreferencedMorphList.isEmpty()){
			for(Element morphElmt:unreferencedMorphList){
				Element elementToParse=null;
				List<Element> segmentList=morphElmt.getChildren(LexiconDocumentData.SEGMENT);
				if(morphElmt.getParentElement().getName().equals(LexiconDocumentData.ALTERNATE_FORM)){
					Element altFormElmt=morphElmt.getParentElement();
					Element wFormElmt=altFormElmt.getChild(LexiconDocumentData.WRITTENFORM);
					elementToParse=wFormElmt;
				} else if(morphElmt.getParentElement().getName().equals(LexiconDocumentData.LEXICAL_ENTRY)){
					Element lexicalEntryElmt=morphElmt.getParentElement();
					Element lexemeElmt=lexicalEntryElmt.getChild(LexiconDocumentData.LEXEME);
					elementToParse=lexemeElmt;
				}
				boolean lookForReferences=false;
				SegmentToDefine segmentToDefine=new SegmentToDefine(elementToParse.getText());
				for(Element segmentElmt: segmentList){
					List<Element> refElmtList=findReferenceElmts(segmentElmt.getText());
					//List<String> refIds=findReferenceId(segmentElmt.getText());
					if(!refElmtList.isEmpty()){
						if(refElmtList.size()==1){
							Element lexicalEntryElmt=refElmtList.get(0).getParentElement();
							String refId=lexicalEntryElmt.getAttributeValue(LexiconDocumentData.ID);
							Attribute refAtt=new Attribute(LexiconDocumentData.REFERENCE, refId);
							segmentElmt.setAttribute(refAtt);
						} else{
							System.out.println("choose between attributes");
							List<Element> elementReferences=new ArrayList<Element>();
							for(Element refElmt:refElmtList){
								lookForReferences=true;
								//Element element=LexiconDocumentDatas.getElement(id);
								elementReferences.add(refElmt);
								//System.out.println("id="+id);
							}
							segmentToDefine.addSegmentToDefine(segmentElmt.getText(), elementReferences);
							
						}
					} else{
						System.out.println("No element found for "+ segmentElmt.getText()+" wich is a part of the morpheme "+elementToParse.getText());
					}
				}
				if(lookForReferences){
					LXParsingReferenceDialog dialog=new LXParsingReferenceDialog(segmentToDefine);
					//dialog.setVisible(true);	
				}
			}
		}
	}
	
	private List<Element> findReferenceElmts(String elmtValue){
		List<Element> referenceElmtList=new ArrayList<Element>();
		List<Element> lexicalEntryList=LexiconDocumentData.DOCUMENT.getRootElement().getChildren();
		for(Element lexicalEntryElmt:lexicalEntryList){
			Element lexemeElmt=lexicalEntryElmt.getChild(LexiconDocumentData.LEXEME);
			if(lexemeElmt.getText().equals(elmtValue)){
				referenceElmtList.add(lexemeElmt);
//				referenceId.add(lexicalEntryElmt.getAttributeValue(LexiconDocumentDatas.ID));
			}
			Element form=lexicalEntryElmt.getChild(LexiconDocumentData.FORM);
			if(form.getChildren(LexiconDocumentData.ALTERNATE_FORM)!=null){
				List<Element> altFormList=form.getChildren(LexiconDocumentData.ALTERNATE_FORM);
				for(Element altForm:altFormList){
					if(altForm.getChild(LexiconDocumentData.MORPHOLOGY)==null){
						Element wFormElmt=altForm.getChild(LexiconDocumentData.WRITTENFORM);
						if(wFormElmt.getText().equals(elmtValue)){
							//referenceId.add(lexicalEntryElmt.getAttributeValue(LexiconDocumentDatas.ID));
							referenceElmtList.add(lexemeElmt);
						}
					}
				}
			}
		}
		return referenceElmtList;
	}
	
	private List<String> findReferenceId(String elmtValue){
		List<String> referenceId=new ArrayList<String>();
		List<Element> lexicalEntryList=LexiconDocumentData.DOCUMENT.getRootElement().getChildren();
		for(Element lexicalEntryElmt:lexicalEntryList){
			Element lexemeElmt=lexicalEntryElmt.getChild(LexiconDocumentData.LEXEME);
			if(lexemeElmt.getText().equals(elmtValue)){
				referenceId.add(lexicalEntryElmt.getAttributeValue(LexiconDocumentData.ID));
			}
			Element form=lexicalEntryElmt.getChild(LexiconDocumentData.FORM);
			if(form.getChildren(LexiconDocumentData.ALTERNATE_FORM)!=null){
				List<Element> altFormList=form.getChildren(LexiconDocumentData.ALTERNATE_FORM);
				for(Element altForm:altFormList){
					if(altForm.getChild(LexiconDocumentData.MORPHOLOGY)==null){
						Element wFormElmt=altForm.getChild(LexiconDocumentData.WRITTENFORM);
						if(wFormElmt.getText().equals(elmtValue)){
							referenceId.add(lexicalEntryElmt.getAttributeValue(LexiconDocumentData.ID));
						}
					}
				}
			}
		}
		return referenceId;
	}
	
//	private void parseRecord(List<ShoeboxLexiconField> record, int id){
//		Element currentLexicalEntry=null;
//		
//		for(int i=0; i<record.size(); i++){
//			ShoeboxLexiconField shoeboxField=record.get(i);
//			if(shoeboxField.getFieldName().equals(lexiconConfig.getLx())){
//				//create new LexicalEntry
//				Element lexicalEntry = new Element(LexiconDocumentDatas.LEXICAL_ENTRY);
//				Attribute idAtt=new Attribute(LexiconDocumentDatas.ID, new Integer(id).toString());	
//				lexicalEntry.setAttribute(idAtt);
//				root.addContent(lexicalEntry);
//				Element form=new Element(LexiconDocumentDatas.FORM);
//				lexicalEntry.addContent(form);
//				Element lexeme=new Element(LexiconDocumentDatas.LEXEME);
//				lexeme.setText(shoeboxField.getValue());
//				form.addContent(lexeme);
//				currentLexicalEntry=lexicalEntry;
//				Element sense=new Element(LexiconDocumentDatas.SENSE);
//				currentLexicalEntry.addContent(sense);
//				
//			}
//			//add date parameter to the lexeme
//			else if(shoeboxField.getFieldName().equals(lexiconConfig.getDt())){
//				Attribute dtAtt=new Attribute(LexiconDocumentDatas.DATE, shoeboxField.getValue());
//				currentLexicalEntry.setAttribute(dtAtt);
//				List<Element>children= root.getChildren(LexiconDocumentDatas.LEXICAL_ENTRY);
//				for(Element child: children){
//					if(child.getAttributeValue(LexiconDocumentDatas.MAIN_ENTRY)!=null&&child.getAttributeValue(LexiconDocumentDatas.DATE)==null){
//						if(child.getAttributeValue(LexiconDocumentDatas.MAIN_ENTRY).equals(currentLexicalEntry.getAttributeValue(LexiconDocumentDatas.ID))){
//							child.setAttribute(new Attribute(LexiconDocumentDatas.DATE, shoeboxField.getValue()));
//						}
//					}
//				}
//			}
//			//add an alternateForm or a variant
//			else if(shoeboxField.getFieldName().equals(lexiconConfig.getA())){
//				//check if the next field is a parameter
//				if(i+1<record.size()){
//					ShoeboxLexiconField nextField=record.get(i+1);
//					//add uForm parameter to the alternate form as a new record
//					if(nextField.getFieldName().equals(lexiconConfig.getU())){
//						Element alternateEntry=new Element(LexiconDocumentDatas.LEXICAL_ENTRY);
//						Attribute idAtt=new Attribute(LexiconDocumentDatas.ID, id+"ALT"+i);	
//						alternateEntry.setAttribute(idAtt);
//						Attribute mnAtt=new Attribute(LexiconDocumentDatas.MAIN_ENTRY, id+"");	
//						alternateEntry.setAttribute(mnAtt);
//						root.addContent(alternateEntry);
//						
//						Element form=new Element(LexiconDocumentDatas.FORM);
//						alternateEntry.addContent(form);
//						Element lexeme=new Element(LexiconDocumentDatas.LEXEME);
//						lexeme.setText(shoeboxField.getValue());
//						Attribute uFormAtt=new Attribute(LexiconDocumentDatas.UNDERLYING_FORM, nextField.getValue());
//						lexeme.setAttribute(uFormAtt);
//						form.addContent(lexeme);
//						i++;
//					}
//					else{
//						Element variant=new Element(LexiconDocumentDatas.VARIANT);
//						variant.setText(shoeboxField.getValue());
//						currentLexicalEntry.getChild(LexiconDocumentDatas.FORM).addContent(variant);
//					}
//				}
//				else{
//					//add variant (should not be read if the shoebox dictionary is well formed)
//					Element variant=new Element(LexiconDocumentDatas.VARIANT);
//					variant.setText(shoeboxField.getValue());
//					currentLexicalEntry.getChild(LexiconDocumentDatas.FORM).addContent(variant);
//					System.out.println("Mal-Formed Shoebox Dictionary file");
//				}
//			}
//			//add uForm parameter to the lexeme
//			else if(shoeboxField.getFieldName().equals(lexiconConfig.getU())){
//				Attribute uFormAtt=new Attribute(LexiconDocumentDatas.UNDERLYING_FORM, shoeboxField.getValue());
//				currentLexicalEntry.setAttribute(uFormAtt);
//			}
//			//add part of speech tag
//			else if(shoeboxField.getFieldName().equals(lexiconConfig.getPos())){
//				Element pos=new Element(LexiconDocumentDatas.POS);
//				Attribute psAtt=new Attribute(LexiconDocumentDatas.PS, shoeboxField.getValue());
//				pos.setAttribute(psAtt);
//				currentLexicalEntry.getChild(LexiconDocumentDatas.FORM).addContent(pos);
//				
//				List<String> gloseValues=new ArrayList<String>();
//				for (int y=i+1; y<record.size(); y++){
//					ShoeboxLexiconField nextField=record.get(y);
//					if(nextField.getFieldName().equals(lexiconConfig.getGe())){
//						gloseValues.add(nextField.getValue());
//					}
//					else {
//						for(int j=0; j<gloseValues.size(); j++){
//							String ge=gloseValues.get(j);
//							String tam="";
//							if(ge.contains("\\")){
//								int limit=ge.indexOf("\\");
//								tam=ge.substring(limit);
//								ge=ge.substring(0, limit);
//							}
//							Element glose=new Element(LexiconDocumentDatas.GLOSE);
//							Attribute langAtt=new Attribute(LexiconDocumentDatas.LANGUAGE, "en");
//							glose.setAttribute(langAtt);
//							if(nextField.getFieldName().equals(lexiconConfig.getRx())){
//								Attribute rxAtt=new Attribute(LexiconDocumentDatas.RX, nextField.getValue());
//								glose.setAttribute(rxAtt);
//								i=y;
//							}
//							//if rx doesn't exist take the ps value
//							else{
//								Attribute rxAtt=new Attribute(LexiconDocumentDatas.RX, shoeboxField.getValue());
//								glose.setAttribute(rxAtt);
//								i=y-1;
//							}
//							if (!tam.isEmpty()){
//								Attribute tamAtt=new Attribute(LexiconDocumentDatas.TAM, tam);
//								glose.setAttribute(tamAtt);
//							}
//							glose.setText(ge);
//							pos.addContent(glose);
//						}
//						break;
//					}
//				}
//				
//			}
//			//add glose tag (if pos doesn't exist)
//			else if(shoeboxField.getFieldName().equals(lexiconConfig.getGe())){
//				Element pos=new Element(LexiconDocumentDatas.POS);
//				currentLexicalEntry.getChild(LexiconDocumentDatas.FORM).addContent(pos);
//				List<String> gloseValues=new ArrayList<String>();
//				gloseValues.add(shoeboxField.getValue());
//				//check if the next field is a parameter
//				for (int y=i+1; y<record.size(); y++){
//					ShoeboxLexiconField nextField=record.get(y);
//					if(nextField.getFieldName().equals(lexiconConfig.getGe())){
//						gloseValues.add(nextField.getValue());
//					}
//					else {
//						for(int j=0; j<gloseValues.size(); j++){
//							String ge=gloseValues.get(j);
//							String tam="";
//							if(ge.contains("\\")){
//								int limit=ge.indexOf("\\");
//								tam=ge.substring(limit);
//								ge=ge.substring(0, limit);
//							}
//							Element glose=new Element(LexiconDocumentDatas.GLOSE);
//							Attribute langAtt=new Attribute(LexiconDocumentDatas.LANGUAGE, "en");
//							glose.setAttribute(langAtt);
//							if(nextField.getFieldName().equals(lexiconConfig.getRx())){
//								Attribute rxAtt=new Attribute(LexiconDocumentDatas.RX, nextField.getValue());
//								glose.setAttribute(rxAtt);
//								i=y;
//							}
//							else{
//								i=y-1;
//							}
//							if (!tam.isEmpty()){
//								Attribute tamAtt=new Attribute(LexiconDocumentDatas.TAM, tam);
//								glose.setAttribute(tamAtt);
//							}
//							glose.setText(ge);
//							pos.addContent(glose);
//						}
//						break;
//					}
//				}
//			}
//			else{
//				Element undefinedField=new Element(shoeboxField.getFieldName());
//				undefinedField.setText(shoeboxField.getValue());
//				currentLexicalEntry.getChild(LexiconDocumentDatas.SENSE).addContent(undefinedField);
//			}
//		}
//	}

	/**
	 * Creates an eafl file
	 * @param file
	 */
	public static void print(String file){
	   try {
	      XMLOutputter output=new XMLOutputter(Format.getPrettyFormat());
	      output.output(LexiconDocumentData.DOCUMENT, new FileOutputStream(file));
	   }
	   catch (java.io.IOException e){}
	}
	
	private void compilePatterns(){
		field_pattern=Pattern.compile("^\\\\.[^ ]*");
		empty_line=Pattern.compile("^$");
//		prefixPattern=Pattern.compile("^.*-$");
//		cliticBPattern=Pattern.compile("^.*=$");
//		suffixPattern=Pattern.compile("^-.*$");
//		cliticEPattern=Pattern.compile("^=.*$");
		//stemPattern=Pattern.compile("^.*_$");
	}
	
}
