package mpi.eudico.client.annotator.lexica;

import java.util.ArrayList;
import java.util.List;

/**
 * LexiconRecord created by an imported shoebox/toolbox dictionary
 * Fields could be added in next versions (such as definition, examples etc.)
 * @author Coralie
 *
 */
public class LexiconRecord2 {

	private LexiconField lexeme;
	private List<LexiconField> variants;
	private List<LexiconField> pos;
	private List<LexiconField> undeterminedFields;
	
	//parameters of the entire record (such as date and id)
	private List<String> parameterField;
	private List<String> parameterValue;
	
	/**
	 * A lexiconRecord has at least a lexeme name
	 * @param lexeme
	 */
	public LexiconRecord2(String lexeme){
		this.lexeme=new LexiconField(LexiconDocumentData.LEXEME, lexeme);
		variants=new ArrayList<LexiconField>();
		//gloses=new ArrayList<LexiconField>();
		undeterminedFields=new ArrayList<LexiconField>();
		pos=new ArrayList<LexiconField>();
		
		parameterField=new ArrayList<String>();
		parameterValue=new ArrayList<String>();

	}
	
	public LexiconField getLexeme(){
		return lexeme;
	}
	
	/**
	 * Adds a variant to the record variant list and return it
	 * @param variant
	 * @return the variant added
	 */
	public LexiconField addVariant(String variant){
		LexiconField lxvariant=new LexiconField(LexiconDocumentData.ALTERNATE_FORM, variant);
		this.variants.add(lxvariant);
		return lxvariant;
	}

	/**
	 * 
	 * @return the variant list
	 */
	public List<LexiconField> getVariants() {
		return variants;
	}
	
	/**
	 * select a variant which field value is the string in parameter
	 * @param value
	 * @return variant
	 */
	public LexiconField getVariant(String value){
		for (LexiconField variant : variants){
			if(variant.getFieldValue().equals(value)){
				return variant;
			}
		}
		return null;
	}

	//partOfSpeech
	public LexiconField addPos(String pos){
		LexiconField lxpos=new LexiconField(LexiconDocumentData.POS, pos);
		this.pos.add(lxpos);
		return lxpos;
	}
	
	public List<LexiconField> getPos(){
		return pos;
	}
	
	public LexiconField getPos(String value){
		for (LexiconField partOfSpeech : pos){
			if(partOfSpeech.getFieldValue().equals(value)){
				return partOfSpeech;
			}
		}
		return null;
	}
	
	//parameters of the record
	public List<String> getParameters(int i){
		List<String> parameter=new ArrayList<String>();
		parameter.add(parameterField.get(i));
		parameter.add(parameterValue.get(i));
		return parameter;
	}
	
	public void addParameter(String key, String value){
		parameterField.add(key);
		parameterValue.add(value);
	}
	
	public int parameterSize(){
		return parameterField.size();
	}

	//other fields that could appear in the record
	public void setUndeterminedFields(List<LexiconField> undeterminedFields) {
		this.undeterminedFields = undeterminedFields;
	}

	public List<LexiconField> getUndeterminedFields() {
		return undeterminedFields;
	}
	
	public LexiconField addUndeterminedField(String undeterminedFieldName, String undeterminedFieldValue){
		LexiconField uField=new LexiconField(undeterminedFieldName, undeterminedFieldValue);
		this.undeterminedFields.add(uField);
		return uField;
	}
	
}