package mpi.eudico.client.annotator.lexica;

import java.awt.Font;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.TableModelEvent;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.grid.GridEditor;

public class LexiconTable extends JTable {
	
	/** stores default minimal widths of columns */
	private static HashMap preferredWidths = new HashMap();

	private static HashMap maxWidths = new HashMap();

	private int width = -1;
	private static int MIN_COLUMN_WIDTH = 5;
	
    private boolean deselectCommits = false;
    
	 static {
		 preferredWidths.put(LexiconTableModel.TIMEPOINT, new Integer(15));
	     preferredWidths.put(LexiconTableModel.COUNT, new Integer(30));
		 preferredWidths.put(LexiconTableModel.LX, new Integer(50));
	     preferredWidths.put(LexiconTableModel.A, new Integer(50));
	     preferredWidths.put(LexiconTableModel.U, new Integer(50));
	     preferredWidths.put(LexiconTableModel.GE, new Integer(50));
	     preferredWidths.put(LexiconTableModel.RX, new Integer(50));
	     
	     maxWidths.put(LexiconTableModel.TIMEPOINT, new Integer(15));
	     maxWidths.put(LexiconTableModel.COUNT, new Integer(40));		// CC 260811
	     maxWidths.put(LexiconTableModel.LX, new Integer(200));
	     maxWidths.put(LexiconTableModel.A, new Integer(200));
	     maxWidths.put(LexiconTableModel.U, new Integer(200));
	     maxWidths.put(LexiconTableModel.GE, new Integer(200));
	     maxWidths.put(LexiconTableModel.RX, new Integer(200));
	 }
	

	//private JComboBox geComboBox;
	 
	private HashMap prefTierFonts;
	
	public LexiconTable(TableModel dataModel) {
	        super(dataModel);
	       // geComboBox=new JComboBox();
	        prefTierFonts = new HashMap();
	        setFont(Constants.DEFAULTFONT);
	        setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
	        setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);

	        for (int i=0; i<dataModel.getColumnCount(); i++) {
	            TableColumn column = getColumnModel().getColumn(i);
	            String columnName = dataModel.getColumnName(i);
	            column.setIdentifier(columnName);
	            setColumnVisible(columnName, true);
	        }
	        getColumn(LexiconTableModel.TIMEPOINT).setResizable(false);
	        getColumn(LexiconTableModel.COUNT).setResizable(true);	// CC 260811

	        getTableHeader().setReorderingAllowed(false);

	        //after this component gets visible, add component listener to parent component		
	        addComponentListener(new ComponentAdapter() {
	                @Override
					public void componentResized(ComponentEvent e) {
	                    //table is visible ?
	                    if (getWidth() > 0) {
	                        //component listener to parent component (e.g. viewport of scrollpane)
	                        getParent().addComponentListener(new ComponentAdapter() {
	                                //adjust columns if width of viewport changes
	                                @Override
									public void componentResized(ComponentEvent e) {
	                                    if (getParent().getWidth() != width) {
	                                        width = getParent().getWidth();
	                                        adjustColumns();
	                                    }
	                                }
	                            });
	                        removeComponentListener(this);
	                        adjustColumns();
	                    }
	                }
	            });
	    }
	
//    public Dimension getPreferredScrollableViewportSize() {
//        return new Dimension(500, 200);
//    }

    /**
     * Restores the default row height after an inline edit operation.
     * If this method is called by the GridEditor, the editor may be called to commit
     * instead of cancel, depending on a preference flag.
     *
     * @param e the change event
     */
    @Override
	public void editingStopped(ChangeEvent e) {
        if (deselectCommits && e.getSource() instanceof GridEditor) {
            ((GridEditor) e.getSource()).commitEdit();
        }

        super.editingStopped(e);
        setRowHeight(getRowHeight());
    }

    /**
     * Restores the default row height after an inline edit operation.
     *
     * @param e the change event
     */
    @Override
	public void editingCanceled(ChangeEvent e) {
        super.editingCanceled(e);
        setRowHeight(getRowHeight());
    }

    /**
     * distributes remaining table space among annotation columns
     * to be called after other columns had been set/removed or component resized
     */
    protected void adjustAnnotationColumns() {
        if (getParent() != null) {
            Set visibleAnnotationColumns = new HashSet();
            int sumOfOtherWidths = 0;

            for (int i = 0; i < getColumnCount(); i++) {
                TableColumn column = getColumnModel().getColumn(i);

                if (!preferredWidths.containsKey(column.getIdentifier()) &&
                        (column.getMaxWidth() > 0)) {
                    visibleAnnotationColumns.add(column);
                } else {
                    sumOfOtherWidths += column.getPreferredWidth();
                }
            }

            int remainingSpace = getParent().getWidth() - sumOfOtherWidths;

            if (remainingSpace > 0) {
                for (Iterator iter = visibleAnnotationColumns.iterator();
                        iter.hasNext();) {
                    ((TableColumn) iter.next()).setPreferredWidth(remainingSpace / visibleAnnotationColumns.size());
                }
            }
        }
    }
    
    /**
     * sets column width default value if true, to 0 if false
     * @param columnName
     * @param visible
     */
    public void setColumnVisible(String columnName, boolean visible) {
        TableColumn column = null;

        try {
            column = getColumn(columnName);

            // attention: don't change order of settings! A.K.
            if (visible) {
                column.setResizable(true);
                column.setMaxWidth(getDefaultMaxWidth(columnName));
                column.setMinWidth(MIN_COLUMN_WIDTH);
                column.setPreferredWidth(getDefaultPreferredWidth(columnName));
            } else {
                column.setMinWidth(0);
                column.setMaxWidth(0); // -> preferredWidth = 0
                column.setResizable(false);
            }
        } catch (IllegalArgumentException e) {
            System.out.println("Warning : no column with name " + columnName);
        }
    }

    /**
     * distributes remaining table space among annotation columns
     * to be called after other columns had been set/removed or component resized
     */
    protected void adjustColumns() {
        if (getParent() != null) {
            Set visibleAnnotationColumns = new HashSet();
            int sumOfOtherWidths = 0;

            for (int i = 0; i < getColumnCount(); i++) {
                TableColumn column = getColumnModel().getColumn(i);

                if(!preferredWidths.containsKey(column.getIdentifier())&&(column.getMaxWidth() > 0)) {
                    visibleAnnotationColumns.add(column);
                }else {
                    sumOfOtherWidths += column.getPreferredWidth();
                }
            }

            int remainingSpace = getParent().getWidth() - sumOfOtherWidths;

            if (remainingSpace > 0) {
                for (Iterator iter = visibleAnnotationColumns.iterator();
                        iter.hasNext();) {
                    ((TableColumn) iter.next()).setPreferredWidth(remainingSpace / visibleAnnotationColumns.size());
                }
            }
        }
    }

    /**
     * returns if column is visible (e.g. width>0)
     * @param columnName
     * @return boolean
     */
    public boolean isColumnVisible(String columnName) {
        TableColumn column = null;
        try {
            column = getColumn(columnName);
        } catch (IllegalArgumentException e) {
            System.out.println("Warning : no column with name " + columnName);
        }
        return (column != null) ? (column.getWidth() > 0) : false;
    }

    /**
     * Returns default width for columns.
     * @param columnName the column name
     * @return the default preferred width
     */
    private int getDefaultPreferredWidth(String columnName) {
        return preferredWidths.containsKey(columnName)
        ? ((Integer) preferredWidths.get(columnName)).intValue()
        : MIN_COLUMN_WIDTH;

        //swing default
    }

    /**
     * Returns default width for columns.
     * @param columnName the column name
     * @return the default max width
     */
    private int getDefaultMaxWidth(String columnName) {
        return maxWidths.containsKey(columnName)
        ? ((Integer) maxWidths.get(columnName)).intValue() : Integer.MAX_VALUE;

        //swing default
    }

    /**
      * method from ElanLocaleListener not implemented in AbstractViewer
      */
    public void updateLocale() {
        for (int i = 0; i < dataModel.getColumnCount(); i++) {
            int index = getColumnModel().getColumnIndex(dataModel.getColumnName(
                        i));
            getColumnModel().getColumn(index).setHeaderValue(ElanLocale.getString(
                    "Frame.GridFrame." + dataModel.getColumnName(i)));
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param fontSize DOCUMENT ME!
     */
   /* public void setFontSize(int fontSize) {
        setFont(Constants.DEFAULTFONT.deriveFont((float) fontSize));
        setRowHeight((((fontSize + 2) < 16) ? 16 : (fontSize + 3)));

        // update preferred fonts
        if (prefTierFonts.size() > 0) {
            Object key;
            Font f;
            Iterator keyIt = prefTierFonts.keySet().iterator();

            while (keyIt.hasNext()) {
                key = keyIt.next();
                f = (Font) prefTierFonts.get(key);

                if (f != null) {
                    prefTierFonts.put(key, f.deriveFont((float) fontSize));
                }
            }
        }
    }*/

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int getFontSize() {
        return getFont().getSize();
    }

    /**
     * Sets the names of the fonts for tiers. Based on this map and the current
     * fontsize, Font objects are created.
     *
     * @param fontNames : a map of tier name to font name<br>
     * NB the table model identifier for the annotations of the main tier in the
     * table is used for that tier instead of its name
     */
    public void setFontsForTiers(HashMap fontNames) {
        prefTierFonts.clear();

        if (fontNames != null) {
            Iterator keyIt = fontNames.keySet().iterator();
            String key;
            String fn;

            while (keyIt.hasNext()) {
                key = (String) keyIt.next();
                fn = (String) fontNames.get(key);

                if (fn != null) {
                    prefTierFonts.put(key, new Font(fn, Font.PLAIN, getFontSize()));
                }
            }
        }
    }

    /**
     * Returns the font for the specified column. Only applicable for annotation
     * columns.
     *
     * @param column the column index
     * @return the (preferred) font
     */
    public Font getFontForColumn(int column) {
        if (dataModel instanceof LexiconTableModel&&(prefTierFonts.size() > 0)) {
            String name = ((LexiconTableModel) dataModel).getColumnName(column);
            Font f = (Font) prefTierFonts.get(name);
            if(f==null){
                return getFont();
            } else{
                return f;
            }
        }
        return getFont();
    }

    /**
     * invokes adjustment of annotation columns
     * @see javax.swing.event.TableModelListener#tableChanged(TableModelEvent)
     */
    @Override
	public void tableChanged(TableModelEvent e) {
        super.tableChanged(e);
        adjustColumns();
    }

    /**
     * When true deselecting a cell that is being edited leads to a commit of the changes.
     * @param deselectCommits The deselectCommits to set.
     */
    public void setDeselectCommits(boolean deselectCommits) {
        this.deselectCommits = deselectCommits;
    }
}