package mpi.eudico.client.annotator.lexica;

import java.util.ArrayList;

import java.util.List;
import javax.swing.table.AbstractTableModel;

import org.jdom.Element;

/**
 * Structure of the Lexicon table
 * @author Coralie
 *
 */

public class LexiconTableModel extends AbstractTableModel{
	public static final String TIMEPOINT = "ColumnTriangle";

    /**DOCUMENT ME! */
    public static final String COUNT = "ColumnCount";

    /**DOCUMENT ME! */
    public static final String LX = "ColumnLx";

    /**DOCUMENT ME! */
    public static final String GE = "ColumnGe";

    /**DOCUMENT ME! */
    public static final String RX = "ColumnRx";

    /**DOCUMENT ME! */
    public static final String U = "ColumnU";

    /**DOCUMENT ME! */
    public static final String A = "ColumnA";

    protected List usedColumnNames;
    
    /**DOCUMENT ME! */
    public static final String[] fixedColumnNames = {
        TIMEPOINT, COUNT, LX, GE, RX, A, U
    };
    
    private List<Element> lexicalEntries;
    private List<LexiconTableContent> entries;
 //   private Document document;
    
    /**
     * Creates a GridResultModel using the specified dictionary as the data provider and the id's
     * as the columnidentifiers.
     *
     * @param annotations the (parent) annotations
     */
    
    public LexiconTableModel(List<LexiconTableContent> entries){
    	this.entries=entries;
    	usedColumnNames=new ArrayList();
    	usedColumnNames.add(TIMEPOINT);
    	usedColumnNames.add(COUNT);
    	usedColumnNames.add(LX);
    	usedColumnNames.add(A);
    	usedColumnNames.add(GE);
    	usedColumnNames.add(RX);
    	usedColumnNames.add(U);
    }
    
    public LexiconTableModel() {
    	if(LexiconDocumentData.DOCUMENT!=null){
    		lexicalEntries=LexiconDocumentData.DOCUMENT.getRootElement().getChildren();
    		entries=new ArrayList<LexiconTableContent>();
    		createLexiconTableContent(lexicalEntries);
    	}
    	usedColumnNames=new ArrayList();
    	usedColumnNames.add(TIMEPOINT);
    	usedColumnNames.add(COUNT);
    	usedColumnNames.add(LX);
    	usedColumnNames.add(A);
    	usedColumnNames.add(GE);
    	usedColumnNames.add(RX);
    	usedColumnNames.add(U);

    }
    
    private void createLexiconTableContent(List<Element> lexicalEntries){
    	LexiconTableContent currentEntry=null; 
    	int k=0;
    	for(Element lexicalEntryElmt:lexicalEntries){
    		List<Element> altFormElmtList=null;
    		List<Element> glossElmtList=null;
    		List<Element> segmentElmtList=null;
    		
    		String id=lexicalEntryElmt.getAttributeValue(LexiconDocumentData.ID);
    		Element lexemeElmt=lexicalEntryElmt.getChild(LexiconDocumentData.LEXEME);
    		Element formElmt=lexicalEntryElmt.getChild(LexiconDocumentData.FORM);
    		if(formElmt!=null){
    			altFormElmtList=formElmt.getChildren(LexiconDocumentData.ALTERNATE_FORM);
    		}
    		Element senseElmt=lexicalEntryElmt.getChild(LexiconDocumentData.SENSE);
    		if(senseElmt!=null){
    			glossElmtList=senseElmt.getChildren(LexiconDocumentData.GLOSE);
    		}
    		Element lexMorphElmt=lexicalEntryElmt.getChild(LexiconDocumentData.MORPHOLOGY);
    		if(lexMorphElmt!=null){
    			segmentElmtList=lexMorphElmt.getChildren(LexiconDocumentData.SEGMENT);
    		}
    		
    		if(glossElmtList!=null){
    			for(Element glossElmt:glossElmtList){
    				currentEntry=new LexiconTableContent(k);
    				currentEntry.setAssociatedLexicalEntryId(id);
    				currentEntry.setLexemeElmt(lexemeElmt);
    				currentEntry.setType(lexemeElmt.getAttributeValue(LexiconDocumentData.TYPE));
    				currentEntry.setLx(lexemeElmt.getText());
    				String der=glossElmt.getAttributeValue(LexiconDocumentData.DERIVATION);
    				if(der!=null){
    					currentEntry.setDer(der);
    				} 
    				currentEntry.setGe(glossElmt.getText());
    				currentEntry.setRx(glossElmt.getAttributeValue(LexiconDocumentData.RX));
    				if(altFormElmtList!=null){
    					for(Element altFormElmt:altFormElmtList){
    						Element wFormElmt=altFormElmt.getChild(LexiconDocumentData.WRITTENFORM);
    						LexiconVariantContent variantContent=new LexiconVariantContent(wFormElmt.getText());
    						if(wFormElmt.getAttribute(LexiconDocumentData.DERIVATION)!=null){
    							variantContent.setDer(wFormElmt.getAttribute(LexiconDocumentData.DERIVATION).getValue());
    						}
    						if(altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY)!=null){
    							Element altMorphElmt=altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY);
    							List<Element> altSegmentElmtList=altMorphElmt.getChildren(LexiconDocumentData.SEGMENT);
    							List<String> morphemes=new ArrayList<String>();
    							List<LexiconRef> lexiconRefList=new ArrayList<LexiconRef>(); 
    							for(Element altSegmentElmt:altSegmentElmtList){
    								variantContent.addElement(altSegmentElmt.getText());
    								morphemes.add(altSegmentElmt.getText());
    								if(altSegmentElmt.getAttribute(LexiconDocumentData.REFERENCE)!=null){
    									String refId=altSegmentElmt.getAttributeValue(LexiconDocumentData.REFERENCE);
    									//new LexiconRef()    									
    								}
    							}
    							ParsingMorpheme parsingMorpheme=new ParsingMorpheme(wFormElmt.getText(), id, morphemes, lexiconRefList);
    						}
    						currentEntry.addLexiconVariantContent(variantContent);
    					}
    				}
    				entries.add(currentEntry);
    				k++;
    			}
    		} 
    		if(segmentElmtList!=null){
    			currentEntry=new LexiconTableContent(k);
				currentEntry.setAssociatedLexicalEntryId(id);
				currentEntry.setLexemeElmt(lexemeElmt);
				currentEntry.setType(lexemeElmt.getAttributeValue(LexiconDocumentData.TYPE));
				currentEntry.setLx(lexemeElmt.getText());
				currentEntry.setGe("");
				currentEntry.setRx("");
				for(Element segmentElmt:segmentElmtList){
					currentEntry.addSegment(segmentElmt.getText());
				}
				if(altFormElmtList!=null){
					for(Element altFormElmt:altFormElmtList){
						Element wFormElmt=altFormElmt.getChild(LexiconDocumentData.WRITTENFORM);
						LexiconVariantContent variantContent=new LexiconVariantContent(wFormElmt.getText());
						if(wFormElmt.getAttribute(LexiconDocumentData.DERIVATION)!=null){
							variantContent.setDer(wFormElmt.getAttribute(LexiconDocumentData.DERIVATION).getValue());
						}
						if(altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY)!=null){
							Element variantMorphElmt=altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY);
							List<Element> variantSegmentList=variantMorphElmt.getChildren(LexiconDocumentData.SEGMENT);
							for(Element variantSegment:variantSegmentList){
								variantContent.addElement(variantSegment.getText());
							}
						}
						currentEntry.addLexiconVariantContent(variantContent);
					}
				}
				entries.add(currentEntry);
				k++;
    		}
    	}
    }

    /**
     * Returns whether or not the cell at the given row and column can be edited. This is the case
     * if the corresponding value is an instance of Annotation (e.g. not a Match) or a (possibly)
     * empty value in a column of a child tier. ! Be sure that all editable cells can be handled
     * by GridEditor !
     *
     * @param row the row index
     * @param column the column index
     *
     * @return true if the cell can be edited, false otherwise
     */
    @Override
	public boolean isCellEditable(int row, int column) {
    	return false;
    }

    /**
     * Returns the identifier of the column at the specified model index.
     *
     * @param nr the index of the column
     *
     * @return the identifier of the column
     */
    @Override
	public String getColumnName(int nr) {
        if ((0 <= nr) && (nr < usedColumnNames.size())) {
            return (String) usedColumnNames.get(nr);
        }
        return "";
    }
    
    public Object getValueAt(int row, int column) {
    	String lx=entries.get(row).getLx();
    	//HashMap<String, List<Element>> variantList=entries.get(row).getVariantMapList();
    	List<LexiconVariantContent> variantList=entries.get(row).getLexiconVariantList();
    	List<String> segmentList=entries.get(row).getSegmentList();
    	String ge=entries.get(row).getGe();
    	String der=entries.get(row).getDer();
    	String rx=entries.get(row).getRx();

    	String cn = getColumnName(column);

    	if (cn.equals(COUNT)) {
    		return "" + (row + 1);
    	}

    	if (cn.equals(LX)) {
    		return lx;
    	}
    	if (cn.equals(A)) {
    		String a="";
    		for(int i=0; i<variantList.size(); i++){
    			LexiconVariantContent variantContent=variantList.get(i);
    			a+=variantContent.getVariant();
    			String varDer=variantContent.getDer();
    			if(!varDer.isEmpty()){
    				a+="("+varDer+")";
    			}
    			if(i<variantList.size()-1){
    				a+="; ";
    			}
    		}
    		return a;
    	}
    	if (cn.equals(GE)) {
    		if(!der.isEmpty()){
    			ge+="\\"+der; 
    		}
    		return ge;
    	}

    	if (cn.equals(RX)) {
    		return rx;
    	}
    	if (cn.equals(U)) {
    		String u="";
    		for(int i=0; i<segmentList.size(); i++){
    			u+=segmentList.get(i);
    			if(i+1<segmentList.size()){
    				u+="; ";
    			}
    		}
    		return u;
    	}

    	return null;
    }

	public int getColumnCount() {
		return usedColumnNames.size();
	}

	public int getRowCount() {
		if(entries==null){
			return 0;
		} else{
		return entries.size();
		}
	}
	
	public List<LexiconTableContent> getEntries(){
		return entries;
	}
	
	public void setEntries(List<LexiconTableContent> entries){
		this.entries=entries;
	}

}
