package mpi.eudico.client.annotator.lexica;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.table.TableColumn;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.lexica.record.ShowEditLXDataDialog;
import mpi.eudico.client.annotator.viewer.AbstractViewer;
import mpi.eudico.client.annotator.viewer.LexiconViewer;

/**
 * Popup menu to hide a column in the lexicon Grid, modify font size,
 * see the record associated to a data in the table or 
 * export the elan lexicon as a new shoebox/toolbox dictionary
 * @author Coralie
 *
 */
public class LexiconViewerPopUpMenu extends JPopupMenu implements ActionListener{

	private AbstractViewer viewer;
	/** Holds value of property DOCUMENT ME! */
    final protected LexiconTable table;

    /** Holds value of property DOCUMENT ME! */
    final private JMenu fontMenuSize;
 
    /** Holds value of property DOCUMENT ME! */
    final private JMenu fontMenu;

    /** Holds value of property DOCUMENT ME! */
    final private JMenuItem fontMenuItem;
    
    /** Holds value of property DOCUMENT ME! */
    final private JMenuItem toggleMenuItem;

    /** Holds value of property DOCUMENT ME! */
    final private JMenuItem exportMenuItem;
    
    //final private JMenuItem editRecordMenuItem;
    final private JMenuItem showRecordMenuItem;

    /** Holds value of property DOCUMENT ME! */
    final private ButtonGroup fontSizeBG;
 
    /** add CC Holds value of property DOCUMENT ME! */
    final private ButtonGroup fontBG;
    
    private Font font; //CC 31/03/2011
    
    final private List<JMenuItem> columnNameItemList;
    /**
     * Creates a new LexiconViewerPopupMenu instance
     *
     * @param table DOCUMENT ME!
     */
    public LexiconViewerPopUpMenu(LexiconTable table, AbstractViewer viewer) {
    	this.table=table;
    	font=this.table.getFont();
    	this.viewer=viewer;
        fontSizeBG=new ButtonGroup();
        fontMenuSize=new JMenu(ElanLocale.getString("Menu.View.FontSize"));
 
        JRadioButtonMenuItem fontSizeRB;

        for (int i = 0; i < Constants.FONT_SIZES.length; i++) {
            fontSizeRB = new JRadioButtonMenuItem(String.valueOf(Constants.FONT_SIZES[i]));
            fontSizeRB.setActionCommand("font" + Constants.FONT_SIZES[i]);

            if(table.getFont().getSize()==Constants.FONT_SIZES[i]) {
                fontSizeRB.setSelected(true);
            }

            fontSizeRB.addActionListener(this);
            
            fontSizeBG.add(fontSizeRB);
            fontMenuSize.add(fontSizeRB);
        }
        
        //CC font change in Lexicon table
        fontBG=new ButtonGroup();
        fontMenu=new JMenu(ElanLocale.getString("FontDialog.Title"));
 
        //JRadioButtonMenuItem fontRB;

        //add timeformat toggle
        toggleMenuItem = new JMenuItem(ElanLocale.getString("Menu.Options.TimeFormat"));
        toggleMenuItem.setActionCommand("TOGGLETIMEFORMAT");
        toggleMenuItem.addActionListener(this);

        exportMenuItem = new JMenuItem(ElanLocale.getString("Frame.GridFrame.ExportTableAsToolboxDictionary"));
        exportMenuItem.setActionCommand("EXPORT");
        exportMenuItem.addActionListener(this);
        
        showRecordMenuItem=new JMenuItem(ElanLocale.getString("LexiconViewer.Grid.Label.ShowSelectedRecord"));
        showRecordMenuItem.setActionCommand("SHOWRECORD");
        showRecordMenuItem.addActionListener(this);
        
        fontMenuItem=new JMenuItem("Choose your font");
        fontMenuItem.setActionCommand("CHOOSEFONT");
        fontMenuItem.addActionListener(this);
               
        columnNameItemList=new ArrayList<JMenuItem>();
    }
    
	@Override
	public void actionPerformed(ActionEvent e) {
		  String actionCommand = e.getActionCommand();
		  if (actionCommand.equals("EXPORT")) {
			  ((LexiconViewer)viewer).exportLexicon();
		  }
		  else if(actionCommand.equals("SHOWRECORD")){
			  List<LexiconTableContent> entries=((LexiconTableModel)table.getModel()).getEntries();
			  if(table.getSelectedRow()!=-1){
				  int initialRow=Integer.parseInt((String) table.getValueAt(table.getSelectedRow(), 1));
				  LexiconTableContent content=entries.get(initialRow-1);
				  String lexicalEntryId=content.getAssociatedLexicalEntryId();
				  ShowEditLXDataDialog dialog=new ShowEditLXDataDialog(viewer, LexiconDocumentData.getElement(lexicalEntryId));
				  dialog.setVisible(true); 
			  }
		  }
		  else if (actionCommand.indexOf("font")!=-1) {
			  int index=actionCommand.indexOf("font")+4;

			  try {
				  table.setFont(font);
				  int fontSize=Integer.parseInt(actionCommand.substring(index));
				  table.setFont(font.deriveFont((float) fontSize)); // CC 31/03/2011
				  //table.setFontSize(Integer.parseInt(actionCommand.substring(index)));
				  table.repaint();
			  } catch (Exception ex) {
				  //LOG.info("Could not set font size");
			  }
		  }
		  /* else if(actionCommand.equals("CHOOSEFONT")){
			 selectFont();
		  }*/
		  else {
			  table.setColumnVisible(actionCommand, ((JCheckBoxMenuItem) e.getSource()).isSelected());
			  table.adjustAnnotationColumns();
		  }
		  table.doLayout();

	}
	
	protected void makeLayout() {
        removeAll();
        columnNameItemList.clear();
        
    	showRecordMenuItem.setEnabled(true);
        if (table.getSelectedRow()==-1){
        	showRecordMenuItem.setEnabled(false);
        }
        add(showRecordMenuItem);
        
        addSeparator();
        
        for (int i = 0; i < table.getColumnCount(); i++) {
            TableColumn column = table.getColumnModel().getColumn(i);
            String columnName = (String) column.getIdentifier();

            //Column ANNOTATION should not be removable and thus is not in selection menu
            if (!columnName.equals(LexiconTableModel.TIMEPOINT)) {
                JMenuItem menuItem=new JCheckBoxMenuItem(column.getHeaderValue().toString());
                menuItem.setActionCommand(columnName);
                menuItem.setSelected(table.isColumnVisible(columnName));
                menuItem.addActionListener(this);
                menuItem.setEnabled(true);
                columnNameItemList.add(menuItem);
            }
        }

        for(JMenuItem columnNameItem:columnNameItemList){
        	add(columnNameItem);
        }
        
        addSeparator();
        add(fontMenuSize);
        // add(fontMenu);	CC 31/03/2011
        addSeparator();

        add(exportMenuItem);
    }
	
    @Override
	public void setVisible(boolean b) {
        if (b) {
            makeLayout();
        }

        super.setVisible(b);
    }

}
