package mpi.eudico.client.annotator.lexica;

import java.awt.Component;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;
import javax.swing.border.LineBorder;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.annotator.viewer.AbstractViewer;

public class MorphemeTableRenderer implements TableCellRenderer{
	protected final AbstractViewer viewer;
	protected final TableModel tableModel;
	
	private JTextArea component;
	 
	public MorphemeTableRenderer(AbstractViewer viewer, TableModel tableModel) {
		this.viewer=viewer;
		this.tableModel=tableModel;
	}
	
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
		JTextArea label=new JTextArea();
		if(value instanceof LexiconRef){
			label.setText(((LexiconRef) value).getMorphemeForm());
		}
		else if (value instanceof String[]){
			label.setLineWrap(true);
			label.setWrapStyleWord(true);
			for (int i=0; i<((String[])value).length; i++) {
				label.append(((String[])value)[i]);
				if(i+1<((String[])value).length){
					label.append("\n");
				}
			}
			table.setRowHeight(table.getSelectedRow(), table.getRowHeight()*((String[])value).length);
		}
		else{
			label.setText((String) value);
		}
		label.setOpaque(true);
		component=label;
		component.setEnabled(true);
		
		boolean isActive=false;
		if(table.getSelectedColumn()==column &&row==table.getSelectedRow()&&table.getValueAt(row, column)!=null){
			isActive=true;
		}

		setComponentLayout(component, table, isSelected, isActive);
        setAlignment(component, table.getColumnName(column));
		return component; 
	}

	protected static void setComponentLayout(JComponent component,JTable table, boolean isSelected, boolean isActive) {
		component.setFont(table.getFont());
		component.setBorder(BorderFactory.createEmptyBorder(0,1, 0, 3));
		component.setBackground(isSelected ? Constants.SELECTIONCOLOR: table.getBackground());
		component.setBorder(isActive ? new LineBorder(Constants.ACTIVEANNOTATIONCOLOR): table.getBorder());
	}
	  
	protected static void setAlignment(JComponent component, String columnName) {
		component.setAlignmentX(SwingConstants.CENTER);
	}
	  
//	@Override
//	public void itemStateChanged(ItemEvent e) {
//		// TODO Auto-generated method stub
//		if (e.getStateChange() == ItemEvent.SELECTED){
//			
//		}
//	}
}
