package mpi.eudico.client.annotator.lexica.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.RowFilter;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.lexica.InterlinearizationProcess;
import mpi.eudico.client.annotator.lexica.LexiconRef;
import mpi.eudico.client.annotator.lexica.LexiconTable;
import mpi.eudico.client.annotator.lexica.LexiconTableContent;
import mpi.eudico.client.annotator.lexica.LexiconTableModel;
import mpi.eudico.client.annotator.viewer.AbstractViewer;
import mpi.eudico.client.annotator.viewer.LexiconViewer;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.type.Constraint;
import mpi.eudico.server.corpora.clom.Annotation;
import mpi.eudico.server.corpora.clom.Transcription;

/**
 * Contains the elements of the Annotate tab
 * @author Coralie
 *
 */
public class LexiconAnnotationPanel extends JPanel implements ActionListener{

	private AbstractViewer viewer;
	//private InterlinearizationProcess interlinearizationProcess;
	
//	private List<ParsingMorpheme> rootUnderlyingForms;
//	private List<ParsingMorpheme> suffixUnderlyingForms;
//	private List<ParsingMorpheme> prefixUnderlyingForms;
	private LexiconTable lxTable;
	private LexiconTableModel lxTableModel;
	
	private List<LexiconRef> rootForms;
	private List<LexiconRef> suffixForms;
	private List<LexiconRef> prefixForms;
	
	private JButton annotate;
	private JButton autoAnnotate;
	private List<Integer> filterIndexes;
	private JLabel label;
	
	public LexiconAnnotationPanel(AbstractViewer viewer, LexiconTableModel model, LexiconTable lxTable, InterlinearizationProcess interlinearizationProcess){
		this.viewer=viewer;
		this.lxTable=lxTable;
		//lxTable.addMouseListener(this);
		lxTableModel=model;
		
		initLexiconRefs(interlinearizationProcess);
		filterIndexes=new ArrayList<Integer>();
		
		setLayout(new BorderLayout());
		JPanel buttonPanel=new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		
//		setLayout(new GridBagLayout());
//		GridBagConstraints gbc=new GridBagConstraints();
//		gbc.fill=GridBagConstraints.BOTH;
//		gbc.gridx=0;
//		gbc.gridy=0;	
		annotate=new JButton(ElanLocale.getString("LexiconAnnotationPanel.JButton.Annotate"));
		annotate.addActionListener(this);
		buttonPanel.add(annotate);
		
//		gbc.gridx=1;
//		gbc.gridy=0;
		autoAnnotate=new JButton(ElanLocale.getString("LexiconAnnotationPanel.JButton.AutoAnnotate"));
		autoAnnotate.addActionListener(this);
		buttonPanel.add(autoAnnotate);
//		gbc.gridx=0;
//		gbc.gridy=1;

		add(buttonPanel, BorderLayout.NORTH);
		
		label=new JLabel();
		add(label, BorderLayout.CENTER);
		
		add(Box.createHorizontalStrut(200), BorderLayout.SOUTH);
		
	}
	
	private void initLexiconRefs(InterlinearizationProcess interlinearizationProcess){
		rootForms=interlinearizationProcess.getRootForms();
		suffixForms=interlinearizationProcess.getSuffixForms();
		prefixForms=interlinearizationProcess.getPrefixForms();
	}
	
	private void searchAnnotation(String morpheme){
		if(InterlinearizationProcess.beginningPattern.matcher(morpheme).matches()){
//		if(InterlinearizationProcess.prefixPattern.matcher(morpheme).matches()||InterlinearizationProcess.cliticBPattern.matcher(morpheme).matches()){
			for(LexiconRef prefix:prefixForms){
				if(prefix.getMorphemeForm().equals(morpheme)){
					for(LexiconTableContent lexicon:prefix.getLexicons()){
						filterIndexes.add(lexicon.getGridIndex());
					}
					break;
				}
			}
		} 
		else if(InterlinearizationProcess.endingPattern.matcher(morpheme).matches()){
		//else if(InterlinearizationProcess.suffixPattern.matcher(morpheme).matches()||InterlinearizationProcess.cliticEPattern.matcher(morpheme).matches()||InterlinearizationProcess.redupPattern.matcher(morpheme).matches()){
			for(LexiconRef suffix:suffixForms){
				if(suffix.getMorphemeForm().equals(morpheme)){
					for(LexiconTableContent lexicon:suffix.getLexicons()){
						filterIndexes.add(lexicon.getGridIndex());
					}
					break;
				}
			}
		}
		else{
			for(LexiconRef root:rootForms){
				if(root.getMorphemeForm().equals(morpheme)){
					for(LexiconTableContent lexicon:root.getLexicons()){
						filterIndexes.add(lexicon.getGridIndex());
					}
					break;
				}
			}
		}
	}

	private boolean checkHierarchy(){
		Annotation ann=viewer.getActiveAnnotation();
		Transcription transcription=((LexiconViewer)viewer).getTranscription();
		AnnotationTierHierarchyInfos infos=new AnnotationTierHierarchyInfos(viewer, transcription);
		
		String prefTierParse=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.ANNOTATION_TIER_PARSE);
		String prefTierGloss=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.ANNOTATION_TIER_GLOSS);
		String prefTierPos=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.ANNOTATION_TIER_POS);

		boolean correctHierarchy=false;
		boolean correctGlossLingType=true;
		boolean correctPosLingType=true;
		
		if(ann.getTier().getName().startsWith(prefTierParse)){
			TierImpl parseTier=(TierImpl) ann.getTier();
			infos.setAnnParseTier(parseTier);
			Vector<TierImpl> parseTierChildren=parseTier.getChildTiers();
			
			TierImpl glossTier=null;
			TierImpl posTier=null;
			for(TierImpl annTier:parseTierChildren){
				if(annTier.getName().startsWith(prefTierGloss)){
					glossTier=annTier;
					if(glossTier.getLinguisticType().getConstraints()!=null){
						if(glossTier.getLinguisticType().getConstraints().getStereoType()!=Constraint.SYMBOLIC_ASSOCIATION){
							correctGlossLingType=false;
						}
					}
					else{
						correctGlossLingType=false;
					}
					infos.setAnnGlossTier(glossTier);
				} else if(annTier.getName().startsWith(prefTierPos)){
					posTier=annTier;
					if(posTier.getLinguisticType().getConstraints()!=null){
						if(posTier.getLinguisticType().getConstraints().getStereoType()!=Constraint.SYMBOLIC_ASSOCIATION){
							correctPosLingType=false;
						}
					}
					else{
						correctPosLingType=false;
					}
					infos.setAnnPosTier(posTier);
				}
			}
			if(glossTier!=null&&posTier!=null){
				correctHierarchy=true;
			}
		}
			
		if(correctHierarchy&&correctGlossLingType&&correctPosLingType){
			return true;
		}
		else{
			new WrongAnnotationParametersFrame(viewer, infos);
			return false;
		}
	}
	
	private void filterLexiconTable(){
		TableRowSorter<TableModel> sorter=new TableRowSorter<TableModel>(lxTableModel);
		lxTable.setRowSorter(sorter);

		RowFilter<Object,Object> rowFilter = new RowFilter<Object,Object>() {
			@Override
			public boolean include(Entry<? extends Object, ? extends Object> entry) {
				for(int i=0; i<entry.getValueCount();){
					for(int l=0; l<filterIndexes.size(); l++){
						if(entry.getIdentifier().equals(filterIndexes.get(l))) {
							return true;
						}
					}
					return false;
				}
				return false;
			}
		};
		sorter.setRowFilter(rowFilter);
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		if(viewer.getActiveAnnotation()!=null){
			if (e.getSource()==annotate) {
				if(checkHierarchy()){
					annotate();
				}
			}
			if(e.getSource()==autoAnnotate){
				if(checkHierarchy()){
					TierImpl mbTier=(TierImpl)viewer.getActiveAnnotation().getTier();
					autoAnnotate(mbTier);					
				}
			}				
		}
	}
	
	private void annotate(){
		String annotation=viewer.getActiveAnnotation().getValue();
		label.setText(annotation);
		filterIndexes.clear();
		searchAnnotation(annotation);
		filterLexiconTable();
	}
	
	private void autoAnnotate(TierImpl mbTier){
		annotate();
		if(filterIndexes.size()==1){
			int geInt=lxTableModel.findColumn(LexiconTableModel.GE);
			int rxInt=lxTableModel.findColumn(LexiconTableModel.RX);

			String ge=(String) lxTable.getValueAt(0, geInt);
			String rx=(String) lxTable.getValueAt(0, rxInt);

			annotateTiers(mbTier, ge, rx);
			if(viewer.getActiveAnnotation()!=null){
				autoAnnotate(mbTier);
			}
		}
	}
	
	public void annotateTiers(TierImpl mbTier, String ge, String rx){
		int transMorphBreakChar=(Integer)((LexiconViewer) viewer).getMorphemeBreakPreferences(LexiconViewer.TRANS_BREAK_CHAR);
		
		TierImpl geTier=null;
		TierImpl rxTier=null;
		Vector mbChildren=mbTier.getChildTiers();
		if(mbChildren.size()>=2){
			for(int k=0; k<mbChildren.size(); k++){
				TierImpl currentMbChildren=(TierImpl)mbChildren.get(k);
				String cMbChildren=currentMbChildren.getName();
				String gloss=(String) ((LexiconViewer) viewer).getTierPreferences(LexiconViewer.ANNOTATION_TIER_GLOSS);
				String pos=(String) ((LexiconViewer) viewer).getTierPreferences(LexiconViewer.ANNOTATION_TIER_POS);
				//if(cMbChildren.getName().startsWith((String) ((LexiconViewer) viewer).getTierPreferences(LexiconViewer.ANNOTATION_TIER_GLOSS))){
				if(cMbChildren.equals(gloss)){		// CC 310811
					geTier=currentMbChildren;
				}
				else if(cMbChildren.equals(pos)){
				//else if(currentMbChildren.getName().startsWith((String) ((LexiconViewer) viewer).getTierPreferences(LexiconViewer.ANNOTATION_TIER_POS))){ // CC 310811
					rxTier=currentMbChildren;
				}
			}
		}
		Annotation activeAnnotation=viewer.getActiveAnnotation();
		Vector<Annotation> geAnnotations=activeAnnotation.getChildrenOnTier(geTier);
		for(Annotation annotation:geAnnotations){
			geTier.removeAnnotation(annotation);	
		}
		Vector<Annotation> rxAnnotations=activeAnnotation.getChildrenOnTier(rxTier);
		for(Annotation annotation:rxAnnotations){
			rxTier.removeAnnotation(annotation);	
		}
		
		if(mbTier==null||geTier==null||rxTier==null){
			System.out.println("Problem in the hierarchy !");
		}
		else{
			if(transMorphBreakChar==1){
				if(InterlinearizationProcess.endingPattern.matcher(activeAnnotation.getValue()).matches()){
					char endChar=activeAnnotation.getValue().charAt(0);
					ge=endChar+ge;
					rx=endChar+rx;
				} else if(InterlinearizationProcess.beginningPattern.matcher(activeAnnotation.getValue()).matches()){
					char beginningChar=activeAnnotation.getValue().charAt(activeAnnotation.getValue().length()-1);
					ge=ge+beginningChar;
					rx=rx+beginningChar;
				}
			}
			Annotation geAnn=geTier.createAnnotation(activeAnnotation.getBeginTimeBoundary(), activeAnnotation.getBeginTimeBoundary());
			geAnn.setValue(ge);
			Annotation rxAnn=rxTier.createAnnotation(activeAnnotation.getBeginTimeBoundary(), activeAnnotation.getBeginTimeBoundary());
			rxAnn.setValue(rx);
			
			viewer.setActiveAnnotation(mbTier.getAnnotationAtTime(activeAnnotation.getEndTimeBoundary()));
			removeAnnotationInformation();
		}
	}


	private void removeAnnotationInformation(){
		filterIndexes.clear();
		lxTable.setRowSorter(null);
		label.setText("");
		repaint();
	}
//	@Override
//	public void mouseClicked(MouseEvent arg0) {
//		// TODO Auto-generated method stub
//		
//	}
//
//	@Override
//	public void mouseEntered(MouseEvent arg0) {
//		// TODO Auto-generated method stub
//		
//	}
//
//	@Override
//	public void mouseExited(MouseEvent arg0) {
//		// TODO Auto-generated method stub
//		
//	}
//
//	@Override
//	public void mousePressed(MouseEvent e) {
//		if (e.getClickCount()>=2){
//			System.out.println("double click");
//			int row=lxTable.getSelectedRow();
//			int lxInt=lxTableModel.findColumn(lxTableModel.LX);
//			int geInt=lxTableModel.findColumn(lxTableModel.GE);
//			int rxInt=lxTableModel.findColumn(lxTableModel.RX);
//			int aInt=lxTableModel.findColumn(lxTableModel.A);
//
//			String lx=(String) lxTable.getValueAt(row, lxInt);
//			String ge=(String) lxTable.getValueAt(row, geInt);
//			String rx=(String) lxTable.getValueAt(row, rxInt);
//			String a=(String) lxTable.getValueAt(row, aInt);
//			String[] parsingA=a.split("; ");
//		}
//		
//	}
//
//	@Override
//	public void mouseReleased(MouseEvent arg0) {
//		// TODO Auto-generated method stub
//		
//	}
}
