package mpi.eudico.client.annotator.lexica.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTable;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.lexica.InterlinearizationProcess;
import mpi.eudico.client.annotator.lexica.LexiconRef;
import mpi.eudico.client.annotator.lexica.LexiconTable;
import mpi.eudico.client.annotator.lexica.LexiconTableContent;
import mpi.eudico.client.annotator.lexica.LexiconTableModel;
import mpi.eudico.client.annotator.lexica.LexiconVariantContent;
import mpi.eudico.client.annotator.viewer.AbstractViewer;
import mpi.eudico.client.annotator.viewer.LexiconViewer;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;

/**
 * Panel with tabbedPane which included SegmentationPanel and AnnotationPanel
 * @author Coralie
 * CC 050911 : add the '\der' of a variant to the end of the mb line segment. mot: targi > mb: touareg\PL
 * with targi as a variant of tuareg, with der=PL
 */
public class LexiconSegmentationAnnotationPanel extends JPanel implements MouseListener{
	private AbstractViewer lxViewer;
	private JTabbedPane tabPane;
	private LexiconSegmentationPanel lexiconSegmentationPane;
	private LexiconAnnotationPanel lexiconAnnotationPane;
	
	private LexiconTableModel lxTableModel;
	private LexiconTable lxTable;
	private InterlinearizationProcess interlinearizeProcess;
	private TranscriptionImpl transcription;	// CC 310811
	
	// CC 310811
	public LexiconSegmentationAnnotationPanel(AbstractViewer viewer, LexiconTableModel model, LexiconTable lxTable, InterlinearizationProcess process){
		lxViewer=viewer;
		this.lxTableModel=model;
		this.lxTable=lxTable;
		this.lxTable.addMouseListener(this);
		interlinearizeProcess=process;
		tabPane=new JTabbedPane();
		setLayout(new GridBagLayout());
		initTabPane();
	}
	
	private void initTabPane(){
		GridBagConstraints gbc=new GridBagConstraints();
		gbc.fill=GridBagConstraints.BOTH;
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.weightx=1;
		gbc.weighty=1;
		// CC 310811 ajout transcription
		lexiconSegmentationPane=new LexiconSegmentationPanel(lxViewer, lxTableModel, lxTable, interlinearizeProcess);
		lexiconAnnotationPane=new LexiconAnnotationPanel(lxViewer, lxTableModel, lxTable, interlinearizeProcess);
		tabPane.addTab(ElanLocale.getString("LexiconAnnotationSegmentationPanel.TabPane.Parse"), lexiconSegmentationPane);
		tabPane.addTab(ElanLocale.getString("LexiconAnnotationSegmentationPanel.TabPane.Annotate"), lexiconAnnotationPane);
		add(tabPane, gbc);
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		
	}

	@Override
	public void mouseExited(MouseEvent e) {
		
	}

	@Override
	public void mousePressed(MouseEvent e) {		// lexicon area listener
		//lexiconSegmentationPane
		if(lxViewer.getActiveAnnotation()!=null){ 	// a word to interlinearize is selected
			if(tabPane.getSelectedIndex()==0){		// Interlinearize tab
				JTable parsingTable=lexiconSegmentationPane.getParsingTable();
				int selectedParsingRow=parsingTable.getSelectedRow();
				int selectedParsingColumn=parsingTable.getSelectedColumn();
				//double-click on a lexicon row = choose the correct morphem
				if (e.getClickCount()>=2){	
					if(parsingTable.getSelectedRow()!=-1){	// pre-selected segmentation
						int parseAnnotation=0;

						for(int i=0; i<parsingTable.getColumnCount(); i++){
							if(parsingTable.getValueAt(selectedParsingRow, i) instanceof String[]){
								parseAnnotation++;		//number of segments
							}
						}
						//parsingIndex
						int row=lxTable.getSelectedRow();

						String count=(String)lxTable.getValueAt(row, lxTableModel.findColumn(LexiconTableModel.COUNT));
						
						LexiconTableContent tableContent=null;
						for(LexiconTableContent entry:lxTableModel.getEntries()){
							if(entry.getGridIndex()+1==Integer.parseInt(count)){
								tableContent=entry;
								break;
							}
						}

						String lx=tableContent.getLx();
						String ge=tableContent.getGe();
						String der=tableContent.getDer();
						String rx=tableContent.getRx();
						List<LexiconVariantContent> variantContentList=tableContent.getLexiconVariantList();

						String morphemeForm="";
						if(parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn) instanceof LexiconRef){
							morphemeForm=((LexiconRef)parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn)).getMorphemeForm();
						}
						else{
							if(parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn) instanceof String[]){
								morphemeForm=((String[])parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn))[0];
							}
						}

						if(parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn)!=null){
							//check if the value equals an lx or a variant form
							if(morphemeForm.equals(lx)){
								if(!der.isEmpty()){
									ge+="\\"+der;
								}
								String[] tableValue={morphemeForm, lx, ge, rx};
								lexiconSegmentationPane.associateMorpheme(selectedParsingRow, selectedParsingColumn, tableValue, parseAnnotation);
							}
							else{			//variant
								for(LexiconVariantContent variantContent:variantContentList){
									if(morphemeForm.equals(variantContent.getVariant())){
										if(!variantContent.getDer().isEmpty()){
											ge+="\\"+variantContent.getDer();
											lx+="\\"+variantContent.getDer();		// CC 050911 add the \der of the variant to the segment (mb line)
										} else if(!der.isEmpty()){
											ge+="\\"+der;
										}
										String[] tableValue={morphemeForm, lx, ge, rx};
										lexiconSegmentationPane.associateMorpheme(selectedParsingRow, selectedParsingColumn, tableValue, parseAnnotation);

									}
								}
							}
						}
					}
				}
			}
			//lexiconAnnotationPane
			else{			// Annotate tab
				if (e.getClickCount()>=2){
					int row=lxTable.getSelectedRow();
					int lxInt=lxTableModel.findColumn(LexiconTableModel.LX);
					int geInt=lxTableModel.findColumn(LexiconTableModel.GE);
					int rxInt=lxTableModel.findColumn(LexiconTableModel.RX);

					String lx=(String) lxTable.getValueAt(row, lxInt);
					String ge=(String) lxTable.getValueAt(row, geInt);
					String rx=(String) lxTable.getValueAt(row, rxInt);
					TierImpl mbTier=(TierImpl)lxViewer.getActiveAnnotation().getTier();
					String mbName=mbTier.getName();
					if (mbName.startsWith((String) ((LexiconViewer) lxViewer).getTierPreferences(LexiconViewer.ANNOTATION_TIER_PARSE))){
					//if (mbTier.getName()==(String) ((LexiconViewer) lxViewer).getTierPreferences(LexiconViewer.ANNOTATION_TIER_PARSE)){		// CC 310811
						if(lxViewer.getActiveAnnotation().getValue().equals(lx)){
							lexiconAnnotationPane.annotateTiers(mbTier, ge, rx);
						}
					}
				}
			}
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
	}

}
