package mpi.eudico.client.annotator.lexica.gui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.RowFilter;
import javax.swing.SwingUtilities;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;
import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.commands.NewAnnotationWithValueCommand;	// CC 310811
import mpi.eudico.client.annotator.commands.AnnotationValAfterCommand;	// CC 310811
import mpi.eudico.client.annotator.commands.DeleteSelectedAnnotationsCommand;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.lexica.InterlinearizationProcess;
import mpi.eudico.client.annotator.lexica.LexiconDocumentData;
import mpi.eudico.client.annotator.lexica.LexiconMorphemes2;
import mpi.eudico.client.annotator.lexica.LexiconParsingTableModel;
import mpi.eudico.client.annotator.lexica.LexiconRef;
import mpi.eudico.client.annotator.lexica.LexiconTable;
import mpi.eudico.client.annotator.lexica.LexiconTableContent;
import mpi.eudico.client.annotator.lexica.LexiconTableModel;
import mpi.eudico.client.annotator.lexica.MorphemeTableRenderer;
import mpi.eudico.client.annotator.lexica.ParseDocumentData;
import mpi.eudico.client.annotator.viewer.AbstractViewer;
import mpi.eudico.client.annotator.viewer.LexiconViewer;
import mpi.eudico.server.corpora.clom.Annotation;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.AbstractAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.type.Constraint;

/**
 * Contains the parsingTable that permit to choose between different parse possibilities 
 * when a user start interlinearize process 
 * @author Coralie
 * CC 31/03/2011 Table des segments utilise la police de la table du lexique
 */
public class LexiconSegmentationPanel extends JPanel implements ActionListener, MouseListener, ListSelectionListener{
	private static final List List = null;
	private TitledBorder border;
	private JPanel buttonPanel;
	private JButton fullLexicon;		// CC 260811 button to display all the lexicon
	private JButton interlinearize;
	private JButton autoInterlinearize;
	private JCheckBox parseFileCheckBox;
	
	private JTable parsingTable;
	private LexiconParsingTableModel parsingTableModel;

	private AbstractViewer viewer;
	private JLabel activeAnnotation=new JLabel();
	private LexiconTableModel lxTableModel;
	private LexiconTable lxTable;
	private Transcription transcription;		// CC 310811

	private LexiconSegmentationPanelPopUpMenu popup;

	private List<LexiconMorphemes2> parsedEntries;

	private List<Integer> filterIndexes;

	private String morphemetoAdd;
	
	private InterlinearizationProcess interlinearizeProcess;
	private Font font;
	private DeleteSelectedAnnotationsCommand comDel;		// CC 310811
	private NewAnnotationWithValueCommand comNewMb;			// CC 310811
	private NewAnnotationWithValueCommand comNewGe;			// CC 310811
	private NewAnnotationWithValueCommand comNewRx;			// CC 310811
	private AnnotationValAfterCommand comNewAfterMb;		// CC 310811
	private AnnotationValAfterCommand comNewAfterGe;		// CC 310811
	private AnnotationValAfterCommand comNewAfterRx;		// CC 310811
	/**
	 * LexiconPanel
	 * @param viewer
	 * @param model
	 */
	public LexiconSegmentationPanel(AbstractViewer viewer, LexiconTableModel model, LexiconTable lxTable, InterlinearizationProcess process) {
		this.lxTable=lxTable;
		this.viewer=viewer;
		this.lxTableModel=model;
		font=this.lxTable.getFont();
		interlinearizeProcess=process;
		parsedEntries=new ArrayList<LexiconMorphemes2>();
		filterIndexes=new ArrayList<Integer>();

		//border=new TitledBorder(ElanLocale.getString("Recognizer.SegmentationsPanel.Title"));
		//setBorder(border);
		setLayout(new BorderLayout());

		buttonPanel=new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));

	    fullLexicon=new JButton("Full Lexicon");  // CC 260811 button to see all the lexicon
	    fullLexicon.addActionListener(this);
	    buttonPanel.add(fullLexicon);		
	    JComboBox insertBox=new JComboBox();
		
		interlinearize=new JButton(ElanLocale.getString("LexiconViewer.Button.Interlinearize"));
		interlinearize.addActionListener(this);

		autoInterlinearize=new JButton(ElanLocale.getString("LexiconSegmentationPanel.JButton.AutoInterlinearize"));
		autoInterlinearize.addActionListener(this);

		buttonPanel.add(interlinearize);
		buttonPanel.add(autoInterlinearize);

		parseFileCheckBox=new JCheckBox(ElanLocale.getString("LexiconSegmentationPanel.JCheckBox.SearchInParse"));
		if(ParseDocumentData.DOCUMENT==null){
			parseFileCheckBox.setVisible(false);
		}
		else{
			parseFileCheckBox.setVisible(true);			
		}
		
		buttonPanel.add(parseFileCheckBox);
		buttonPanel.add(activeAnnotation);
		add(buttonPanel, BorderLayout.NORTH);

		parsingTable=new JTable();
		parsingTable.setShowHorizontalLines(false);
		parsingTable.setShowVerticalLines(false);
		parsingTable.setTableHeader(null);
		parsingTable.getSelectionModel().addListSelectionListener(this);
		parsingTable.addMouseListener(this);
		parsingTable.setBackground(getBackground());
		Font font=lxTable.getFont();  //CC 31/03/2011 utilise la police
		parsingTable.setFont(font);

		//can select row or cell
		parsingTable.setRowSelectionAllowed(true);
		parsingTableModel=new LexiconParsingTableModel();
		
		popup=new LexiconSegmentationPanelPopUpMenu(parsingTable, viewer);

		JScrollPane scrollPane = new JScrollPane(parsingTable);
		add(scrollPane, BorderLayout.CENTER);

		// this forces the minimal width of the selection panel
		add(Box.createHorizontalStrut(200), BorderLayout.SOUTH);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		activeAnnotation.setText("");
		filterIndexes.clear();
		lxTable.setAutoCreateRowSorter(true);
		parsedEntries.clear();
		parsingTableModel=new LexiconParsingTableModel();
		parsingTable.setModel(parsingTableModel);
		if (e.getSource()=="fullLexicon") {
			if(LexiconDocumentData.DOCUMENT==null){
				JOptionPane.showMessageDialog(this, "You should create or load lexicon by the lexicon menu", "Error", JOptionPane.ERROR_MESSAGE);
			}else{
				
			}
		}				
		initTableModel();
		if (e.getSource()==interlinearize) {
			if(viewer.getActiveAnnotation()!=null){
				if(checkHierarchy()){
					interlinearizeProcess.interlinearize(viewer, false);
					activeAnnotation.setText(" "+viewer.getActiveAnnotation().getValue());
					activeAnnotation.setFont(font);
					parsedEntries=interlinearizeProcess.getParsedEntries();
					if(!parsedEntries.isEmpty()){
						initTableModel();
						parsingTable.setRowSorter(null);
						parsingTable.setModel(parsingTableModel);
						parsingTable.setDefaultRenderer(Object.class, new MorphemeTableRenderer(viewer, parsingTableModel));
						if(parsingTable.getRowCount()>0&&parsingTable.getColumnCount()>0){
							parsingTable.setRowSelectionInterval(0, 0);
							parsingTable.setColumnSelectionInterval(0, 0);
						}
					}
					for(LexiconMorphemes2 m:parsedEntries){
						updateFilterIndexes(m);
					}
					filterLexiconTable();
				}
			}
		}
		else if(e.getSource()==autoInterlinearize){
			if(viewer.getActiveAnnotation()!=null){
				if(checkHierarchy()){
					boolean parsePreference=false;
					if(parseFileCheckBox.isSelected()&&ParseDocumentData.DOCUMENT!=null){
						parsePreference=true;
					}
					interlinearizeProcess.autoInterlinearize(viewer, parsePreference);
					activeAnnotation.setText(" "+viewer.getActiveAnnotation().getValue());
					parsedEntries=interlinearizeProcess.getParsedEntries();
					if(!parsedEntries.isEmpty()){
						initTableModel();
						parsingTable.setRowSorter(null);
						parsingTable.setModel(parsingTableModel);
						parsingTable.setDefaultRenderer(Object.class, new MorphemeTableRenderer(viewer, parsingTableModel));
						if(parsingTable.getRowCount()>0&&parsingTable.getColumnCount()>0){
							parsingTable.setRowSelectionInterval(0, 0);
							parsingTable.setColumnSelectionInterval(0, 0);
						}
					}
					for(LexiconMorphemes2 m:parsedEntries){
						updateFilterIndexes(m);
					}
					filterLexiconTable();
				}
			}
		}
		repaint();
	}

	/**Check if the hierarchy is correct to do the interlinearization correctly
	 * */
	private boolean checkHierarchy(){
		Annotation ann=viewer.getActiveAnnotation();
		Transcription transcription=((LexiconViewer)viewer).getTranscription();
		AnnotationTierHierarchyInfos infos=new AnnotationTierHierarchyInfos(viewer, transcription);
		
		String prefTierWord=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_WORD);
		String prefTierParse=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_PARSE);
		String prefTierGloss=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_GLOSS);
		String prefTierPos=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_POS);
		
		
		boolean correctHierarchy=false;
		boolean correctParseLingType=true;
		boolean correctGlossLingType=true;
		boolean correctPosLingType=true;
		
		//if(ann.getTier().getName().startsWith(prefTierWord)){
		String test=ann.getTier().getName();
		if(test.equals(prefTierWord)){
			TierImpl wordTier=(TierImpl)ann.getTier();
			infos.setInterlWordTier(wordTier);
			Vector<TierImpl> wordTierChildren=wordTier.getChildTiers();
			for(TierImpl parseTier:wordTierChildren){
				//if(parseTier.getName().startsWith(prefTierParse)){
				if(parseTier.getName().equals(prefTierParse)){
					if(parseTier.getLinguisticType().getConstraints()!=null){
						if(parseTier.getLinguisticType().getConstraints().getStereoType()!=Constraint.SYMBOLIC_SUBDIVISION){
							correctParseLingType=false;
						}
					}else{
						correctParseLingType=false;
					}
					infos.setInterlParseTier(parseTier);
					Vector<TierImpl> parseTierChildren=parseTier.getChildTiers();

					TierImpl glossTier=null;
					TierImpl posTier=null;
					for(TierImpl annTier:parseTierChildren){
						//if(annTier.getName().startsWith(prefTierGloss)){
						if(annTier.getName().equals(prefTierGloss)){
							glossTier=annTier;
							if(glossTier.getLinguisticType().getConstraints()!=null){
								if(glossTier.getLinguisticType().getConstraints().getStereoType()!=Constraint.SYMBOLIC_ASSOCIATION){
									correctGlossLingType=false;
								}
							}
							else{
								correctGlossLingType=false;
							}
							infos.setInterlGlossTier(glossTier);
						//} else if(annTier.getName().startsWith(prefTierPos)){
						} else if(annTier.getName().equals(prefTierPos)){
							posTier=annTier;
							if(posTier.getLinguisticType().getConstraints()!=null){
								if(posTier.getLinguisticType().getConstraints().getStereoType()!=Constraint.SYMBOLIC_ASSOCIATION){
									correctPosLingType=false;
								}
							}
							else{
								correctPosLingType=false;
							}
							infos.setInterlPosTier(posTier);
						}
					}
					if(glossTier!=null&&posTier!=null){
						correctHierarchy=true;
					}
				}
			}
		}
		infos.setCorrectParseLingType(correctParseLingType);
		infos.setCorrectGlossLingType(correctGlossLingType);
		infos.setCorrectPosLingType(correctPosLingType);
		infos.setCorrectHierarchy(correctHierarchy);
		
		if(correctHierarchy&&correctParseLingType&&correctGlossLingType&&correctPosLingType){
			return true;
		}
		else{
			new WrongInterlinearizationParametersFrame(viewer, infos);
			return false;
		}
	}
	
	/**
	 * initialize the table that contains the parsing possibilities
	 */
	public void initTableModel(){			// CC 250811
		int columnCount=0;
		for(LexiconMorphemes2 data:parsedEntries){
			if(columnCount<data.getSize()){
				columnCount=data.getSize();
			}
		}
		parsingTableModel.setColumnCount(columnCount);
		for(LexiconMorphemes2 data:parsedEntries){
			//System.out.println(data.getMorphemes());
			parsingTableModel.addRow(data.toArray());
		}
	}

	/**
	 * update lexicon indexes
	 * @param morphemes
	 */
	private void updateFilterIndexes(LexiconMorphemes2 morphemes){
		for(int i=0; i<morphemes.getRoots().size();i++){
			for(LexiconRef ref: morphemes.getRoots()){
				for(LexiconTableContent tableContent:ref.getLexicons()){
					filterIndexes.add(tableContent.getGridIndex());
				}
			}
		}
		for(int i=0; i<morphemes.getSuffixes().size();i++){
			for(LexiconRef ref: morphemes.getSuffixes()){
				for(LexiconTableContent tableContent:ref.getLexicons()){
					filterIndexes.add(tableContent.getGridIndex());
				}
			}
		}
		for(int j=0; j<morphemes.getPrefixes().size();j++){
			for(LexiconRef ref: morphemes.getPrefixes()){
				for(LexiconTableContent tableContent:ref.getLexicons()){
					filterIndexes.add(tableContent.getGridIndex());
				}
			}
		}			
	}

	/**
	 * Filter Lexicon table
	 */
	public void filterLexiconTable(){
		TableRowSorter<TableModel> sorter=new TableRowSorter<TableModel>(lxTableModel);
		lxTable.setRowSorter(sorter);

		RowFilter<Object,Object> rowFilter = new RowFilter<Object,Object>() {
			@Override
			public boolean include(Entry<? extends Object, ? extends Object> entry) {
				for(int i=0; i<entry.getValueCount();){
					for(int l=0; l<filterIndexes.size(); l++){
						if(entry.getIdentifier().equals(filterIndexes.get(l))) {
							return true;
						}
					}
					return false;
				}
				return false;
			}
		};
		sorter.setRowFilter(rowFilter);
	}

	/**
	 * Hide all rows but the parameter row in the Parsing table
	 */
	private void filterParsingTable(final int row){
		LexiconMorphemes2 morphemes=parsedEntries.get(row);
		parsedEntries.clear();
		parsedEntries.add(morphemes);
		if(!parsedEntries.isEmpty()){
			parsingTableModel=new LexiconParsingTableModel();
			initTableModel();
			parsingTable.setRowSorter(null);
			parsingTable.setModel(parsingTableModel);
			parsingTable.setDefaultRenderer(Object.class, new MorphemeTableRenderer(viewer, parsingTableModel));
		}
		parsingTable.setRowSelectionInterval(0, 0);
	}

	private void updateLexiconRowFilter(int index){
		if(parsingTable.getSelectedRow()!=-1){
			LexiconMorphemes2 morphemes=parsedEntries.get(index);
			filterIndexes.clear();
			updateFilterIndexes(morphemes);
			filterLexiconTable();
		}
	}


	/**
	 * associate the specific lexicon morpheme in the list to the parsed morpheme 
	 * @param selectedParsingRow row index
	 * @param selectedParsingColumn column index
	 * @param tableValue morpheme values
	 * @param associationDoneNumber count how many morphemes in the parsing table are associated to a lexicon morpheme
	 */
	public void associateMorpheme(int selectedParsingRow, int selectedParsingColumn, String[] tableValue, int associationDoneNumber){
		if(parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn)!=null){
			if(associationDoneNumber==0){
				filterParsingTable(selectedParsingRow);
			}
			parsingTable.setValueAt(tableValue, parsingTable.getSelectedRow(), selectedParsingColumn);
			//select the next morpheme
			if(selectedParsingColumn<parsingTable.getColumnCount()-1){
				parsingTable.setColumnSelectionInterval(selectedParsingColumn+1, selectedParsingColumn+1);
			}
		}

		updateLexiconRowFilter(parsingTable.getSelectedRow());
		boolean isComplete=true;
		for(int i=0; i<parsingTable.getColumnCount(); i++){
			if(parsingTable.getValueAt(parsingTable.getSelectedRow(), i) instanceof String[]){
			}
			else{
				if(parsingTable.getValueAt(parsingTable.getSelectedRow(), i)!=null){
					isComplete=false;
					break;
				}
			}
		}

		//if each cell is a String array
		if(isComplete){
			annotateTiers(associationDoneNumber+1, selectedParsingRow);
			updateLexiconRowFilter(parsingTable.getSelectedRow());
		}
	}
	

	/**
	 * annotate tier mb, ge and rx
	 * @param parseAnnotation
	 * @param selectedParsingRow
	 */
	private void annotateTiers(int parseAnnotation, int selectedParsingRow){
		//long beginTime=viewer.getActiveAnnotation().getBeginTimeBoundary();
		Annotation annMot=viewer.getActiveAnnotation();
		Transcription transcription=((LexiconViewer)viewer).getTranscription();
		boolean mbOK=false;
		boolean geOK=false;
		boolean rxOK=false;
		
		//Undoable deleting of (pre-existing)) mb annotations
		ArrayList<Annotation> annDeleteList = new ArrayList<Annotation>(); 
		TierImpl tierMot=(TierImpl) annMot.getTier();	// mot tier
		Vector childTiers = tierMot.getChildTiers();	// mot children
			
		int numCh=childTiers.size();
		if (numCh==0) {
			return;
		}
		TierImpl ct=null;
		TierImpl mbTier=null;
		TierImpl geTier=null;
		TierImpl rxTier=null;
		// control 
		for(int j=0; j<numCh; j++) {   // mb, ge, rx
			ct=(TierImpl) childTiers.get(j);
			//if(ct.getName().startsWith((String) ((LexiconViewer) viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_PARSE))){
			if(ct.getName().equals(((LexiconViewer)viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_PARSE))){		// CC 300811
				mbOK=true;
				mbTier=(TierImpl)childTiers.get(j);		// mb
				Vector mbChildren=mbTier.getChildTiers();
				if(mbChildren.size()==2){
					for(int k=0; k<mbChildren.size(); k++){
						TierImpl currentMbChildren=(TierImpl)mbChildren.get(k);
						//if(currentMbChildren.getName().startsWith((String) ((LexiconViewer) viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_GLOSS))){
						if(currentMbChildren.getName().equals(((LexiconViewer) viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_GLOSS))){
							geTier=currentMbChildren;
							geOK=true;
						}
						//else if(currentMbChildren.getName().startsWith((String) ((LexiconViewer) viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_POS))){
						else if(currentMbChildren.getName().equals(((LexiconViewer) viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_POS))){
							rxTier=currentMbChildren;
							rxOK=true;
						}
					}
				}
			}
		}
		if (mbOK && geOK && rxOK){
			Vector<Annotation> annotations=annMot.getChildrenOnTier(mbTier);		
			for(Annotation annotation:annotations){
				annDeleteList.add(annotation);			// CC 310811 build a list of MB annotations to be deleted
			}
			comDel = (DeleteSelectedAnnotationsCommand) ELANCommandFactory.createCommand(transcription,
	                ELANCommandFactory.DELETE_MULTIPLE_ANNOS);
			Object receiver = transcription;		
			Object[] args = new Object[1];
			args[0]=annDeleteList;
			comDel.execute(receiver,args);
			
			if(ct==null||mbTier==null||geTier==null||rxTier==null){
				System.out.println("Problem in the hierarchy !");
			}

		int transMorphBreakChar=(Integer)((LexiconViewer) viewer).getMorphemeBreakPreferences(LexiconViewer.TRANS_BREAK_CHAR);

		Annotation currentMbAnn=null;
		Annotation currentGeAnn=null;
		Annotation currentRxAnn=null;
 		long BT=annMot.getBeginTimeBoundary();
 		long ET=annMot.getEndTimeBoundary();
 
		Vector motChildren;		
		// insert all the parse annotation mb, ge, rx

    	for(int i=0; i<parseAnnotation; i++){
			Object[] annAdd = new Object[3];
			Object[] annAddAfter = new Object[2];
			
			String[] data=(String[])parsingTable.getValueAt(selectedParsingRow, i);
			if(data.length==4){
				String mb=""+data[1];
				String ge=""+data[2];
				String rx=""+data[3];
				
				// transmission of morphemeBreakCharacter to ge and rx
				if(transMorphBreakChar==1){	
					if(InterlinearizationProcess.endingPattern.matcher(mb).matches()){
						char endChar=mb.charAt(0);
						ge=endChar+ge;
						rx=endChar+rx;
					} else if(InterlinearizationProcess.beginningPattern.matcher(mb).matches()){
						char beginningChar=mb.charAt(mb.length()-1);
						ge=ge+beginningChar;
						rx=rx+beginningChar;
					}
				}
				
				if(i>0){	// Other segments				
					motChildren=annMot.getChildrenOnTier(mbTier);
					Annotation lastAnnMb = (AbstractAnnotation) motChildren.lastElement();
					annAddAfter[0]=lastAnnMb;
					annAddAfter[1] = mb;
					comNewAfterMb=(AnnotationValAfterCommand) ELANCommandFactory.createCommand(transcription,
			                ELANCommandFactory.NEW_ANNOTATION_VAL_AFTER);
					comNewAfterMb.execute(mbTier,annAddAfter);
					//the beginTime of the new created annotation
					motChildren=annMot.getChildrenOnTier(mbTier);
					lastAnnMb = (AbstractAnnotation) motChildren.lastElement();
					annAdd[0]=lastAnnMb.getBeginTimeBoundary();
					annAdd[1]=annAdd[0];

				}
				else{		// First segment
					annAdd[0]=BT;
					annAdd[1]=BT;
					annAdd[2]=mb;
					comNewMb= (NewAnnotationWithValueCommand) ELANCommandFactory.createCommand(transcription,
			            ELANCommandFactory.NEW_ANNOTATION_VAL);
					comNewMb.execute(mbTier,annAdd);
				}
				annAdd[2]=ge;
				comNewGe= (NewAnnotationWithValueCommand) ELANCommandFactory.createCommand(transcription,
		                ELANCommandFactory.NEW_ANNOTATION_VAL);
				comNewGe.execute(geTier,annAdd);
				annAdd[2]=rx;
				comNewRx= (NewAnnotationWithValueCommand) ELANCommandFactory.createCommand(transcription,
		                ELANCommandFactory.NEW_ANNOTATION_VAL);	
				comNewRx.execute(rxTier,annAdd);
			}
		}
			viewer.setActiveAnnotation(tierMot.getAnnotationAtTime(ET));
			removeSegmentationInformation();
		} // end of mbOK && geOK && rxOK
	}
	
	/**
	 * Remove all segmentations possibility when the segmentation is done 
	 */
	private void removeSegmentationInformation(){
		filterIndexes.clear();
		lxTable.setRowSorter(null);
		parsedEntries.clear();
		parsingTableModel=new LexiconParsingTableModel();
		//tableData.clear();
		initTableModel();
		parsingTable.setModel(parsingTableModel);
		parsingTable.setDefaultRenderer(Object.class, new MorphemeTableRenderer(viewer, parsingTableModel));
		activeAnnotation.setText("");
		repaint();
	}

	@Override
	public void mousePressed(MouseEvent e) {
		int selectedParsingRow=parsingTable.getSelectedRow();
		int selectedParsingColumn=parsingTable.getSelectedColumn();
		if(e.getSource()==parsingTable){
			//if is right click
			if (SwingUtilities.isRightMouseButton(e) || e.isPopupTrigger()) {
				if(LexiconDocumentData.DOCUMENT!=null){
					if(parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn)!=null){
						morphemetoAdd="";
						if(parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn) instanceof LexiconRef){
							morphemetoAdd=((LexiconRef)parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn)).getMorphemeForm();
						}
						else{
							if(parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn) instanceof String[]){
								morphemetoAdd=((String[])parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn))[0];
							}
						}
						popup.setMorpheme(morphemetoAdd);
						popup.show(parsingTable, e.getPoint().x, e.getPoint().y);
					}
				}
			}
			if (e.getClickCount()>=2){
				if(parsingTable.getSelectedRow()!=-1){
					if(parsingTable.getValueAt(parsingTable.getSelectedRow(), parsingTable.getSelectedColumn())!=null){
						LexiconMorphemes2 morphemes=parsedEntries.get(parsingTable.getSelectedRow());
						LexiconRef lexicon=morphemes.getMorpheme(parsingTable.getSelectedColumn());
						filterIndexes.clear();
						for(LexiconTableContent content:lexicon.getLexicons()){
							filterIndexes.add(content.getGridIndex());
						}
						filterLexiconTable();
					}
				}
			}
			else{
				updateLexiconRowFilter(parsingTable.getSelectedRow());
			}
		}
/*		//click on the lexiconTable
		else{
			if (e.getClickCount()>=2){
				if(parsingTable.getSelectedRow()!=-1){
					int parseAnnotation=0;

					for(int i=0; i<parsingTable.getColumnCount(); i++){
						if(parsingTable.getValueAt(selectedParsingRow, i) instanceof String[]){
							parseAnnotation++;
						}
					}

//					parsingIndex
					int row=lxTable.getSelectedRow();
					int lxInt=lxTableModel.findColumn(lxTableModel.LX);
					int geInt=lxTableModel.findColumn(lxTableModel.GE);
					int rxInt=lxTableModel.findColumn(lxTableModel.RX);
					int aInt=lxTableModel.findColumn(lxTableModel.A);

					String lx=(String) lxTable.getValueAt(row, lxInt);
					String ge=(String) lxTable.getValueAt(row, geInt);
					String rx=(String) lxTable.getValueAt(row, rxInt);
					String a=(String) lxTable.getValueAt(row, aInt);
					String[] parsingA=a.split("; ");

					String morphemeForm="";
					if(parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn) instanceof LexiconRef){
						morphemeForm=((LexiconRef)parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn)).getMorphemeForm();
					}
					else{
						if(parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn) instanceof String[]){
							morphemeForm=((String[])parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn))[0];
						}
					}

					String[] tableValue={morphemeForm, lx, ge, rx};
					if(parsingTable.getValueAt(selectedParsingRow, selectedParsingColumn)!=null){
						//check if the value equals an lx or a form
						if(morphemeForm.equals(lx)){
							associateMorpheme(selectedParsingRow, selectedParsingColumn, tableValue, parseAnnotation);
						}
						else{
							for(int i=0; i<parsingA.length; i++){
								if(morphemeForm.equals(parsingA[i])){
									associateMorpheme(selectedParsingRow, selectedParsingColumn, tableValue, parseAnnotation);
								}
							}
						}
					}
				}
			}
		}*/
	}

	@Override
	public void valueChanged(ListSelectionEvent arg0) {
		updateLexiconRowFilter(parsingTable.getSelectedRow());
	}

	@Override
	public void mouseClicked(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public JTable getParsingTable() {
		return parsingTable;
	}
	
//	public List<LexiconMorphemes2> getParsedEntries(){
//		return parsedEntries;
//	}
//	
//	public List<Integer> getFilterIndexes(){
//		return filterIndexes;
//	}
//	
//	public LexiconSegmentationPanelPopUpMenu getPopup(){
//		return popup;
//	}
}