package mpi.eudico.client.annotator.lexica.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import mpi.eudico.client.annotator.viewer.LexiconViewer;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;

/**
 * Frame to parameter the morpheme break character
 * By default the morpheme break character is an hyphen
 * Configure if the hyphen have to be transmitted from the morpheme to the gloss and the tier X
 * @author Coralie
 *
 */
public class MorphemeBreakCharacterFrame extends JFrame implements ActionListener {
	private LexiconViewer viewer;
	private TranscriptionImpl transcription;
	//private JLabel cliticCharLb;
	private JTextField cliticCharTxt;
	//private JTextArea cliticCharTxtArea;
	//private JLabel stemCharLb;
	//private JTextArea stemCharTxtArea;
	private JTextField stemCharTxt;
	private JCheckBox transMorphemeBreakChar;
	private JButton cancelButton;
	private JButton okButton;
	
//	private String suffixMorphBreak;
//	private String stemMorphBreak;
//	private Integer transmitMorphBreak;
//	
//	static public final String SUFFIX_CHAR="MorphemeBreakCharacter.Suffix";
//	static public final String STEM_CHAR="MorphemeBreakCharacter.Stem";
//	static public final String TRANS_BREAK_CHAR="MorphemeBreakCharacter.Transmit";
//	
//	static public final String DEFAULT_SUFFIX_VALUE="-=";
//	static public final String DEFAULT_STEM_VALUE="_";
//	static public final Integer DEFAULT_TRANSMIT_BREAK_CHAR=0;
	
	public MorphemeBreakCharacterFrame(LexiconViewer viewer, TranscriptionImpl transcription){
		this.viewer=viewer;
		this.transcription=transcription;
		setTitle("Parameter Morpheme break characters");
		init();
		this.setVisible(true);
		pack();
	}
	
	private void init(){
		//initPreferences();
		
		setLayout(new GridBagLayout());
		GridBagConstraints gbc=new GridBagConstraints();
		Insets insets= new Insets(0, 3, 2, 2);
		gbc.insets=insets;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.gridx=0;
		gbc.gridy=0;
		JLabel cliticCharLb=new JLabel("Suffix, prefix, clitic... character(s) :");
		add(cliticCharLb, gbc);
		gbc.gridx=1;
		gbc.gridy=0;
		//cliticCharTxtArea=new JTextArea(2, 3);
		cliticCharTxt=new JTextField(4);
		cliticCharTxt.setText(getCliticCharElmts());
		add(cliticCharTxt, gbc);
		gbc.gridx=0;
		gbc.gridy=1;
		JLabel stemCharLb=new JLabel("Stem character(s) :");
		add(stemCharLb, gbc);
		gbc.gridx=1;
		gbc.gridy=1;
		//stemCharTxtArea=new JTextArea(2, 3);
		stemCharTxt=new JTextField(4);
		stemCharTxt.setText(getStemCharElmts());
		add(stemCharTxt, gbc);
		gbc.gridwidth=2;
		gbc.gridx=0;
		gbc.gridy=2;
		transMorphemeBreakChar=new JCheckBox("transmit morpheme break character to annotation tiers");
		transMorphemeBreakChar.setSelected(isMorphemeTransmetted());
		add(transMorphemeBreakChar, gbc);
		gbc.gridwidth=2;
		gbc.gridx=0;
		gbc.gridy=3;
		JPanel buttonPane=new JPanel();
		cancelButton=new JButton("Cancel");
		cancelButton.addActionListener(this);
		buttonPane.add(cancelButton);
		okButton=new JButton("OK");
		okButton.addActionListener(this);
		buttonPane.add(okButton);
		add(buttonPane, gbc);
		
	}
	
//	private void initPreferences(){
//		suffixMorphBreak=(String)viewer.getMorphemeBreakPreferences(SUFFIX_CHAR);
//		if(suffixMorphBreak==null){
//			suffixMorphBreak=DEFAULT_SUFFIX_VALUE;
//			viewer.setMorphemeBreakPreferences(SUFFIX_CHAR, suffixMorphBreak);
//		}
//		stemMorphBreak=(String)viewer.getMorphemeBreakPreferences(STEM_CHAR);
//		if(stemMorphBreak==null){
//			stemMorphBreak=DEFAULT_STEM_VALUE;
//			viewer.setMorphemeBreakPreferences(DEFAULT_STEM_VALUE, stemMorphBreak);
//		}
//		transmitMorphBreak=(Integer)viewer.getMorphemeBreakPreferences(TRANS_BREAK_CHAR);
//		if(transmitMorphBreak==null){
//			transmitMorphBreak=DEFAULT_TRANSMIT_BREAK_CHAR;
//			viewer.setMorphemeBreakPreferences(TRANS_BREAK_CHAR, transmitMorphBreak);
//		}
//	}
	
	private boolean isMorphemeTransmetted(){
		if((Integer)viewer.getMorphemeBreakPreferences(LexiconViewer.TRANS_BREAK_CHAR)==1){
			return true;
		} else{
			return false;			
		}
	}
	
	private String getCliticCharElmts(){
		return (String) viewer.getMorphemeBreakPreferences(LexiconViewer.SUFFIX_CHAR);
	}

	private String getStemCharElmts(){
		return (String) viewer.getMorphemeBreakPreferences(LexiconViewer.STEM_CHAR);
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==cancelButton){
			setVisible(false);
			dispose();
		} else if(e.getSource()==okButton){
			viewer.setMorphemeBreakPreferences(LexiconViewer.SUFFIX_CHAR, cliticCharTxt.getText());
			viewer.setMorphemeBreakPreferences(LexiconViewer.STEM_CHAR, stemCharTxt.getText());
			if(transMorphemeBreakChar.isSelected()){
				viewer.setMorphemeBreakPreferences(LexiconViewer.TRANS_BREAK_CHAR, 1);				
			}
			else{
				viewer.setMorphemeBreakPreferences(LexiconViewer.TRANS_BREAK_CHAR, 0);
			}
			viewer.initializeInterlinearizeProcess();
			setVisible(false);
			dispose();
		}		
	}
}
