package mpi.eudico.client.annotator.lexica.gui;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Vector;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTextField;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.viewer.LexiconViewer;
import mpi.eudico.server.corpora.clom.Tier;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.server.corpora.clomimpl.type.Constraint;
import mpi.eudico.server.corpora.clomimpl.type.LinguisticType;
import mpi.eudico.client.annotator.commands.AddTierCommand;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;

/**
 * Create Tiers for annotation
 * @author Coralie
 *
 */
public class TierAnnotationParameterFrame extends TierParameterFrame{
    private AddTierCommand comGe;		// CC 300811
    private AddTierCommand comRx;		// CC 300811
    
	public TierAnnotationParameterFrame(LexiconViewer viewer, TranscriptionImpl transcription, HashMap<String, String> map){
		super(viewer, transcription, map);
	}
	
	@Override
	void initHashMapElements() {
		mbValue=map.get(LexiconViewer.ANNOTATION_PARSE_TIER_NAME);
		geValue=map.get(LexiconViewer.ANNOTATION_GLOSS_TIER_NAME);
		rxValue=map.get(LexiconViewer.ANNOTATION_POS_TIER_NAME);
	}
	@Override
	void updateLocale() {
		setTitle(ElanLocale.getString("TierParameterFrame.Annotation.Title.Parameter"));
		chooseTierLabel.setText(ElanLocale.getString("TierParameterFrame.Annotation.JLabel.Choose"));
	}
	
	@Override
	void updateLocaleParameterPane() {
		Vector<LinguisticType> ltypes=transcription.getLinguisticTypes();
		List<String> linguisticTypeNamesSubdiv=new ArrayList<String>();
		List<String> linguisticTypeNamesAssoc=new ArrayList<String>();
		for(LinguisticType lt: ltypes){
			Constraint cs=lt.getConstraints();
			if(cs!=null){
				if(cs.getStereoType()==Constraint.SYMBOLIC_SUBDIVISION){
					linguisticTypeNamesSubdiv.add(lt.getLinguisticTypeName());
				} else if(cs.getStereoType()==Constraint.SYMBOLIC_ASSOCIATION){
					linguisticTypeNamesAssoc.add(lt.getLinguisticTypeName());
				}
			}
		}
		createTierTitle.setTitle(ElanLocale.getString("TierParameterFrame.Annotation.Title.Create"));		
		if(tierLbList.size()==tierTxtFdList.size()&&tierTxtFdList.size()==lingTypeCbBoxList.size()){
			for(int i=0; i<tierLbList.size(); i++){
				JLabel tierLabel=tierLbList.get(i);
				JTextField tierTextField=tierTxtFdList.get(i);
				JComboBox lingTypeCbBox=lingTypeCbBoxList.get(i);
				//ge
				if(i==0){
					tierLabel.setText(ElanLocale.getString("TierParameterFrame.Label.Tier.Gloss"));
					tierTextField.setText(geValue);
					for(String assocType:linguisticTypeNamesAssoc){
						lingTypeCbBox.addItem(assocType);						
					}
					if(transcription.getLinguisticTypeByName("ge")!=null){
						lingTypeCbBox.setSelectedItem("ge");
					}
				}
				//rx
				else if(i==1){
					tierLabel.setText(ElanLocale.getString("TierParameterFrame.Label.Tier.Pos"));
					tierTextField.setText(rxValue);
					for(String assocType:linguisticTypeNamesAssoc){
						lingTypeCbBox.addItem(assocType);						
					}
					if(transcription.getLinguisticTypeByName("rx")!=null){
						lingTypeCbBox.setSelectedItem("rx");
					}
				}
			}
		}
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==chooseTierButton){
			chooseTierButton.setEnabled(false);
			chooseTierCbBox.setEnabled(false);
			initParameterPane();
			updateLocaleParameterPane();
			
			getRootPane().repaint();
			pack();
		}
		else if (e.getSource()==createTiers){
			String geTierName="";
			String rxTierName="";
			String geLtName="";
			String rxLtName="";
			for(int i=0; i<tierTxtFdList.size(); i++){
				JTextField textField=tierTxtFdList.get(i);
				JComboBox ltCbBox=lingTypeCbBoxList.get(i);
				//ge
				if(i==0){
					if(!textField.getText().isEmpty()){
						geTierName=textField.getText();
					}
					geLtName=(String)ltCbBox.getSelectedItem();
				}
				//rx
				else if(i==1){
					if(!textField.getText().isEmpty()){
						rxTierName=textField.getText();
					}
					rxLtName=(String)ltCbBox.getSelectedItem();
				} 
			}
			String mbTierName=(String)chooseTierCbBox.getSelectedItem();
			viewer.setTierPreferences(LexiconViewer.ANNOTATION_TIER_PARSE, mbTierName);
			viewer.setTierPreferences(LexiconViewer.ANNOTATION_TIER_GLOSS, geTierName);
			viewer.setTierPreferences(LexiconViewer.ANNOTATION_TIER_POS, rxTierName);
			if(!mbTierName.isEmpty()&&!geTierName.isEmpty()&&!rxTierName.isEmpty()){
				Vector<Tier> tierVector=transcription.getTiers();
				Vector<Tier> copyTierVector=new Vector<Tier>(tierVector);
				if (copyTierVector != null) {
					ListIterator<Tier> it=copyTierVector.listIterator();
					while(it.hasNext()){
						TierImpl tierImpl=(TierImpl) it.next();
						//if(tierImpl.getName().startsWith(mbTierName)){
						if(tierImpl.getName()==mbTierName){		// CC 300811
							TierImpl geTier=new TierImpl(tierImpl, geTierName, null, transcription, null);
							/*
							geTier.setLinguisticType(transcription.getLinguisticTypeByName(geLtName));
							geTier.setParticipant(tierImpl.getParticipant());
							geTier.setAnnotator(tierImpl.getAnnotator());
							geTier.setDefaultLocale(tierImpl.getDefaultLocale());
							transcription.addTier(geTier);
							*/
							//CC 300811 For Undo 
							comGe = (AddTierCommand) ELANCommandFactory.createCommand(transcription,
			                        ELANCommandFactory.ADD_TIER);
							TierImpl mbTier = (TierImpl) (transcription.getTierWithId(mbTierName));
							Object receiver = transcription;		
							Object[] args = new Object[6];
							args[0]=geTierName;
							args[1]=mbTier;
							args[2]=geLtName;
							args[3]=tierImpl.getParticipant();
							args[4]=tierImpl.getAnnotator();
							args[5]=tierImpl.getDefaultLocale();
							comGe.execute(transcription,args);
								
							TierImpl rxTier=new TierImpl(tierImpl, rxTierName, null, transcription, null);
							/*
							rxTier.setLinguisticType(transcription.getLinguisticTypeByName(rxLtName));
							rxTier.setParticipant(tierImpl.getParticipant());
							rxTier.setAnnotator(tierImpl.getAnnotator());
							rxTier.setDefaultLocale(tierImpl.getDefaultLocale());
							transcription.addTier(rxTier);
							*/
							comRx = (AddTierCommand) ELANCommandFactory.createCommand(transcription,
			                        ELANCommandFactory.ADD_TIER);	// CC 290811
						    args[0]=rxTierName;						
							args[1]=mbTier;
							args[2]=rxLtName;
							args[3]=tierImpl.getParticipant();
							args[4]=tierImpl.getAnnotator();
							args[5]=tierImpl.getDefaultLocale();
							comRx.execute(transcription,args);
						}
					}
					this.setVisible(false);
					this.dispose();
				}
			}
		}
	}
}
