package mpi.eudico.client.annotator.lexica.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.commands.AddTierCommand;	// CC 290811
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.viewer.LexiconViewer;
import mpi.eudico.server.corpora.clom.Tier;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.server.corpora.clomimpl.type.Constraint;
import mpi.eudico.server.corpora.clomimpl.type.LinguisticType;

/**
 * Create tiers for the interlinearization
 * @author Coralie
 *
 */
public class TierInterlinearizeParameterFrame extends JFrame implements ActionListener{
	private LexiconViewer viewer;
	private TranscriptionImpl transcription;
	private JComboBox interlinearizeTierCbBox;
	private JButton interlinearizeTierButton;
	private JPanel tierNamePanel;
	
	private JTextField segmentationTxtField;
	private JTextField glossTxtField;
	private JTextField tierXTxtField;
	private JButton createTiers;
	private String motValue;
	private String mbValue;
	private String geValue;
	private String rxValue;
	private JComboBox ltMbComboBox;
	private JComboBox ltGeComboBox;
	private JComboBox ltRxComboBox;
    private AddTierCommand comMb;		// CC 290811
    private AddTierCommand comGe;		// CC 290811
    private AddTierCommand comRx;		// CC 290811
  
	public TierInterlinearizeParameterFrame(LexiconViewer viewer, TranscriptionImpl transcription, HashMap<String, String> map){
		this.viewer=viewer;
		this.transcription=transcription;
		motValue=map.get(LexiconViewer.INTERLINEARIZE_WORD_TIER_NAME);
		mbValue=map.get(LexiconViewer.INTERLINEARIZE_PARSE_TIER_NAME);
		geValue=map.get(LexiconViewer.INTERLINEARIZE_GLOSS_TIER_NAME);
		rxValue=map.get(LexiconViewer.INTERLINEARIZE_POS_TIER_NAME);
		setTitle("Tier Parameter");
		init();
		this.setVisible(true);
		pack();
	}
	
	private void init(){
		setLayout(new GridBagLayout());
		
		//interlinearize tier
		GridBagConstraints gbc=new GridBagConstraints();
		Insets insets= new Insets(0, 3, 2, 2);
		gbc.insets=insets;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.gridx=0;
		gbc.gridy=0;
		JLabel interlinearizeLabel=new JLabel("choose interlinearize tier : ");
		add(interlinearizeLabel, gbc);
		interlinearizeTierCbBox=new JComboBox();
		Vector<TierImpl> tierVector=transcription.getTiers();
		for(TierImpl tierImpl:tierVector){
			interlinearizeTierCbBox.addItem(tierImpl.getName()/*.replaceAll("@"+tierImpl.getParticipant(), "")*/);			
		}
		gbc.gridx=1;
		add(interlinearizeTierCbBox, gbc);
		
		gbc.gridx=2;
		interlinearizeTierButton=new JButton("OK");
		interlinearizeTierButton.addActionListener(this);
		add(interlinearizeTierButton, gbc);
		
		gbc.gridy=1;
		gbc.gridx=0;
		gbc.gridwidth=3;
		tierNamePanel=new JPanel();
		tierNamePanel.setLayout(new GridBagLayout());
		add(tierNamePanel, gbc);
	}
	
	private boolean hasCorrectTierHierarchy(){
		boolean hasCorrectTierHierarchy=false;
		String motTierName=(String) interlinearizeTierCbBox.getSelectedItem();
		List<TierImpl> motTierImplList=new ArrayList<TierImpl>();
		Vector<TierImpl> tierVector=transcription.getTiers();
		for(TierImpl tierImpl:tierVector){
			if(tierImpl.getName().startsWith(motTierName)){
				motTierImplList.add(tierImpl);
			}
		}
		
		//get the linked file
//		Vector<LinkedFileDescriptor> lfds=transcription.getLinkedFileDescriptors();
//		for(LinkedFileDescriptor lfd:lfds){
//			System.out.println(lfd.relativeLinkURL);
//		}
		
//		List<TierImpl> tierImplList=new ArrayList<TierImpl>();
		for(TierImpl motTier:motTierImplList){
			if(!motTier.getChildTiers().isEmpty()){
				Vector<TierImpl> mbTierList=motTier.getChildTiers();
				for(TierImpl mbTier: mbTierList){
					if(mbTier.getLinguisticType().getConstraints().getStereoType()==Constraint.SYMBOLIC_SUBDIVISION){
						if(!mbTier.getChildTiers().isEmpty()){
							Vector<TierImpl> geTierList=mbTier.getChildTiers();
							for(TierImpl geTier:geTierList){
								if(geTier.getLinguisticType().getConstraints().getStereoType()==Constraint.SYMBOLIC_ASSOCIATION){
									
								}
							}
						}

					}
				}
			}
		}
		return hasCorrectTierHierarchy;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==interlinearizeTierButton){
			//if(!hasCorrectTierHierarchy()){
				Vector<LinguisticType> ltypes=transcription.getLinguisticTypes();
				List<String> linguisticTypeNamesSubdiv=new ArrayList<String>();
				List<String> linguisticTypeNamesAssoc=new ArrayList<String>();
				for(LinguisticType lt: ltypes){
					Constraint cs=lt.getConstraints();
					if(cs!=null){
						if(cs.getStereoType()==Constraint.SYMBOLIC_SUBDIVISION){
							linguisticTypeNamesSubdiv.add(lt.getLinguisticTypeName());
						} else if(cs.getStereoType()==Constraint.SYMBOLIC_ASSOCIATION){
							linguisticTypeNamesAssoc.add(lt.getLinguisticTypeName());
						}
					}
				}
				
				//JOptionPane.showMessageDialog(null, "No segmentation and annotation tier", "Tier", JOptionPane.CLOSED_OPTION);
				interlinearizeTierButton.setEnabled(false);
				interlinearizeTierCbBox.setEnabled(false);
				
				TitledBorder border=new TitledBorder(ElanLocale.getString("TierParameterFrame.Interlinearize.Title.Create"));
				tierNamePanel.setBorder(border);
				GridBagConstraints gbc=new GridBagConstraints();
				Insets insets= new Insets(0, 3, 2, 2);
				gbc.insets=insets;
				gbc.gridx=0;
				gbc.gridy=0;
				JLabel segmentationLb=new JLabel(ElanLocale.getString("TierParameterFrame.Label.Tier.Segmentation"));
				tierNamePanel.add(segmentationLb, gbc);
				gbc.gridx=1;
				segmentationTxtField=new JTextField(mbValue);
				segmentationTxtField.setColumns(5);
				tierNamePanel.add(segmentationTxtField, gbc);
				gbc.gridx=2;
				tierNamePanel.add(new JLabel(ElanLocale.getString("TierParameterFrame.Label.LingType")), gbc);
				gbc.gridx=3;
				ltMbComboBox=new JComboBox(linguisticTypeNamesSubdiv.toArray());
				if(transcription.getLinguisticTypeByName("mb")!=null){
					ltMbComboBox.setSelectedItem("mb");
				}
				tierNamePanel.add(ltMbComboBox, gbc);
				
				gbc.gridy=1;
				gbc.gridx=0;
				JLabel glossLb=new JLabel(ElanLocale.getString("TierParameterFrame.Label.Tier.Gloss"));
				tierNamePanel.add(glossLb, gbc);
				gbc.gridx=1;
				glossTxtField=new JTextField(geValue);
				glossTxtField.setColumns(5);
				tierNamePanel.add(glossTxtField, gbc);
				gbc.gridx=2;
				tierNamePanel.add(new JLabel(ElanLocale.getString("TierParameterFrame.Label.LingType")), gbc);
				gbc.gridx=3;
				ltGeComboBox=new JComboBox(linguisticTypeNamesAssoc.toArray());
				if(transcription.getLinguisticTypeByName("ge")!=null){
					ltGeComboBox.setSelectedItem("ge");
				}
				tierNamePanel.add(ltGeComboBox, gbc);
				
				gbc.gridy=2;
				gbc.gridx=0;
				JLabel tierXLb=new JLabel(ElanLocale.getString("TierParameterFrame.Label.Tier.Pos"));
				tierNamePanel.add(tierXLb, gbc);
				gbc.gridx=1;
				tierXTxtField=new JTextField(rxValue);
				tierXTxtField.setColumns(5);
				tierNamePanel.add(tierXTxtField, gbc);
				gbc.gridx=2;
				tierNamePanel.add(new JLabel(ElanLocale.getString("TierParameterFrame.Label.LingType")), gbc);
				gbc.gridx=3;
				ltRxComboBox=new JComboBox(linguisticTypeNamesAssoc.toArray());
				if(transcription.getLinguisticTypeByName("rx")!=null){
					ltRxComboBox.setSelectedItem("rx");
				}
				tierNamePanel.add(ltRxComboBox, gbc);
				
				gbc.gridy=3;
				gbc.gridx=3;
				createTiers=new JButton(ElanLocale.getString("TierParameterFrame.JButton.Create"));
				createTiers.addActionListener(this);
				tierNamePanel.add(createTiers, gbc);
				
				getRootPane().repaint();
				pack();
			//}
		}
		if (e.getSource()==createTiers){
			String mbTierName=segmentationTxtField.getText();
			String geTierName=glossTxtField.getText();
			String rxTierName=tierXTxtField.getText();
			String motTierName=(String)interlinearizeTierCbBox.getSelectedItem();
			viewer.setTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_WORD, motTierName);
			viewer.setTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_PARSE, mbTierName);
			viewer.setTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_GLOSS, geTierName);
			viewer.setTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_POS, rxTierName);
			if(!motTierName.isEmpty()&&!mbTierName.isEmpty()&&!geTierName.isEmpty()&&!rxTierName.isEmpty()){		
				Vector<Tier> tierVector=transcription.getTiers();
				Vector<Tier> copyTierVector=new Vector<Tier>(tierVector);
				if (copyTierVector != null) {
					ListIterator<Tier> it=copyTierVector.listIterator();
					while(it.hasNext()){
						TierImpl tierImpl=(TierImpl) it.next();
						//if(tierImpl.getName().startsWith(motTierName)){
						if(tierImpl.getName().equals(motTierName)){		// CC 300811
							// CC  10/05/2011 add -cp to avoid creating 'doubles' if a new tier already exists
			                if (transcription.getTierWithId(mbTierName) != null) {mbTierName+="-cp";}
							/*
							TierImpl mbTier=new TierImpl(tierImpl, mbTierName, null, transcription, null);
							mbTier.setLinguisticType(transcription.getLinguisticTypeByName((String) ltMbComboBox.getSelectedItem()));
							mbTier.setParticipant(tierImpl.getParticipant());
							mbTier.setAnnotator(tierImpl.getAnnotator());
							mbTier.setDefaultLocale(tierImpl.getDefaultLocale());
							transcription.addTier(mbTier);
							*/ 
			                //CC 300811 For Undo 
							comMb = (AddTierCommand) ELANCommandFactory.createCommand(transcription,
			                        ELANCommandFactory.ADD_TIER);
							Object receiver = transcription;		
							Object[] args = new Object[6];
							args[0]=mbTierName;					
							args[1]=tierImpl;
							args[2]=ltMbComboBox.getSelectedItem();
							args[3]=tierImpl.getParticipant();
							args[4]=tierImpl.getAnnotator();
							args[5]=tierImpl.getDefaultLocale();
							comMb.execute(receiver,args);	
							
			                if (transcription.getTierWithId(geTierName) != null) {geTierName+="-cp";}
							/*
							TierImpl geTier=new TierImpl(mbTier, geTierName, null, transcription, null);
							geTier.setLinguisticType(transcription.getLinguisticTypeByName((String) ltGeComboBox.getSelectedItem()));
							geTier.setParticipant(tierImpl.getParticipant());
							geTier.setAnnotator(tierImpl.getAnnotator());
							geTier.setDefaultLocale(tierImpl.getDefaultLocale());
							transcription.addTier(geTier);
							*/
							comGe = (AddTierCommand) ELANCommandFactory.createCommand(transcription,
			                        ELANCommandFactory.ADD_TIER);	// CC 290811
							TierImpl mbTier = (TierImpl) (transcription.getTierWithId(mbTierName));
			                args[0]=geTierName;
							args[1]=mbTier;
							args[2]=ltGeComboBox.getSelectedItem();
							args[3]=tierImpl.getParticipant();
							args[4]=tierImpl.getAnnotator();
							args[5]=tierImpl.getDefaultLocale();
							comGe.execute(transcription,args);
							
			                if (transcription.getTierWithId(rxTierName) != null) {rxTierName+="-cp";}
							/*
							TierImpl rxTier=new TierImpl(mbTier, rxTierName, null, transcription, null);
							rxTier.setLinguisticType(transcription.getLinguisticTypeByName((String) ltRxComboBox.getSelectedItem()));
							rxTier.setParticipant(tierImpl.getParticipant());
							rxTier.setAnnotator(tierImpl.getAnnotator());
							rxTier.setDefaultLocale(tierImpl.getDefaultLocale());
							transcription.addTier(rxTier);
							*/
							comRx = (AddTierCommand) ELANCommandFactory.createCommand(transcription,
			                        ELANCommandFactory.ADD_TIER);	// CC 290811
						    args[0]=rxTierName;					// CC 290811
							args[1]=mbTier;
							args[2]=ltRxComboBox.getSelectedItem();
							args[3]=tierImpl.getParticipant();
							args[4]=tierImpl.getAnnotator();
							args[5]=tierImpl.getDefaultLocale();
							comRx.execute(transcription,args);
						}
					}
					this.setVisible(false);
					this.dispose();
				}
			}

		}
	}

}
