package mpi.eudico.client.annotator.lexica.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;

import mpi.eudico.client.annotator.viewer.LexiconViewer;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;

/**
 * Frame to configure a new name for tiers that enter in the interlinearization process
 * @author Coralie
 *
 */
public class TierRenameInterlinearizeFrame extends JFrame implements ActionListener {
	private LexiconViewer viewer;
	private TranscriptionImpl transcription;
	private String motValue;
	private String mbValue;
	private String geValue;
	private String rxValue;
	private JButton renameButton;
	private JTextField motTxtField;
	private JTextField mbTxtField;
	private JTextField geTxtField;
	private JTextField rxTxtField;
	
	public TierRenameInterlinearizeFrame(LexiconViewer viewer, TranscriptionImpl transcription, HashMap<String, String> map){
		this.viewer=viewer;
		this.transcription=transcription;
		motValue=map.get(LexiconViewer.INTERLINEARIZE_WORD_TIER_NAME);
		mbValue=map.get(LexiconViewer.INTERLINEARIZE_PARSE_TIER_NAME);
		geValue=map.get(LexiconViewer.INTERLINEARIZE_GLOSS_TIER_NAME);
		rxValue=map.get(LexiconViewer.INTERLINEARIZE_POS_TIER_NAME);
		setTitle("Parameter Tier Name to Interlinearize");
		init();
		this.setVisible(true);
		pack();
	}
	
	private void init(){
		setLayout(new GridBagLayout());

		//interlinearize tier
		GridBagConstraints gbc=new GridBagConstraints();
		Insets insets= new Insets(0, 3, 2, 2);
		gbc.insets=insets;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.gridx=0;
		gbc.gridy=0;
		JLabel currentMotTier=new JLabel("Current Tier To interlinearize : "+motValue);
		add(currentMotTier, gbc);
		gbc.gridx=1;
		gbc.gridy=0;
		motTxtField=new JTextField(4);
		add(motTxtField, gbc);
		gbc.gridx=0;
		gbc.gridy=1;
		JLabel currentMbTier=new JLabel("Current Segmentation Tier Name : "+mbValue);
		add(currentMbTier, gbc);
		gbc.gridx=1;
		gbc.gridy=1;
		mbTxtField=new JTextField(4);
		add(mbTxtField, gbc);
		gbc.gridx=0;
		gbc.gridy=2;
		JLabel currentGeTier=new JLabel("Current Annotation Tier Name (gloss) : "+geValue);
		add(currentGeTier, gbc);
		gbc.gridx=1;
		gbc.gridy=2;
		geTxtField=new JTextField(4);
		add(geTxtField, gbc);
		gbc.gridx=0;
		gbc.gridy=3;
		JLabel currentRxTier=new JLabel("Current Annotation Tier Name (pos) : "+rxValue);
		add(currentRxTier, gbc);
		gbc.gridx=1;
		gbc.gridy=3;
		rxTxtField=new JTextField(4);
		add(rxTxtField, gbc);
		gbc.gridx=0;
		gbc.gridy=4;
		renameButton=new JButton("Rename Tiers");
		renameButton.addActionListener(this);
		add(renameButton,gbc);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==renameButton){
			if(!motTxtField.getText().isEmpty()){
				motValue=motTxtField.getText();
			} if(!mbTxtField.getText().isEmpty()){
				mbValue=mbTxtField.getText();
			} if(!geTxtField.getText().isEmpty()){
				geValue=geTxtField.getText();
			} if(!rxTxtField.getText().isEmpty()){
				rxValue=rxTxtField.getText();
			}
			viewer.setTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_WORD, motValue);
			viewer.setTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_PARSE, mbValue);
			viewer.setTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_GLOSS, geValue);
			viewer.setTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_POS, rxValue);
			this.setVisible(false);
			dispose();
		}
		
	}
}
