package mpi.eudico.client.annotator.lexica.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.commands.Command;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.gui.EditTierDialog;
import mpi.eudico.client.annotator.gui.EditTypeDialog2;
import mpi.eudico.client.annotator.viewer.AbstractViewer;
import mpi.eudico.client.annotator.viewer.LexiconViewer;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.server.corpora.clomimpl.type.Constraint;
import mpi.eudico.server.corpora.clomimpl.type.LinguisticType;
/**
 * Frame to help the user to correct his annotation hierarchy and the linguistic type problems
 * so as to the program be able to make an interlinearization of his data
 * @author Coralie
 *
 */
public class WrongInterlinearizationParametersFrame extends JFrame implements ActionListener {

	private TierImpl tierToModif;
	private AbstractViewer viewer;
	
	private String errorMessage="<html>";
	
	private JButton createTierAnnotation;
	private JButton renameTier;
	private JButton changeLinguisticType;
	
	private AnnotationTierHierarchyInfos infos;
	
	public WrongInterlinearizationParametersFrame(AbstractViewer viewer, AnnotationTierHierarchyInfos infos){
		this.viewer=viewer;
		this.infos=infos;
		setTitle(ElanLocale.getString("WrongParameters.Interlinearize.Frame.Title"));
		errorMessage="<html>";
		if(!infos.isCorrectHierarchy()){
			errorMessage+=ElanLocale.getString("WrongParameters.Interlinearize.Frame.Error.Hierarchy")+" <br/>";
			if(infos.getInterlWordTier()==null){
				String prefTierWord=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_WORD);
				errorMessage+=ElanLocale.getString("WrongParameters.Frame.Error.WrongTier");
				errorMessage+=" '"+prefTierWord+"'<br/>";
				errorMessage+=ElanLocale.getString("WrongParameters.Interlinearize.Frame.Error.BaseTier")+" '"+prefTierWord+"'<br/>";
//				errorMessage+="Select an annotation in the tier "+ prefTierWord+"<br/>";
				errorMessage+=ElanLocale.getString("WrongParameters.Frame.Error.If")+" "+prefTierWord+" "+ElanLocale.getString("WrongParameters.Interlinearize.Frame.Error.CreateHierarchy");
			}
			else{
				if(infos.getInterlParseTier()==null){
					String prefTierParse=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_PARSE);
					errorMessage+=ElanLocale.getString("WrongParameters.Frame.Error.ParseTier")+" '"+ prefTierParse+"'<br/>";
					errorMessage+=ElanLocale.getString("WrongParameters.Frame.Error.ParseTier.Modify")+" "+ElanLocale.getString("WrongParameters.Interlinearize.Frame.Label.CreateHierarchy");;
				}
				else{
					if(infos.getInterlGlossTier()==null){
						String prefTierGloss=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_GLOSS);
						errorMessage+=ElanLocale.getString("WrongParameters.Frame.Error.GlossTier")+" '"+ prefTierGloss+"'<br/>";
						errorMessage+=ElanLocale.getString("WrongParameters.Frame.Error.GlossTier.Modify")+" "+ElanLocale.getString("WrongParameters.Interlinearize.Frame.Label.CreateHierarchy");
					}
					if(infos.getInterlPosTier()==null){
						String prefTierPos=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_POS);
						errorMessage+=ElanLocale.getString("WrongParameters.Frame.Error.PosTier")+" '"+ prefTierPos+"'<br/>";
						errorMessage+=ElanLocale.getString("WrongParameters.Frame.Error.PosTier.Modify")+" "+ElanLocale.getString("WrongParameters.Interlinearize.Frame.Label.CreateHierarchy");
					}
				}
			}
			changeTierParameters();
		}
		else{
			errorMessage+=ElanLocale.getString("WrongParameters.Frame.Error.LinguisticType")+"<br/>";
			if(!infos.isCorrectParseLingType()){
				errorMessage+="<br/>"+ElanLocale.getString("WrongParameters.Frame.Error.LinguisticType.Incorrect")+" "+infos.getInterlParseTier().getName()+ "<br/> "+ElanLocale.getString("WrongParameters.Frame.Error.LinguisticType.SubdivType");
				tierToModif=infos.getInterlParseTier();
			}
			if(!infos.isCorrectPosLingType()){
				errorMessage+="<br/>"+ElanLocale.getString("WrongParameters.Frame.Error.LinguisticType.Incorrect")+" "+infos.getInterlPosTier().getName()+ "<br/> "+ElanLocale.getString("WrongParameters.Frame.Error.LinguisticType.AssocType");
				tierToModif=infos.getInterlPosTier();
			}
			if(!infos.isCorrectGlossLingType()){
				errorMessage+="<br/>"+ElanLocale.getString("WrongParameters.Frame.Error.LinguisticType.Incorrect")+" "+infos.getInterlGlossTier().getName()+"<br/> "+ElanLocale.getString("WrongParameters.Frame.Error.LinguisticType.AssocType");
				tierToModif=infos.getInterlGlossTier();
			}
			changeLingTypeParameters();
		}
		this.setVisible(true);
		pack();
	}
	
	private void changeTierParameters(){
		setLayout(new GridBagLayout());
		
		GridBagConstraints gbc=new GridBagConstraints();
		Insets insets= new Insets(0, 3, 2, 2);
		gbc.insets=insets;
		gbc.fill=GridBagConstraints.EAST;
		gbc.gridx=0;
		gbc.gridy=0;
		JLabel label=new JLabel(errorMessage+"</html>");
		add(label, gbc);
		JPanel buttonPane=new JPanel();
		createTierAnnotation=new JButton(ElanLocale.getString("WrongParameters.Interlinearize.Frame.JButton.CreateTierAnnotation"));
		createTierAnnotation.addActionListener(this);
		buttonPane.add(createTierAnnotation);
		renameTier=new JButton(ElanLocale.getString("WrongParameters.Interlinearize.Frame.JButton.RenameTier"));
		renameTier.addActionListener(this);
		buttonPane.add(renameTier);

		gbc.gridx=0;
		gbc.gridy=1;
		add(buttonPane, gbc);
	}
	
	//Linguistic Type problem
	private void changeLingTypeParameters(){
		setLayout(new GridBagLayout());

		GridBagConstraints gbc=new GridBagConstraints();
		Insets insets= new Insets(0, 3, 2, 2);
		gbc.insets=insets;
		gbc.fill=GridBagConstraints.CENTER;
		gbc.gridx=0;
		gbc.gridy=0;
		JLabel label=new JLabel(errorMessage+"</html>");
		add(label,gbc);
		gbc.gridx=0;
		gbc.gridy=1;
		changeLinguisticType=new JButton(ElanLocale.getString("WrongParameters.Interlinearize.Frame.JButton.ChangeLinguisticType"));
		changeLinguisticType.addActionListener(this);
		add(changeLinguisticType, gbc);
		
	}
	
	private boolean hasToCreateLingType(TranscriptionImpl transcription){
		boolean hasToCreateLingType=false;
		Vector<LinguisticType> ltypes=transcription.getLinguisticTypes();
		boolean symbolicAssociation=false;
		boolean symbolicSubdivision=false;
		for(LinguisticType lt: ltypes){
			Constraint cs=lt.getConstraints();
			if(cs!=null){
				if(cs.getStereoType()==Constraint.SYMBOLIC_SUBDIVISION){
					symbolicSubdivision=true;
				} else if(cs.getStereoType()==Constraint.SYMBOLIC_ASSOCIATION){
					symbolicAssociation=true;
				}
			}
		}
		if(!symbolicAssociation||!symbolicSubdivision){
			hasToCreateLingType=true;
			String symbolicType="";
			if(!symbolicAssociation){
				symbolicType+=ElanLocale.getString("WrongParameters.Frame.JOptionPane.SymbolicAssoc")+" ";
			}
			if(!symbolicSubdivision){
				symbolicType+=ElanLocale.getString("WrongParameters.Frame.JOptionPane.SymbolicSubdiv")+" ";
			}
			int option=JOptionPane.showConfirmDialog(null, ElanLocale.getString("WrongParameters.Frame.JOptionPane.CreateLinguisticType")+" "+symbolicType+"\n "+ElanLocale.getString("WrongParameters.Frame.JOptionPane.ClickOk"), ElanLocale.getString("WrongParameters.Frame.JOptionPane.LinguisticType"), JOptionPane.OK_OPTION, JOptionPane.ERROR_MESSAGE);
			if(option==JOptionPane.YES_OPTION){
				Command com=ELANCommandFactory.createCommand(transcription, ELANCommandFactory.EDIT_TYPE);
				com.execute(transcription, new Object[] {EditTypeDialog2.ADD});
			}
		}
		return hasToCreateLingType;
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		String prefTierWord=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_WORD);
		String prefTierParse=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_PARSE);
		String prefTierGloss=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_GLOSS);
		String prefTierPos=(String)((LexiconViewer)viewer).getTierPreferences(LexiconViewer.INTERLINEARIZE_TIER_POS);

		HashMap<String, String> map=new HashMap<String, String>();
		map.put(LexiconViewer.INTERLINEARIZE_WORD_TIER_NAME, prefTierWord);
		map.put(LexiconViewer.INTERLINEARIZE_PARSE_TIER_NAME, prefTierParse);
		map.put(LexiconViewer.INTERLINEARIZE_GLOSS_TIER_NAME, prefTierGloss);
		map.put(LexiconViewer.INTERLINEARIZE_POS_TIER_NAME, prefTierPos);
		TranscriptionImpl transcription=(TranscriptionImpl) infos.getTranscription();
		if(e.getSource()==createTierAnnotation){
			if(!hasToCreateLingType(transcription)){	
				new TierInterlinearizeParameterFrame((LexiconViewer)viewer, transcription, map);
			}
			this.setVisible(false);
			dispose();
		} else if (e.getSource()==renameTier){
			new TierRenameInterlinearizeFrame((LexiconViewer) viewer, transcription, map);
			this.setVisible(false);
			dispose();
			
		} else if(e.getSource()==changeLinguisticType){
			if(!hasToCreateLingType(transcription)){
				Command c = ELANCommandFactory.createCommand(infos.getTranscription(), ELANCommandFactory.EDIT_TIER);
				Object[] args = new Object[] {
						new Integer(EditTierDialog.CHANGE), tierToModif
				};
				c.execute(infos.getTranscription(), args);
			}
	        this.setVisible(false);
			dispose();
		}
	}
}
