package mpi.eudico.client.annotator.lexica.record;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.WindowConstants;

import org.jdom.Element;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.viewer.AbstractViewer;

public abstract class AbstractLXDataDialog extends JFrame {
	protected AbstractViewer lxViewer;
	protected JTabbedPane tabPane;
	protected JButton cancelButton;
	protected JLabel titleLabel;
	private JPanel titlePanel;
	
	protected String lexeme;
	protected Element selectedElement;
	
	/**
	 * Empty constructor
	 */
	public AbstractLXDataDialog(AbstractViewer viewer){
		lxViewer=viewer;
		initComponents();
		pack();
	}
	
	/**
	 * Constructor to add a new Record whose lexeme value is the parameter
	 * @param lexeme
	 */
	public AbstractLXDataDialog(AbstractViewer viewer, String lexeme){
		lxViewer=viewer;
		this.lexeme=lexeme;
		initComponents();
		pack();
	}
	
	/**
	 * Constructor to show or edit informations from the Element value
	 * @param selectedElement
	 */
	public AbstractLXDataDialog(AbstractViewer viewer, Element selectedElement){
		lxViewer=viewer;
		this.selectedElement=selectedElement;
		initComponents();
		pack();
		setLocationRelativeTo(getParent());
	}
	
	protected void initComponents(){
		GridBagConstraints gridBagConstraints= new GridBagConstraints();
		titlePanel = new JPanel();
        titleLabel = new JLabel();
		cancelButton = new JButton();
		
		getContentPane().setLayout(new GridBagLayout());
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent evt) {
				setVisible(false);
		        dispose();
			}
		});

		Insets insets = new Insets(2, 6, 2, 6);

		titleLabel.setFont(titleLabel.getFont().deriveFont((float) 16));
		titlePanel.add(titleLabel);

		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 0;
		gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints.anchor = GridBagConstraints.NORTH;
		gridBagConstraints.insets = insets;
		getContentPane().add(titlePanel, gridBagConstraints);

		tabPane=new JTabbedPane();
		initTabPane();
		
		gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = insets;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(tabPane, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = insets;
        getContentPane().add(initDialogButtonPanel(), gridBagConstraints);

        updateLocale();
	}
	
	abstract protected void initTabPane();
	
	abstract protected JPanel initDialogButtonPanel();
	
	 /**
     * Applies localized strings to the ui elements.
     */
	protected void updateLocale() {
        cancelButton.setText(ElanLocale.getString("Button.Close"));
    }
    
    abstract protected void clearInformations();

}
