package mpi.eudico.client.annotator.lexica.record;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.lexica.LexiconDocumentData;
import mpi.eudico.client.annotator.viewer.AbstractViewer;

import org.jdom.Element;

public class EditLXRecordPanel extends ShowLXRecordPanel implements ItemListener, ActionListener{

	protected JTabbedPane tabPane;
	//add or remove GlosePane informations
	private JButton addGloseButton;
	private List<JButton> removeGloseButtonList;
	
	//add or remove UForm informations
	private JButton addSegmentButton;
	private List<JButton> removeSegmentButtonList;
	
	private List<JButton> removeVariantButtonList;
	
	/**
	 * Create the panel
	 * @param document
	 */
	public EditLXRecordPanel(AbstractViewer viewer){
		super(viewer);
	}
	
	public EditLXRecordPanel(AbstractViewer viewer, Element selectedElement){
		super(viewer, selectedElement);
	
		if(selectedElement!=null){
			lexemeTxtField.setText(selectedElement.getText());			
		}
	}
	
	@Override
	protected void initComponents(){
		lexiconPane=new JPanel();
		lexemeLb=new JLabel();
		lexemeTxtField=new JTextField(10);
		lexemeElmtList=new ArrayList<Element>();
		// UTD 10/09/12 Faut-il permettre l'autocomplétion ici ?
		lexemeCbBox=new JSuggestComboBox(LexiconDocumentData.getMorphemesToSelect(false).toArray());
		lexemeCbBox.setRenderer(new JDomElementListCellRenderer());
		selectInComboBox(lexemeCbBox, selectedElement);
		uFormPane=new JPanel();
		glosePane=new JPanel();
		variantPane=new JPanel();
		lexemeCbBox.addItemListener(this);
		gloseTierXPaneList=new ArrayList<LXSubPanel>();
		gloseList=new ArrayList<Element>();
		variantList=new ArrayList<Element>();
		removeGloseButtonList=new ArrayList<JButton>();
		segmentList=new ArrayList<Element>();
		parseSegmentPaneList=new ArrayList<LXSubPanel>();
		removeSegmentButtonList=new ArrayList<JButton>();
		removeVariantButtonList=new ArrayList<JButton>();
		
		variantPaneList=new ArrayList<LXSubPanel>();
		
		tabPane=new JTabbedPane();
		initTabPane();
		
		updateLexemePane();
		
    	GridBagLayout gridbag=new GridBagLayout();
    	Insets insets=new Insets(2, 6, 2, 6);
		setLayout(gridbag);
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets=insets;
		gbc.anchor=GridBagConstraints.NORTH;
		add(lexiconPane, gbc);
		
		updateVariantPane();
		gbc.gridx=0;
		gbc.gridy=1;
		add(variantPane, gbc);
		
		gbc=new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.insets = insets;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
        add(tabPane, gbc);
		
		updateLocale();
	}

	@Override
	protected void updateLexemePane(){
		GridBagLayout gridbag=new GridBagLayout();
		lexiconPane.setLayout(gridbag);
		GridBagConstraints gbc=new GridBagConstraints();
		Insets insets=new Insets(2, 6, 2, 6);
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor=GridBagConstraints.NORTHEAST;
		gbc.insets=insets;
		lexiconPane.add(lexemeLb, gbc);
		gbc.gridx=0;
		gbc.gridx=1;
		if(lexemeCbBox.getSelectedItem()!=null){
			initElementsAssociated((Element)lexemeCbBox.getSelectedItem());
			Element nullElement=null;
			Element lexicalEntry=((Element)lexemeCbBox.getSelectedItem()).getParentElement();
			if(lexicalEntry.getChild(LexiconDocumentData.MORPHOLOGY)==null){
				tabPane.setSelectedIndex(0);
				segmentList.add(nullElement);
				segmentList.add(nullElement);
			} else{
				tabPane.setSelectedIndex(1);
				gloseList.add(nullElement);
			}
			updateGlosePane();
			updateUFormPane();
		}
        lexiconPane.add(lexemeCbBox, gbc);
        gbc.gridx=0;
		gbc.gridx=2;
		lexiconPane.add(lexemeTxtField, gbc);
	}
	
	@Override
	protected void updateVariantPane(){
		Insets insets = new Insets(2, 2, 2, 2);
		GridBagLayout gridbag=new GridBagLayout();
		variantPane.setLayout(gridbag);
		GridBagConstraints gbc=new GridBagConstraints();
    	gbc.insets=insets;
    	gbc.anchor=GridBagConstraints.WEST;
		for(int i=0; i<variantList.size(); i++){
			gbc.gridy=0+i;
	    	gbc.gridx=0;
	    	VariantSubPane variantSubPane;
	    	if(i<variantPaneList.size()){
	    		variantSubPane=(VariantSubPane) variantPaneList.get(i);
	    	} else{
	    		Element altFormElmt=variantList.get(i);
	    		Element morphElmt=altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY);
	    		List<Element> segmentElmtList=null;
	    		if(morphElmt!=null){
	    			segmentElmtList=morphElmt.getChildren(LexiconDocumentData.SEGMENT);
	    		}
	    		variantSubPane=new VariantSubPane(altFormElmt, segmentElmtList);
	    		variantSubPane.setEditable(true);
	    		variantPaneList.add(variantSubPane);
	    	}
	    	variantPane.add(variantSubPane, gbc);
			gbc.gridx=1;
			JButton deleteButton;
			if(i<removeVariantButtonList.size()){
				deleteButton=removeVariantButtonList.get(i);
			} else{
				deleteButton=new JButton(ElanLocale.getString("Button.Delete"));
				deleteButton.addActionListener(this);
				removeVariantButtonList.add(deleteButton);
			}
			variantPane.add(deleteButton, gbc);
		}
	}
	
	//add glose informations
	@Override
	protected JPanel addGloseSubPane(List<Element> gloseList){
		JPanel glosePanel=new JPanel();
		GridBagLayout gridbag=new GridBagLayout();
		glosePanel.setLayout(gridbag);
		GridBagConstraints gbc=new GridBagConstraints();
		Insets insets=new Insets(2, 6, 2, 6);
		gbc.fill=GridBagConstraints.BOTH;
		gbc.anchor=GridBagConstraints.EAST;
		gbc.insets=insets;
		gbc.weightx=1.0;
        gbc.weighty=1.0;
        int i=0;
		while(i<gloseList.size()){
			String gloseText="";
			String tamText="";
			String tierXText="";
			if(gloseList.get(i)!=null){
				gloseText=gloseList.get(i).getText();
				tamText=gloseList.get(i).getAttributeValue(LexiconDocumentData.DERIVATION);
				tierXText=gloseList.get(i).getAttributeValue(LexiconDocumentData.RX);
			}
			gbc.gridx=0;
			gbc.gridy=i;
			GloseTierXSubPane gloseTierXPane;
			if(i<gloseTierXPaneList.size()){
				gloseTierXPane=(GloseTierXSubPane) gloseTierXPaneList.get(i);
			}
			else{
				gloseTierXPane=new GloseTierXSubPane(gloseText, tamText, tierXText);
				gloseTierXPane.setEditable(true);
				gloseTierXPaneList.add(gloseTierXPane);
			}
			glosePanel.add(gloseTierXPane, gbc);
			
			gbc.gridy=i;
			gbc.gridx=1;
			JButton deleteButton;
			if(i<removeGloseButtonList.size()){
				deleteButton=removeGloseButtonList.get(i);
			} else{
				deleteButton=new JButton(ElanLocale.getString("Button.Delete"));
				deleteButton.addActionListener(this);
				removeGloseButtonList.add(deleteButton);
			}
			if(gloseList.size()==1){
				deleteButton.setEnabled(false);
			} else{
				deleteButton.setEnabled(true);
			}
			glosePanel.add(deleteButton, gbc);
			i++;
		}
		addGloseButton=new JButton(ElanLocale.getString("Button.Add"));
		addGloseButton.addActionListener(this);
		gbc.gridx=1;
		gbc.gridy=i+1;
		glosePanel.add(addGloseButton, gbc);
		
		return glosePanel;
	}
	
	@Override
	protected void updateUFormPane(){
		Insets insets = new Insets(2, 2, 2, 2);
		GridBagLayout gridbag=new GridBagLayout();
		uFormPane.setLayout(gridbag);
		GridBagConstraints gbc=new GridBagConstraints();
    	gbc.insets=insets;
    	gbc.anchor=GridBagConstraints.WEST;
    	int i=0;
		while(i<segmentList.size()){
			gbc.gridy=0+i;
	    	gbc.gridx=0;
	    	ParseSegmentsSubPane parsePane;
	    	if(i<parseSegmentPaneList.size()){
	    		parsePane=(ParseSegmentsSubPane) parseSegmentPaneList.get(i);
	    	} else{
	    		parsePane=new ParseSegmentsSubPane(segmentList.get(i));
	    		parseSegmentPaneList.add(parsePane);
	    		parsePane.getShowButton().addActionListener(this);
	    	}
	    	uFormPane.add(parsePane, gbc);
	    	
	    	gbc.gridy=i;
			gbc.gridx=1;
	    	JButton deleteButton;
			if(i<removeSegmentButtonList.size()){
				deleteButton=removeSegmentButtonList.get(i);
			} else{
				deleteButton=new JButton(ElanLocale.getString("Button.Delete"));
				deleteButton.addActionListener(this);
				removeSegmentButtonList.add(deleteButton);
			}
			if(segmentList.size()<=2){
				deleteButton.setEnabled(false);
			} else{
				deleteButton.setEnabled(true);
			}
			uFormPane.add(deleteButton, gbc);
	    	
	    	i++;
		}
		addSegmentButton=new JButton(ElanLocale.getString("Button.Add"));
		addSegmentButton.addActionListener(this);
		gbc.gridx=1;
		gbc.gridy=i+1;
		uFormPane.add(addSegmentButton, gbc);
	}
	
	
	protected void initTabPane() {
		tabPane.addTab("Gloss values", glosePane);
		tabPane.addTab("Parse values", uFormPane);
	}
	
	public boolean isAParsingRecord(){
		if (tabPane.getSelectedIndex()==0){
			return false;
		} else{
			return true;
		}
	}
	
	public List<LXSubPanel> getModifiedPane(){
		if(tabPane.getSelectedIndex()==0){
			return gloseTierXPaneList; 
		}else{
			return parseSegmentPaneList;
		}
	}
	
	public List<LXSubPanel> getVariantSubPane(){
		return variantPaneList;
	}
	
	public String getLexeme(){
		return lexemeTxtField.getText();
	}
	
	@Override
	public void itemStateChanged(ItemEvent e) {
		super.itemStateChanged(e);
		Element lexemeElmt=(Element)lexemeCbBox.getSelectedItem();
		if(lexemeElmt!=null){
	        lexemeTxtField.setText(lexemeElmt.getText());			
		}
		else{
			lexemeTxtField.setText("");
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==addGloseButton){
			Element nullElement=null;
			gloseList.add(nullElement);
			glosePane.removeAll();
			updateGlosePane();
			getRootPane().repaint();
			SwingUtilities.windowForComponent(this).pack();
		}
		else if(e.getSource()==addSegmentButton){
			Element nullElement=null;
			segmentList.add(nullElement);
			uFormPane.removeAll();
			updateUFormPane();
			getRootPane().repaint();
			SwingUtilities.windowForComponent(this).pack();
		}
		for(int i=0; i<removeGloseButtonList.size(); i++){
			if (e.getSource()==removeGloseButtonList.get(i)){
				gloseTierXPaneList.remove(i);
				removeGloseButtonList.remove(i);
				gloseList.remove(i);
				glosePane.removeAll();
				updateGlosePane();
				getRootPane().repaint();
				SwingUtilities.windowForComponent(this).pack();
				break;
			}
		}
		for(int i=0; i<removeSegmentButtonList.size(); i++){
			if(e.getSource()==removeSegmentButtonList.get(i)){
				parseSegmentPaneList.remove(i);
				removeSegmentButtonList.remove(i);
				segmentList.remove(i);
				uFormPane.removeAll();
				updateUFormPane();
				getRootPane().repaint();
				SwingUtilities.windowForComponent(this).pack();
				break;
			}
		}
		for(LXSubPanel parsePane:parseSegmentPaneList){
			if(e.getSource()==parsePane.getShowButton()){
				Element elementToShow=parsePane.getSelectedUFormElement();
				if(elementToShow!=null){
					ShowEditLXDataDialog dialog=new ShowEditLXDataDialog(lxViewer, elementToShow);
					dialog.setVisible(true);
				}
			}
		}
		for(int i=0;i<removeVariantButtonList.size(); i++){
			if(e.getSource()==removeVariantButtonList.get(i)){
				removeVariantButtonList.remove(i);
				variantPaneList.remove(i);
				variantList.remove(i);
				variantPane.removeAll();
				updateVariantPane();
				getRootPane().repaint();
				SwingUtilities.windowForComponent(this).pack();
				break;
			}
		}
	}
}
