package mpi.eudico.client.annotator.lexica.record;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.lexica.LexiconDocumentData;
import mpi.eudico.client.annotator.viewer.AbstractViewer;

import org.jdom.Element;

/**
 * 
 * @author Coralie
 *
 */
public class EditVariantToRecordPanel extends ShowVariantToRecordPanel implements ActionListener{

	private JButton showLexemeButton;
	
	//add or remove UForm informations
	private JButton addSegmentButton;
	private List<JButton> removeSegmentButtonList;
	
	/**
	 * Create the panel
	 */
	public EditVariantToRecordPanel(AbstractViewer viewer){
		super(viewer);
	}
	
	public EditVariantToRecordPanel(AbstractViewer viewer, Element selectedElement){
		super(viewer, selectedElement);
		//removeSegmentButtonList=new ArrayList<JButton>();
		if(selectedElement!=null){
	        lexemeTxtField.setText(selectedElement.getText());			
		}
	}
	
	protected void updateLexemePane(){
		GridBagLayout gridbag=new GridBagLayout();
		lexiconPane.setLayout(gridbag);
		GridBagConstraints gbc=new GridBagConstraints();
		Insets insets=new Insets(2, 6, 2, 6);
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor=GridBagConstraints.NORTHEAST;
		gbc.insets=insets;
		lexemeLb.setText(ElanLocale.getString("VariantPanel.JLabel.WordForm"));
		lexiconPane.add(lexemeLb, gbc);
		gbc.gridx=1;
		gbc.gridy=0;
		lexiconPane.add(wordFormCbBox, gbc);
		if(wordFormCbBox.getSelectedItem()!=null){
			Element nullElement=null;
			Element altFormElmt=((Element)wordFormCbBox.getSelectedItem()).getParentElement();
			if(altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY)!=null){
				initSegmentationElements(altFormElmt);
				updateUFormPane();
			} else{
				segmentList.add(nullElement);
				segmentList.add(nullElement);
				updateUFormPane();
			}
		}
        gbc.gridx=2;
        gbc.gridy=0;
        lexiconPane.add(lexemeTxtField, gbc);
	}
	
	protected void updateGlosePane(){
		derTxtField=new JTextField();
		GridBagLayout gridbag=new GridBagLayout();
		glosePane.setLayout(gridbag);
		GridBagConstraints gbc=new GridBagConstraints();
		Insets insets=new Insets(2, 6, 2, 6);
		if(wordFormCbBox.getSelectedItem()!=null){
			Element altFormElmt=((Element)wordFormCbBox.getSelectedItem()).getParentElement();
			Element formElmt=altFormElmt.getParentElement();
			Element lexicalEntry=formElmt.getParentElement();
			Element lexemeElmt=lexicalEntry.getChild(LexiconDocumentData.LEXEME);
			gbc.gridy=0;
			gbc.gridx=0;
			gbc.insets=insets;
			JLabel isAVariantLb=new JLabel(ElanLocale.getString("VariantPanel.JLabel.IsAVariantOf"));
			glosePane.add(isAVariantLb, gbc);
			gbc.gridx=1;
			lexemeCbBox.setSelectedItem(lexemeElmt);
			glosePane.add(lexemeCbBox, gbc);
			gbc.gridx=2;
			showLexemeButton=new JButton(ElanLocale.getString("VariantPanel.JLabel.ShowLexeme"));
			showLexemeButton.addActionListener(this);
			glosePane.add(showLexemeButton, gbc);
			Element wFormElmt=altFormElmt.getChild(LexiconDocumentData.WRITTENFORM);
			derTxtField.setColumns(5);
			if(wFormElmt.getAttribute(LexiconDocumentData.DERIVATION)!=null){
				derTxtField.setText(wFormElmt.getAttributeValue(LexiconDocumentData.DERIVATION));
			}
			gbc.gridy=1;
			gbc.gridx=0;
			JLabel derLabel=new JLabel(ElanLocale.getString("VariantPanel.JLabel.Slash"));
			glosePane.add(derLabel, gbc);
			gbc.gridx=1;
			glosePane.add(derTxtField, gbc);
		}
	}
	
	protected void updateUFormPane(){
		Insets insets = new Insets(2, 2, 2, 2);
		GridBagLayout gridbag=new GridBagLayout();
		uFormPane.setLayout(gridbag);
		GridBagConstraints gbc=new GridBagConstraints();
    	gbc.insets=insets;
    	gbc.anchor=GridBagConstraints.WEST;
    	int i=0;
		while(i<segmentList.size()){
			gbc.gridy=0+i;
	    	gbc.gridx=0;
	    	ParseSegmentsSubPane parsePane;
	    	if(i<parseSegmentPaneList.size()){
	    		parsePane=(ParseSegmentsSubPane) parseSegmentPaneList.get(i);
	    	} else{
	    		parsePane=new ParseSegmentsSubPane(segmentList.get(i));
	    		parseSegmentPaneList.add(parsePane);
	    	}
	    	parsePane.getShowButton().addActionListener(this);
	    	uFormPane.add(parsePane, gbc);
	    	
	    	gbc.gridy=i;
			gbc.gridx=1;
	    	JButton deleteButton;
			if(i<removeSegmentButtonList.size()){
				deleteButton=removeSegmentButtonList.get(i);
			} else{
				deleteButton=new JButton(ElanLocale.getString("Button.Delete"));
				deleteButton.addActionListener(this);
				removeSegmentButtonList.add(deleteButton);
			}
			if(segmentList.size()<=2){
				deleteButton.setEnabled(false);
			} else{
				deleteButton.setEnabled(true);
			}
			uFormPane.add(deleteButton, gbc);
	    	
	    	i++;
		}
		JPanel buttonPanel=new JPanel();
		addSegmentButton=new JButton(ElanLocale.getString("Button.Add"));
		addSegmentButton.addActionListener(this);
		buttonPanel.add(addSegmentButton);
		gbc.gridx=1;
		gbc.gridy=i+1;
		uFormPane.add(buttonPanel, gbc);
	}
	
	protected void initComponents(){
		lexiconPane=new JPanel();
		lexemeLb=new JLabel();
		lexemeTxtField=new JTextField(10);
		lexemeElmtList=new ArrayList<Element>();
		//lexemes for records
		lexemeCbBox=new JSuggestComboBox(LexiconDocumentData.getMorphemesToSelect(false).toArray());
		lexemeCbBox.setRenderer(new JDomElementListCellRenderer());
		selectInComboBox(lexemeCbBox, selectedElement);
		//wordform for variant elements
		wordFormCbBox=new JSuggestComboBox(LexiconDocumentData.getMorphemesToSelect(true).toArray());
		wordFormCbBox.setRenderer(new JDomElementListCellRenderer());
		selectInComboBox(wordFormCbBox, selectedElement);
		//super.initComponents();
		uFormPane=new JPanel();
		glosePane=new JPanel();
		variantPane=new JPanel();
		//lexemeCbBox.addItemListener(this);
		
		gloseTierXPaneList=new ArrayList<LXSubPanel>();
		gloseList=new ArrayList<Element>();
		variantList=new ArrayList<Element>();
		segmentList=new ArrayList<Element>();
		parseSegmentPaneList=new ArrayList<LXSubPanel>();
		wordFormCbBox.addItemListener(this);
		removeSegmentButtonList=new ArrayList<JButton>();
		variantPaneList=new ArrayList<LXSubPanel>();
		
		updateLexemePane();
		
    	GridBagLayout gridbag=new GridBagLayout();
    	Insets insets=new Insets(2, 6, 2, 6);
		setLayout(gridbag);
		GridBagConstraints gbc = new GridBagConstraints();
		int y=0;
		gbc.gridx=0;
		gbc.gridy=y;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets=insets;
		gbc.anchor=GridBagConstraints.NORTH;
		add(lexiconPane, gbc);

//		y++;
//		updateVariantPane();
//		gbc.gridx=0;
//		gbc.gridy=y;
//		add(variantPane, gbc);

		updateGlosePane();
		
		y++;
		gbc.gridx=0;
		gbc.gridy=y;
		add(glosePane, gbc);
		
		y++;
		gbc.gridx=0;
		gbc.gridy=y;
		add(uFormPane, gbc);
		
	
	}

	public void itemStateChanged(ItemEvent e) {
		super.itemStateChanged(e);
		Element lexemeElmt=(Element)wordFormCbBox.getSelectedItem();
		if(lexemeElmt!=null){
	        lexemeTxtField.setText(lexemeElmt.getText());			
		}
		else{
			lexemeTxtField.setText("");
		}
	}
	
	public String getDerInfo(){
			return derTxtField.getText(); 
	}
	
	public List<LXSubPanel> getParsePane(){
		return parseSegmentPaneList;
	}
	
	public String getWordform(){
		return lexemeTxtField.getText();
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==addSegmentButton){
			Element nullElement=null;
			segmentList.add(nullElement);
			uFormPane.removeAll();
			updateUFormPane();
			getRootPane().repaint();
			SwingUtilities.windowForComponent(this).pack();
		}
		else if(e.getSource()==showLexemeButton){
			Element elementToShow=(Element)lexemeCbBox.getSelectedItem();
			if(elementToShow!=null){
				ShowEditLXDataDialog dialog=new ShowEditLXDataDialog(lxViewer, elementToShow);
				dialog.setVisible(true);
			}
		}
		for(int i=0; i<removeSegmentButtonList.size(); i++){
			if(e.getSource()==removeSegmentButtonList.get(i)){
				parseSegmentPaneList.remove(i);
				removeSegmentButtonList.remove(i);
				segmentList.remove(i);
				uFormPane.removeAll();
				updateUFormPane();
				getRootPane().repaint();
				SwingUtilities.windowForComponent(this).pack();
				break;
			}
		}
		for(LXSubPanel parsePane:parseSegmentPaneList){
			if(e.getSource()==parsePane.getShowButton()){
				Element elementToShow=parsePane.getSelectedUFormElement();
				if(elementToShow!=null){
					ShowEditLXDataDialog dialog=new ShowEditLXDataDialog(lxViewer, elementToShow);
					dialog.setVisible(true);
				}
			}
		}
	}
}
