package mpi.eudico.client.annotator.lexica.record;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.lexica.LexiconDocumentData;
import mpi.eudico.client.annotator.viewer.AbstractViewer;
import mpi.eudico.client.annotator.viewer.LexiconViewer;

public class InsertLXDataDialog extends AbstractLXDataDialog implements ActionListener{
	protected JButton saveLexiconButton;

	public InsertLXDataDialog(AbstractViewer viewer){
		super(viewer);
	}

	public InsertLXDataDialog(AbstractViewer viewer, String lexeme){
		super(viewer, lexeme);
	}

	@Override
	protected JPanel initDialogButtonPanel() {
		JPanel dialogButtonPane=new JPanel();
		saveLexiconButton=new JButton();
		dialogButtonPane.setLayout(new GridLayout(1, 2, 6, 0));
		cancelButton.addActionListener(this);
		dialogButtonPane.add(cancelButton);

		saveLexiconButton.addActionListener(this);
		dialogButtonPane.add(saveLexiconButton); 
		return dialogButtonPane;
	}

	@Override
	protected void initTabPane() {
		tabPane.addTab("Insert Record", new InsertLXRecordPanel(lxViewer, lexeme));
		tabPane.addTab("Insert Underlying-Form", new InsertParseRecordPanel(lxViewer, lexeme));
	}

	@Override
	protected void updateLocale(){
		super.updateLocale();
		setTitle("Insert Lexicon Data");
		titleLabel.setText("Insert Lexicon Data");
		saveLexiconButton.setText(ElanLocale.getString("LexiconViewer.RecordDlg.Button.Save"));
	}

	@Override
	protected void clearInformations(){
		tabPane.setComponentAt(0, new InsertLXRecordPanel(lxViewer));
		tabPane.setComponentAt(1, new InsertParseRecordPanel(lxViewer));
		getRootPane().repaint();
		pack();
		setLocationRelativeTo(getParent());
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==cancelButton){
			//int option=JOptionPane.showConfirmDialog(null, "Are you sure you want close this window ?\nAll the unsaved datas will be lost", "Close Option", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
			//if(option==JOptionPane.OK_OPTION){
				this.setVisible(false);
				this.dispose();
			//}
			
		} else if(e.getSource()==saveLexiconButton){
			if(tabPane.getSelectedComponent() instanceof InsertLXRecordPanel){
				InsertLXRecordPanel insertLXRecordPanel=(InsertLXRecordPanel)tabPane.getSelectedComponent();
				String lexeme=insertLXRecordPanel.getLexemeTextField().getText();
				if (!lexeme.isEmpty()){
					List<GloseTierXSubPane> glosePaneList=insertLXRecordPanel.getGloseTierXPaneList();
					LXRecordQueries recordQuery=new LXRecordQueries(lxViewer);
					boolean isInserted=recordQuery.actionInsertRecord(lexeme, glosePaneList);
					if(isInserted){
//						((LexiconViewer) lxViewer).updateLexicon2();
						((LexiconViewer) lxViewer).updateLexiconChange();
						LexiconDocumentData.incrementMaxId();
						LexiconDocumentData.clearAllLists();
						JOptionPane.showMessageDialog(this,"The record '"+lexeme+"' was successfully inserted","Record Inserted",JOptionPane.INFORMATION_MESSAGE);
						clearInformations();
						this.setVisible(false);
						this.dispose();
					}
					else{
						JOptionPane.showMessageDialog(this, "Record insertion failed\n Please check your datas", "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
				else{
					JOptionPane.showMessageDialog(this, "Record insertion failed\n Please fill in the lexeme name", "Error", JOptionPane.ERROR_MESSAGE);				
				}
			}
			else{
				InsertParseRecordPanel insertParseRecordPanel=(InsertParseRecordPanel)tabPane.getSelectedComponent();
				String lexeme=insertParseRecordPanel.getLexemeTextField().getText();
				if(!lexeme.isEmpty()){
					List<JComboBox> uFormCbBoxList=insertParseRecordPanel.getUFormCbBoxList();
					LXRecordQueries recordQuery=new LXRecordQueries(lxViewer);
					boolean isInserted=recordQuery.actionInsertParseRecord(lexeme, uFormCbBoxList);
					if(isInserted){
						((LexiconViewer) lxViewer).updateLexiconChange();
						//((LexiconViewer) lxViewer).updateLexicon();
						LexiconDocumentData.incrementMaxId();
						LexiconDocumentData.clearAllLists();
						JOptionPane.showMessageDialog(this,"The record '"+lexeme+"' was successfully inserted","Record Inserted",JOptionPane.INFORMATION_MESSAGE);					
						clearInformations();
						this.setVisible(false);
						this.dispose();
					}
					else{
						JOptionPane.showMessageDialog(this, "Record insertion failed\n Please check your datas", "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
				else{
					JOptionPane.showMessageDialog(this, "Record insertion failed\n Please fill in the lexeme name", "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}

	}
}
