package mpi.eudico.client.annotator.lexica.record;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JOptionPane;

import mpi.eudico.client.annotator.lexica.LexiconDocumentData;
import mpi.eudico.client.annotator.viewer.AbstractViewer;
import mpi.eudico.client.annotator.viewer.LexiconViewer;

import org.jdom.Element;

/**
 * Contains Panel so as to Insert variant of a record or variant with an underlying-form
 * @author Coralie
 *
 */
public class InsertVariantDataDialog extends InsertLXDataDialog implements ActionListener{

	/**
	 * Empty constructor
	 */
	public InsertVariantDataDialog(AbstractViewer viewer){
		super(viewer);
	}
	
	/**
	 * Constructor with the information to insert
	 * @param lexeme
	 */
	public InsertVariantDataDialog(AbstractViewer viewer, String lexeme){
		super(viewer, lexeme);
	}
	
	@Override
	protected void clearInformations() {
		tabPane.setComponentAt(0, new InsertVariantToLXRecordPanel(lxViewer));
		tabPane.setComponentAt(1, new InsertParseVariantToLXRecordPanel(lxViewer));
		getRootPane().repaint();
		pack();
		setLocationRelativeTo(getParent());
	}

	@Override
	protected void updateLocale(){
		super.updateLocale();
		setTitle("Insert Variant to Lexicon Data");
		titleLabel.setText("Insert Variant");
	}

	@Override
	protected void initTabPane() {
		tabPane.addTab("Insert Variant", new InsertVariantToLXRecordPanel(lxViewer, lexeme));
		tabPane.addTab("Insert Underlying-Form", new InsertParseVariantToLXRecordPanel(lxViewer, lexeme));
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==cancelButton){
			this.setVisible(false);
			this.dispose();
		} 
		else if(e.getSource()==saveLexiconButton){
			//add a variant to the selected record
			if(tabPane.getSelectedComponent() instanceof InsertVariantToLXRecordPanel){
				InsertVariantToLXRecordPanel variantPanel=(InsertVariantToLXRecordPanel)tabPane.getSelectedComponent();
				String variant=variantPanel.getLexemeTextField().getText();
				if(!variant.isEmpty()&&variantPanel.getSelectedLexemeInComboBox()!=null){
					String derInfo=variantPanel.getDerivationInformation();
					Element lexemeParent=variantPanel.getSelectedLexemeInComboBox();
					Element lexicalEntry=lexemeParent.getParentElement();
					String id=lexicalEntry.getAttributeValue(LexiconDocumentData.ID);
					LXRecordQueries recordQuery=new LXRecordQueries(lxViewer, id);
					boolean isInserted=recordQuery.actionAddVariant(variant, null, derInfo);
					if(isInserted){
						((LexiconViewer) lxViewer).updateLexiconChange();
						LexiconDocumentData.clearAllLists();
						JOptionPane.showMessageDialog(this,"The variant '"+variant+"' was successfully added to the record '"+lexemeParent.getText()+"'","Record Inserted",JOptionPane.INFORMATION_MESSAGE);
						clearInformations();
						this.setVisible(false);
						this.dispose();
					}
					else{
						JOptionPane.showMessageDialog(this, "Variant insertion failed\n Please check your datas", "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
				else{
					JOptionPane.showMessageDialog(this, "Variant insertion failed\n Please fill in the variant name or\n select the record associated to this variant", "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
			//add a variant with an underlying-form to the selected record
			else{
				InsertParseVariantToLXRecordPanel parseVariantPanel=(InsertParseVariantToLXRecordPanel)tabPane.getSelectedComponent();
				String variant=parseVariantPanel.getLexemeTextField().getText();
				if(!variant.isEmpty()&&parseVariantPanel.getSelectedLexemeInComboBox()!=null){
					Element lexemeParent=parseVariantPanel.getSelectedLexemeInComboBox();
					Element lexicalEntry=lexemeParent.getParentElement();
					String id=lexicalEntry.getAttributeValue(LexiconDocumentData.ID);
					List<JComboBox> uFormCbBoxList=parseVariantPanel.getUFormCbBoxList();
					LXRecordQueries recordQuery=new LXRecordQueries(lxViewer, id);
					boolean isInserted=recordQuery.actionAddVariant(variant, uFormCbBoxList, "");
					if(isInserted){
						((LexiconViewer) lxViewer).updateLexiconChange();
						LexiconDocumentData.incrementMaxId();
						LexiconDocumentData.clearAllLists();
						JOptionPane.showMessageDialog(this,"The parsing variant '"+variant+"' was successfully associated to the record '"+lexemeParent.getText()+"'","Record Inserted",JOptionPane.INFORMATION_MESSAGE);
						clearInformations();
						this.setVisible(false);
						this.dispose();
					}
					else{
						JOptionPane.showMessageDialog(this, "Variant insertion failed\n Please check your datas", "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
				else{
					JOptionPane.showMessageDialog(this, "Variant insertion failed\n Please fill in the variant name or\n select the record associated to this variant", "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
			
		}
		
	}

}
