package mpi.eudico.client.annotator.lexica.record;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.jdom.Element;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.lexica.LexiconDocumentData;
import mpi.eudico.client.annotator.viewer.AbstractViewer;

/**
 * Panel of the first tab from the insertVariant frame
 * @author Coralie
 *
 */
public class InsertVariantToLXRecordPanel extends LXRecordPanel implements ActionListener{
	protected JLabel variantLb;
	protected int lexemeSelectedIndex=0;
	protected JButton showLexiconButton;
	protected JComboBox lexemeCbBox;
	private JPanel derivationPane;
	private JTextField derTxtField;
	
	
	public InsertVariantToLXRecordPanel(AbstractViewer viewer){
		super(viewer);
	}
	
	public InsertVariantToLXRecordPanel(AbstractViewer viewer, String lexeme){
		super(viewer, lexeme);
	}
	
	@Override
	protected void initComponents(){
		super.initComponents();
		Insets insets = new Insets(2, 6, 2, 6);
		GridBagLayout gridbag=new GridBagLayout();
		setLayout(gridbag);
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.insets=insets;
		gbc.anchor=GridBagConstraints.WEST;
		updateLexiconPane();
		add(lexiconPane, gbc);
		gbc.gridx=0;
		gbc.gridy=1;
		
		updateDerivationPane();
		gbc.gridx=0;
		gbc.gridy=2;
		add(derivationPane, gbc);
		
		updateLocale();
	}
	
	 @Override
	protected void updateLocale() {
		 lexemeLb.setText(ElanLocale.getString("VariantPanel.JLabel.WordForm"));
		 lexemeTxtField.setText(lexemeText);
	 }

	/**
	 * Gives the possibility to add an information on the variant inserted (such as PL, F, 3SG etc.)
	 * that will substitute the DER information of the gloss lexical entry (if its already exist)
	 */
	private void updateDerivationPane(){
		derivationPane=new JPanel();
		Insets insets = new Insets(2, 6, 2, 6);
		GridBagLayout gridbag=new GridBagLayout();
		derivationPane.setLayout(gridbag);
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.insets=insets;
		JLabel derLabel=new JLabel(ElanLocale.getString("VariantPanel.JLabel.Slash"));
		derivationPane.add(derLabel, gbc);
		gbc.gridx=1;
		derTxtField=new JTextField(3);
		derivationPane.add(derTxtField, gbc);
			
	}
	
	protected void updateLexiconPane(){
		lexiconPane=new JPanel();
		Insets insets = new Insets(3, 6, 3, 6);
		GridBagLayout gridbag=new GridBagLayout();
		lexiconPane.setLayout(gridbag);
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.insets=insets;
		gbc.anchor=GridBagConstraints.WEST;
        lexiconPane.add(lexemeLb, gbc);
        gbc.gridx=1;
        lexiconPane.add(lexemeTxtField, gbc);
        gbc.gridx=2;
        variantLb=new JLabel(ElanLocale.getString("LexiconViewer.RecordDlg.Label.IsAVariant"));
        lexiconPane.add(variantLb, gbc);
        gbc.gridx=3;
        // UTD : 100912 changed JComboBox to JSuggestComboBox
        lexemeCbBox=new JSuggestComboBox(LexiconDocumentData.getMorphemesToSelect(false).toArray());
        lexemeCbBox.setRenderer(new JDomElementListCellRenderer());
        lexemeCbBox.setSelectedIndex(lexemeSelectedIndex);
        lexiconPane.add(lexemeCbBox, gbc);
        gbc.gridx=4;
        showLexiconButton=new JButton(ElanLocale.getString("LexiconViewer.Button.Show"));
        showLexiconButton.addActionListener(this);
        lexiconPane.add(showLexiconButton, gbc);
	}
	
	public Element getSelectedLexemeInComboBox(){
		return (Element)lexemeCbBox.getSelectedItem();
	}
	
	public String getDerivationInformation(){
		return derTxtField.getText();
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==showLexiconButton){
			Element elementToShow=(Element)lexemeCbBox.getSelectedItem();
			ShowEditLXDataDialog dialog=new ShowEditLXDataDialog(lxViewer, elementToShow);
			dialog.setVisible(true);
		}
	}
}
