package mpi.eudico.client.annotator.lexica.record;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.ComboBoxEditor;
import javax.swing.ComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JTextField;

import org.jdom.Element;


/**
 * Une surcharge de JComboBox utilisant la classe ComboKeyHandler pour
 * proposer de l'auto-suggestion.
 *  
 * @author Pierre DITTGEN
 */
public class JSuggestComboBox extends JComboBox {

	/** Serial version uid. */
	private static final long serialVersionUID = 4800166218268693934L;

	/**
	 * Constructor
	 */
	public JSuggestComboBox() {
		super();
		init();
	}
	
	/**
	 * Constructor
	 */
	public JSuggestComboBox(ComboBoxModel model) {
		super(model);
		init();
	}
	
	/**
	 * Constructor
	 */
	public JSuggestComboBox(Object[] items) {
		super(items);
		init();
	}
	
	/**
	 * Constructor
	 */
	public JSuggestComboBox(Vector<?> items) {
		super(items);
		init();
	}

	/**
	 * Le petit plus pour l'auto-suggest
	 */
	private void init() {
		setEditor(new SuggestComboBoxEditor());
		setEditable(true);
		setSelectedIndex(-1);
        JTextField field = (JTextField)getEditor().getEditorComponent();
        field.setText("");
        field.addKeyListener(new ComboKeyHandler(this));
	}
}
/**
 * Combo box editor
 */
class SuggestComboBoxEditor implements ComboBoxEditor {

	private JTextField textField;
	private Element elt = null;

	public SuggestComboBoxEditor() {
		textField = new JTextField();
	}
	
	@Override
	public void addActionListener(ActionListener l) {
		// nothing
	}

	@Override
	public Component getEditorComponent() {
		return textField;
	}

	@Override
	public Object getItem() {
		return elt;
	}

	@Override
	public void removeActionListener(ActionListener l) {
		throw new UnsupportedOperationException();
	}

	@Override
	public void selectAll() {
		throw new UnsupportedOperationException();
	}

	@Override
	public void setItem(Object anObject) {
		// Remise à zéro
		if (anObject == null) {
			elt = null;
			textField.setText("");
			return;
		}
		
		// Une string ?
		if (anObject instanceof String) {
			textField.setText((String)anObject);
			elt = null;
			return;
		}
			
		// Un élément ?
		if (anObject instanceof Element) {
			elt = (Element)anObject;
			textField.setText(elt.getText());
		}
	}
}

