package mpi.eudico.client.annotator.lexica.record;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.swing.JComboBox;

import mpi.eudico.client.annotator.lexica.LexiconDocumentData;
import mpi.eudico.client.annotator.lexica.ParsingMorpheme;
import mpi.eudico.client.annotator.viewer.AbstractViewer;

import org.jdom.Attribute;
import org.jdom.Element;

/**Insert, Delete or Modify datas from the current lexicon document
 * 
 * @author Coralie
 *
 */
public class LXRecordQueries {
	private AbstractViewer lxViewer;
	private String id;
	private Element lexicalEntry;
	private Element senseElmt;
	private Element morphemeElmt;
	
	public LXRecordQueries(AbstractViewer viewer){
		lxViewer=viewer;
		id=LexiconDocumentData.MAX_ID+"";
	}
	
	public LXRecordQueries(AbstractViewer viewer, String id){
		lxViewer=viewer;
		this.id=id;
	}
	
	/**
	 * Add a variant to the current element
	 * @param element
	 * @return
	 */
	public boolean actionAddVariant(String variant, List<JComboBox> uFormCbBoxList, String derInfo){
		boolean isValid=true;
		Element lexemeToModify=LexiconDocumentData.getElement(id);
		lexicalEntry=lexemeToModify.getParentElement();
		Element formElmt=lexicalEntry.getChild(LexiconDocumentData.FORM);
		Element altFormElmt=addVariantToLexicon(variant, derInfo);
		if(uFormCbBoxList!=null){
			Element morphElmt=new Element(LexiconDocumentData.MORPHOLOGY);
			for(int i=0; i<uFormCbBoxList.size(); i++){
				JComboBox comboBox=uFormCbBoxList.get(i);
				Element cbBoxElmt=(Element)comboBox.getSelectedItem();
				if(cbBoxElmt==null){
					isValid=false;
				} else{
					Element segmentElmt=new Element(LexiconDocumentData.SEGMENT);
					segmentElmt.setText(cbBoxElmt.getText());
					Element associatedLexEntryElmt=cbBoxElmt.getParentElement();
					String associatedId=associatedLexEntryElmt.getAttributeValue(LexiconDocumentData.ID);
					Attribute refAtt=new Attribute(LexiconDocumentData.REFERENCE, associatedId);
					segmentElmt.setAttribute(refAtt);
					morphElmt.addContent(segmentElmt);
				}
			}
			altFormElmt.addContent(morphElmt);
		}
		if(isValid){
			formElmt.addContent(altFormElmt);
			LexiconDocumentData.isModified(true);
			LexiconDocumentData.setModifiedElementId(Integer.parseInt(id));
		}

		return isValid;
	}
	
	/**
	 * Add the written form variant
	 * @param variant
	 * @return
	 */
	private Element addVariantToLexicon(String variant, String derInfo){
		Element altFormElmt=new Element(LexiconDocumentData.ALTERNATE_FORM);
		Element wFormElmt=new Element(LexiconDocumentData.WRITTENFORM);
		wFormElmt.setText(variant);
		if(!derInfo.isEmpty()&&derInfo!=null){
			Attribute derAtt=new Attribute(LexiconDocumentData.DERIVATION, derInfo);
			wFormElmt.setAttribute(derAtt);
		}
		altFormElmt.addContent(wFormElmt);
		
		return altFormElmt;
	}
	
	/**
	 * Edit variant
	 * @param variant
	 * @param parseList
	 * @return
	 */
	public boolean actionEditVariant(String variant, Element variantToModify, List<LXSubPanel> parseList, String derInfo){
		boolean isValid=true;
		
		Element lexemeToModify=LexiconDocumentData.getElement(id);
		lexicalEntry=lexemeToModify.getParentElement();
		Element formElmt=lexicalEntry.getChild(LexiconDocumentData.FORM);
		Element altFormElmt=addVariantToLexicon(variant, derInfo);
		
		Element morphElmt=new Element(LexiconDocumentData.MORPHOLOGY);
		if(!parseList.isEmpty()){
			int nullElmtCmpt=0;
			for(LXSubPanel parsePane:parseList){
				Element uFormElmt=parsePane.getSelectedUFormElement();
				if(uFormElmt==null){
					nullElmtCmpt++;
				} else{
					String uFormTxt=uFormElmt.getText();
					Element lexicalUFormElmt=uFormElmt.getParentElement();
					String refId=lexicalUFormElmt.getAttributeValue(LexiconDocumentData.ID);
					Element segmentElmt=new Element(LexiconDocumentData.SEGMENT);
					segmentElmt.setText(uFormTxt);
					Attribute refIdAtt=new Attribute(LexiconDocumentData.REFERENCE, refId);
					segmentElmt.setAttribute(refIdAtt);
					morphElmt.addContent(segmentElmt);
				}
			}
//			for(LXSubPanel parsePane:parseList){
//				Element uFormElmt=parsePane.getSelectedUFormElement();
//				if(uFormElmt==null){
//					nullElmtCmpt++;
//				} else{
//					String uFormTxt=uFormElmt.getText();
//					Element lexicalUFormElmt=uFormElmt.getParentElement();
//					String refId=lexicalUFormElmt.getAttributeValue(LexiconDocumentData.ID);
//					Element segmentElmt=new Element(LexiconDocumentData.SEGMENT);
//					segmentElmt.setText(uFormTxt);
//					Attribute refIdAtt=new Attribute(LexiconDocumentData.REFERENCE, refId);
//					segmentElmt.setAttribute(refIdAtt);
//					morphElmt.addContent(segmentElmt);
//				}
//			}

			if(nullElmtCmpt>0&&nullElmtCmpt!=parseList.size()){
				isValid=false;
			} 
		}
		
		if(isValid){
			if(formElmt.getChildren(LexiconDocumentData.ALTERNATE_FORM).size()>0){
				List<Element> altFormList=formElmt.getChildren(LexiconDocumentData.ALTERNATE_FORM);
				for(int i=0; i<altFormList.size(); i++){
					if(altFormList.get(i)==variantToModify.getParentElement()){
						formElmt.removeContent(altFormList.get(i));
						break;
					}
				}
			}
			if(!morphElmt.getChildren().isEmpty()){
				altFormElmt.addContent(morphElmt);
			}
			formElmt.addContent(altFormElmt);
			LexiconDocumentData.setModifiedElementId(Integer.parseInt(id));
			LexiconDocumentData.isModified(true);
		}
		return isValid;
	}
	
	/**
	 * Edit parse informations of the current lexical Entry
	 * @param lexeme
	 * @param parseList
	 */
	public boolean actionEditParseRecord(String lexeme, List<LXSubPanel> parseList, List<LXSubPanel> variantList){
		boolean isValid=true;
		
		morphemeElmt=LexiconDocumentData.getElement(id);
		if(morphemeElmt.getText().isEmpty()){
			isValid=false;
		}
		lexicalEntry=morphemeElmt.getParentElement();
		
		Element formElmt=new Element(LexiconDocumentData.FORM);
		if (!variantList.isEmpty()){
			for(LXSubPanel variantPane:variantList){
				Element altFormElmt=new Element(LexiconDocumentData.ALTERNATE_FORM);
				String wForm=variantPane.getVariantTextField().getText();
				Element writtenFormElmt=new Element(LexiconDocumentData.WRITTENFORM);
				if(wForm.isEmpty()){
					isValid=false;
				}
				else{
					writtenFormElmt.setText(wForm);
					altFormElmt.addContent(writtenFormElmt);
					if(variantPane.getReferences()!=null){
						Element morphElmt=new Element(LexiconDocumentData.MORPHOLOGY);
						morphElmt.addContent(variantPane.getReferences());
						altFormElmt.addContent(morphElmt);
					}
					formElmt.addContent(altFormElmt);
				}
			}
		}
		
		Element morphElmt=new Element(LexiconDocumentData.MORPHOLOGY);
		int parseNum=0;
		for(LXSubPanel parsePane:parseList){
			Element uFormElmt=parsePane.getSelectedUFormElement();
			if(uFormElmt==null){
				isValid=false;
			} else{
				String uFormTxt=uFormElmt.getText();
				Element lexicalUFormElmt=uFormElmt.getParentElement();
				String refId=lexicalUFormElmt.getAttributeValue(LexiconDocumentData.ID);
				Element segmentElmt=new Element(LexiconDocumentData.SEGMENT);
				segmentElmt.setText(uFormTxt);
				Attribute refIdAtt=new Attribute(LexiconDocumentData.REFERENCE, refId);
				segmentElmt.setAttribute(refIdAtt);
				morphElmt.addContent(segmentElmt);
				parseNum++;
			}
		}
		
		if(parseNum<2){
			isValid=false;
		}
		
		if(isValid){
			morphemeElmt.setText(lexeme);
			if(morphemeElmt.getAttribute(LexiconDocumentData.TYPE).equals(LexiconDocumentData.LEM)){
				Attribute typeAtt=new Attribute(LexiconDocumentData.TYPE, LexiconDocumentData.WF);
				morphemeElmt.setAttribute(typeAtt);
			}
			if(lexicalEntry.getChild(LexiconDocumentData.FORM)!=null){
				lexicalEntry.removeChild(LexiconDocumentData.FORM);
			}
			lexicalEntry.addContent(formElmt);
			Element senseElmt=lexicalEntry.getChild(LexiconDocumentData.SENSE);
			senseElmt.removeChildren(LexiconDocumentData.GLOSE);
			lexicalEntry.removeChild(LexiconDocumentData.MORPHOLOGY);
			lexicalEntry.addContent(morphElmt);
			LexiconDocumentData.setModifiedElementId(Integer.parseInt(id));
			LexiconDocumentData.isModified(true);
		}
		
		return isValid;
	}
	
	
	/**
	 * Edit glose informations of the current lexical Entry
	 * @param lexeme
	 * @param gloseList
	 */
	public boolean actionEditGloseRecord(String lexeme, List<LXSubPanel> gloseList, List<LXSubPanel> variantList){
		boolean isValid=true;
		
		List<Element> gloseElmtList=new ArrayList<Element>();
		morphemeElmt=LexiconDocumentData.getElement(id);
		if(morphemeElmt.getText().isEmpty()){
			isValid=false;
		}
		lexicalEntry=morphemeElmt.getParentElement();
		
		Element formElmt=new Element(LexiconDocumentData.FORM);
		if (!variantList.isEmpty()){
			for(LXSubPanel variantPane:variantList){
				Element altFormElmt=new Element(LexiconDocumentData.ALTERNATE_FORM);
				String wForm=variantPane.getVariantTextField().getText();
				Element writtenFormElmt=new Element(LexiconDocumentData.WRITTENFORM);
				if(wForm.isEmpty()){
					isValid=false;
				}
				else{
					writtenFormElmt.setText(wForm);
					if(!variantPane.getVariantDerTextField().getText().isEmpty()){
						Attribute derAtt=new Attribute(LexiconDocumentData.DERIVATION, variantPane.getVariantDerTextField().getText());
						writtenFormElmt.setAttribute(derAtt);
					}
					altFormElmt.addContent(writtenFormElmt);
					if(variantPane.getReferences()!=null){
						Element morphElmt=new Element(LexiconDocumentData.MORPHOLOGY);
						for(Element reference:variantPane.getReferences()){
							String segmentId=reference.getAttributeValue(LexiconDocumentData.REFERENCE);
							Element segmentElmt=new Element(LexiconDocumentData.SEGMENT);
							segmentElmt.setText(reference.getText());
							Attribute refAtt=new Attribute(LexiconDocumentData.REFERENCE, segmentId);
							segmentElmt.setAttribute(refAtt);
							morphElmt.addContent(segmentElmt);
						}
						altFormElmt.addContent(morphElmt);
					}
					formElmt.addContent(altFormElmt);
				}
			}
		}
		
		int gloseNum=0;
		for(LXSubPanel glosePane:gloseList){
			String glose=glosePane.getGloseTxtField().getText();
			String tam=glosePane.getTamTxtField().getText();
			String tierX=glosePane.getTierXTxtField().getText();
			if(!(glose.isEmpty()&&tam.isEmpty()&&tierX.isEmpty())){
				gloseNum++;
				gloseElmtList.add(addGloseElement(glose, tam, tierX));
			}
		}
		if(gloseNum==0){
			System.out.println("No ge/rx element");
			isValid=false;
		}
		if(isValid){
			morphemeElmt.setText(lexeme);
			Attribute typeAtt=new Attribute(LexiconDocumentData.TYPE, getTypeValue(lexeme));
			morphemeElmt.setAttribute(typeAtt);
			lexicalEntry.removeChild(LexiconDocumentData.MORPHOLOGY);
			if(lexicalEntry.getChild(LexiconDocumentData.FORM)!=null){
				lexicalEntry.removeChild(LexiconDocumentData.FORM);
			}
			lexicalEntry.addContent(formElmt);
			senseElmt=lexicalEntry.getChild(LexiconDocumentData.SENSE);
			senseElmt.removeChildren(LexiconDocumentData.GLOSE);
			for(Element gloseElmt:gloseElmtList){
				senseElmt.addContent(gloseElmt);
			}
			LexiconDocumentData.setModifiedElementId(Integer.parseInt(id));
			LexiconDocumentData.isModified(true);
		}
		
		return isValid;
	}
	
	/**
	 * Edit the references for the segments of a lexicon
	 * @param parsingMorpheme
	 * @param references
	 * @return
	 */
	public boolean actionEditParseReferences(ParsingMorpheme parsingMorpheme, List<String> references){
		boolean isValid=true;
		Element elementToModify=null;
		
		Element morphologyElmt=null;
		
		morphemeElmt=LexiconDocumentData.getElement(id);
		if(morphemeElmt.getText().isEmpty()){
			isValid=false;
		}
		else{
			lexicalEntry=morphemeElmt.getParentElement();
			String morphemeToParse=parsingMorpheme.getMorphemeToParse();
			if(morphemeElmt.getText().equals(morphemeToParse)){
//				Element morphElmt=new Element(LexiconDocumentData.MORPHOLOGY);
//				List<String> segmentList=parsingMorpheme.getLexiconMorphemesList();
//				for(int i=0; i<segmentList.size(); i++){
//					Element segmentElmt=new Element(LexiconDocumentData.SEGMENT);
//					segmentElmt.setText(segmentList.get(i));
//					Attribute refAtt=new Attribute(LexiconDocumentData.REFERENCE, references.get(i));
//					segmentElmt.setAttribute(refAtt);
//					morphElmt.addContent(segmentElmt);
//				}
				morphologyElmt=createMorphElemt(parsingMorpheme, references);
				elementToModify=lexicalEntry;
			} else{
				Element formElmt=lexicalEntry.getChild(LexiconDocumentData.FORM);
				if(formElmt!=null){
					List<Element> altFormElmtList=formElmt.getChildren(LexiconDocumentData.ALTERNATE_FORM);
					if(!altFormElmtList.isEmpty()){
						for(Element altFormElmt:altFormElmtList){
							Element morphElmt=altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY);
							if(morphElmt!=null){
								Element wFormElmt=altFormElmt.getChild(LexiconDocumentData.WRITTENFORM);
								if(wFormElmt.getText().equals(morphemeToParse)){
									morphologyElmt=createMorphElemt(parsingMorpheme, references);
									elementToModify=altFormElmt;
									break;
								}
							}
						}
					}
				}
			}

			if(elementToModify==null||morphologyElmt==null){
				isValid=false;
			}
		}
		
		if(isValid){
			elementToModify.removeChild(LexiconDocumentData.MORPHOLOGY);
			elementToModify.addContent(morphologyElmt);
			
			LexiconDocumentData.setModifiedElementId(Integer.parseInt(id));
			LexiconDocumentData.isModified(true);
		}
		
		return isValid;
	}
	
	private Element createMorphElemt(ParsingMorpheme parsingMorpheme, List<String> references){
		Element morphElmt=new Element(LexiconDocumentData.MORPHOLOGY);
		List<String> segmentList=parsingMorpheme.getLexiconMorphemesList();
		for(int i=0; i<segmentList.size(); i++){
			Element segmentElmt=new Element(LexiconDocumentData.SEGMENT);
			segmentElmt.setText(segmentList.get(i));
			Attribute refAtt=new Attribute(LexiconDocumentData.REFERENCE, references.get(i));
			segmentElmt.setAttribute(refAtt);
			morphElmt.addContent(segmentElmt);
		}
		return morphElmt;
	}
	
	
	/**
	 * Insert Lexicon Record
	 * @param morpheme
	 * @param gloseList
	 * @return true if the informations are valid
	 */
	public boolean actionInsertRecord(String lexeme, List<GloseTierXSubPane> gloseList){
		boolean isValid=true;

		int gloseNum=0;
		actionInsertMorpheme(lexeme);
		for(GloseTierXSubPane glosePane:gloseList){
			String glose=glosePane.getGloseTxtField().getText();
			String tam=glosePane.getTamTxtField().getText();
			String tierX=glosePane.getTierXTxtField().getText();
			if(!(glose.isEmpty()&&tam.isEmpty()&&tierX.isEmpty())){
				gloseNum++;
				senseElmt.addContent(addGloseElement(glose, tam, tierX));
			}
		}
		if(gloseNum==0){
			System.out.println("No ge/rx element");
			isValid=false;
		}
		if(isValid){
			addRecordToDocument();
		}
		return isValid;
	}
	
	public boolean actionInsertParseRecord(String lexeme, List<JComboBox> comboBoxList){
		boolean isValid=true;
		actionInsertMorpheme(lexeme);
		Element morphElmt=new Element(LexiconDocumentData.MORPHOLOGY);
		for(int i=0; i<comboBoxList.size(); i++){
			JComboBox comboBox=comboBoxList.get(i);
			Element cbBoxElmt=(Element)comboBox.getSelectedItem();
			if(cbBoxElmt==null){
				isValid=false;
			} else{
				Element lexicalEntryElmt=cbBoxElmt.getParentElement();
				String id=lexicalEntryElmt.getAttributeValue(LexiconDocumentData.ID);
				Element segmentElmt=new Element(LexiconDocumentData.SEGMENT);
				segmentElmt.setText(cbBoxElmt.getText());
				Attribute refAtt=new Attribute(LexiconDocumentData.REFERENCE, id);
				segmentElmt.setAttribute(refAtt);
				morphElmt.addContent(segmentElmt);
			}
		}
		lexicalEntry.addContent(morphElmt);
		if(isValid){
			if(morphemeElmt.getAttributeValue(LexiconDocumentData.TYPE).equals(LexiconDocumentData.LEM)){
				morphemeElmt.setAttribute(new Attribute(LexiconDocumentData.TYPE, LexiconDocumentData.WF));				
			}
			addRecordToDocument();
		}
		return isValid;
	}
	
	/**
	 * insert element for the new morpheme
	 * @param morpheme
	 */
	private void actionInsertMorpheme(String morpheme){
		 lexicalEntry=new Element(LexiconDocumentData.LEXICAL_ENTRY);
		 Attribute idAtt=new Attribute(LexiconDocumentData.ID, id);
		 lexicalEntry.setAttribute(idAtt);
		 SimpleDateFormat dateFormat=new SimpleDateFormat("dd/MMM/yyyy", Locale.ENGLISH);
		 Attribute dtAtt=new Attribute(LexiconDocumentData.DATE, dateFormat.format(new Date()));
		 lexicalEntry.setAttribute(dtAtt);
		 morphemeElmt=new Element(LexiconDocumentData.LEXEME);
		 morphemeElmt.setText(morpheme);
		 Attribute typeAtt=new Attribute(LexiconDocumentData.TYPE, getTypeValue(morpheme));
		 morphemeElmt.setAttribute(typeAtt);
		 lexicalEntry.addContent(morphemeElmt);
		 Element formElmt=new Element(LexiconDocumentData.FORM);
		 lexicalEntry.addContent(formElmt);
		 senseElmt=new Element(LexiconDocumentData.SENSE);
		 lexicalEntry.addContent(senseElmt);
	}
	
	/**
	 * Calculate the type Attribute Value 
	 * @param morpheme
	 * @return
	 */
	private String getTypeValue(String morpheme){
		String typeValue=LexiconDocumentData.LEM;
		 if(morpheme.startsWith("-")||morpheme.startsWith("=")){
			 typeValue=LexiconDocumentData.SUFF;
		 } else if(morpheme.endsWith("-")||morpheme.endsWith("=")){
			 typeValue=LexiconDocumentData.PREF;
		 } else if(morpheme.endsWith("_")){
			 typeValue=LexiconDocumentData.STEM;
		 }
		 return typeValue;
	}
	
	/**
	 * insert glose informations
	 * @param glose
	 * @param tam
	 * @param tierX
	 */
	private Element addGloseElement(String glose, String tam, String tierX){
		 Element gloseElmt=new Element(LexiconDocumentData.GLOSE);
		 gloseElmt.setText(glose);
		 Attribute langAtt=new Attribute(LexiconDocumentData.LANGUAGE, "en");
		 gloseElmt.setAttribute(langAtt);
		 if(!tam.isEmpty()){
			 Attribute tamAtt=new Attribute(LexiconDocumentData.DERIVATION, tam);
			 gloseElmt.setAttribute(tamAtt);
		 }
		 Attribute tierXAtt=new Attribute(LexiconDocumentData.RX, tierX);
		 gloseElmt.setAttribute(tierXAtt);
		 return gloseElmt;
	}
	
	/**
	 * Delete the selected record
	 * @return
	 */
	public boolean actionDeleteRecord(){
		boolean isValid=true;
		morphemeElmt=LexiconDocumentData.getElement(id);
		if(morphemeElmt==null){
			isValid=false;
		}
		if(isValid){
			Element lexicalEntry=morphemeElmt.getParentElement();
			Element rootElmt=LexiconDocumentData.DOCUMENT.getRootElement();
			rootElmt.removeContent(lexicalEntry);
			LexiconDocumentData.setDeletedElementId(Integer.parseInt(id));
			LexiconDocumentData.isModified(true);
		}
		
		return isValid;
	}
	
	/**
	 * Delete the selected Variant
	 * @return
	 */
	public boolean actionDeleteVariant(Element variantToDelete){
		boolean isValid=true;
		
		Element lexemeToModify=LexiconDocumentData.getElement(id);
		lexicalEntry=lexemeToModify.getParentElement();
		Element formElmt=lexicalEntry.getChild(LexiconDocumentData.FORM);
		
		if(formElmt.getChildren(LexiconDocumentData.ALTERNATE_FORM).size()>0){
			List<Element> altFormList=formElmt.getChildren(LexiconDocumentData.ALTERNATE_FORM);
			for(int i=0; i<altFormList.size(); i++){
				if(altFormList.get(i)==variantToDelete.getParentElement()){
					formElmt.removeContent(altFormList.get(i));
					LexiconDocumentData.setModifiedElementId(Integer.parseInt(id));
					LexiconDocumentData.isModified(true);
					break;
				}
			}
		}
		else{
			isValid=false;
		}
		
		return isValid;
	}
	
	/**
	 * 
	 */
	private void addRecordToDocument(){
		LexiconDocumentData.DOCUMENT.getRootElement().addContent(lexicalEntry);
		LexiconDocumentData.setAddedElementId(Integer.parseInt(id));
		LexiconDocumentData.isModified(true);
	}

	/**
	 * Insert a new morpheme in the uForm panel
	 * @param morphemePanel
	 */
	public Element actionInsertMorpheme(MorphemeSubPane morphemePane){
		boolean isValid=true;
		String morpheme=morphemePane.getMorphemeTxtField().getText();
		actionInsertMorpheme(morpheme);
		String glose=morphemePane.getGloseTxtField().getText();
		String tam=morphemePane.getTamTxtField().getText();
		String tierX=morphemePane.getTierXTxtField().getText();
		if(!glose.isEmpty()){
			senseElmt.addContent(addGloseElement(glose, tam, tierX));
		} else{
			System.out.println("No ge/rx element");
			isValid=false;
		}	
		if(isValid){
			addRecordToDocument();
			return morphemeElmt;
		}
		else{
			return null;
		}
	}
}
