package mpi.eudico.client.annotator.lexica.record;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.lexica.LexiconDocumentData;
import mpi.eudico.client.annotator.viewer.AbstractViewer;
import mpi.eudico.client.annotator.viewer.LexiconViewer;

import org.jdom.Element;

public class ShowEditLXDataDialog extends AbstractLXDataDialog implements ActionListener, ChangeListener{
	protected JButton editButton;
	protected JButton deleteButton;
	
	public ShowEditLXDataDialog(AbstractViewer viewer){
		super(viewer);
	}
	
	public ShowEditLXDataDialog(AbstractViewer viewer, Element selectedElement){
		super(viewer, selectedElement);
	}
	
	@Override
	protected void clearInformations() {
		tabPane.setComponentAt(0, new ShowLXRecordPanel(lxViewer));
		
		tabPane.setComponentAt(1, new EditLXRecordPanel(lxViewer));
		getRootPane().repaint();
		pack();
		setLocationRelativeTo(getParent());
		
	}

	@Override
	protected JPanel initDialogButtonPanel() {
		JPanel dialogButtonPanel=new JPanel();
        dialogButtonPanel.setLayout(new GridLayout(1, 2, 6, 0));
        
        cancelButton.addActionListener(this);
        dialogButtonPanel.add(cancelButton);
        editButton=new JButton();
        editButton.addActionListener(this);
        editButton.setEnabled(false);
        dialogButtonPanel.add(editButton);
        deleteButton=new JButton();
        deleteButton.addActionListener(this);
		deleteButton.setEnabled(false);
		dialogButtonPanel.add(deleteButton);
		
		return dialogButtonPanel;
	}
	
	@Override
	protected void updateLocale(){
		super.updateLocale();
		setTitle(ElanLocale.getString("ShowEditLXDataDialog.Dialog.Title"));
		titleLabel.setText(ElanLocale.getString("ShowEditLXDataDialog.Dialog.Title"));
		editButton.setText(ElanLocale.getString("EditRecordPanel.JButton.Edit"));
		deleteButton.setText(ElanLocale.getString("Button.Delete"));
	}

	@Override
	protected void initTabPane() {
		tabPane.addTab(ElanLocale.getString("LexiconViewer.Button.Show"), new ShowLXRecordPanel(lxViewer, selectedElement));
		tabPane.addTab(ElanLocale.getString("LexiconViewer.Button.Edit"), new EditLXRecordPanel(lxViewer, selectedElement));
		tabPane.addChangeListener(this);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==cancelButton){
			this.setVisible(false);
			this.dispose();
		}
		else if(e.getSource()==editButton){
			EditLXRecordPanel editRecordPanel=(EditLXRecordPanel)tabPane.getSelectedComponent();
			String lexeme=editRecordPanel.getLexeme();
			Element elementToModify=editRecordPanel.getSelectedElement();
			if(elementToModify!=null){
				Element lexicalEntryElmt=elementToModify.getParentElement();
				String id=lexicalEntryElmt.getAttributeValue(LexiconDocumentData.ID);
				
				//uForm element to modify
				if(editRecordPanel.isAParsingRecord()){
					List<LXSubPanel> subPaneList=editRecordPanel.getModifiedPane();
					LXRecordQueries query=new LXRecordQueries(lxViewer, id);
					boolean isModified=query.actionEditParseRecord(lexeme, subPaneList, editRecordPanel.getVariantSubPane());
					if(isModified){
						((LexiconViewer) lxViewer).updateLexiconChange();
						JOptionPane.showMessageDialog(this,"The record '"+lexeme+"' was successfully modified","Record Modified",JOptionPane.INFORMATION_MESSAGE);
						LexiconDocumentData.clearAllLists();
						clearInformations();
						this.setVisible(false);
						this.dispose();
					} else{
						JOptionPane.showMessageDialog(this, "Record edition failed\n Please check your datas", "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
				//gloss element to modify
				else{
					List<LXSubPanel> subPaneList=editRecordPanel.getModifiedPane();
					LXRecordQueries query=new LXRecordQueries(lxViewer, id);
					boolean isModified=query.actionEditGloseRecord(lexeme, subPaneList, editRecordPanel.getVariantSubPane());
					if(isModified){
						((LexiconViewer) lxViewer).updateLexiconChange();
						LexiconDocumentData.clearAllLists();
						JOptionPane.showMessageDialog(this,"The record '"+lexeme+"' was successfully modified","Record Modified",JOptionPane.INFORMATION_MESSAGE);
						clearInformations();
						this.setVisible(false);
						this.dispose();
					}
					else{
						JOptionPane.showMessageDialog(this, "Record edition failed\n Please check your datas", "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
			} else{
				JOptionPane.showMessageDialog(this, "Select a morpheme to modify", "Error", JOptionPane.ERROR_MESSAGE);
			}
		} else if(e.getSource()==deleteButton){
			int option=JOptionPane.showConfirmDialog(null, "Are you sure you want to delete this record ?", "Delete Record", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
			if(option==JOptionPane.OK_OPTION){
				EditLXRecordPanel editRecordPanel=(EditLXRecordPanel)tabPane.getSelectedComponent();
				String lexeme=editRecordPanel.getLexeme();
				Element elementToDelete=editRecordPanel.getSelectedElement();
				if(elementToDelete!=null){
					Element lexicalEntryElmt=elementToDelete.getParentElement();
					String id=lexicalEntryElmt.getAttributeValue(LexiconDocumentData.ID);
					LXRecordQueries query=new LXRecordQueries(lxViewer, id);
					boolean isDeleted=query.actionDeleteRecord();
					if(isDeleted){
						((LexiconViewer) lxViewer).updateLexiconChange();
						//((LexiconViewer) lxViewer).updateLexicon();
						JOptionPane.showMessageDialog(this,"The record '"+lexeme+"' was successfully deleted","Record Deleted",JOptionPane.INFORMATION_MESSAGE);
						LexiconDocumentData.clearAllLists();
						clearInformations();
						this.setVisible(false);
						this.dispose();
					}
					else{
						JOptionPane.showMessageDialog(this, "Record delete failed\n Please check your datas", "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
				else{
					JOptionPane.showMessageDialog(this, "Select a morpheme to delete", "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
		
	}
	
	/**
	 * UTD 05/09/12
	 * méthode modifié pour prendre en compte l'autocomplétion
	 */
	@Override
	public void stateChanged(ChangeEvent arg0) {
		//Show Lexicon tab
		if(tabPane.getSelectedComponent()==tabPane.getComponentAt(0)){
			Element showTabSelectedElement=((EditLXRecordPanel)tabPane.getComponentAt(1)).getSelectedElement();
			if(showTabSelectedElement!=null){
				tabPane.setComponentAt(0, new ShowLXRecordPanel(lxViewer, showTabSelectedElement));
				pack();
			}
			editButton.setEnabled(false);
			deleteButton.setEnabled(false);
		}
		//Edit Lexicon tab
		else{
			Element editTabSelectedElement=((ShowLXRecordPanel)tabPane.getComponentAt(0)).getSelectedElement();
			if(editTabSelectedElement!=null){
				tabPane.setComponentAt(1, new EditLXRecordPanel(lxViewer, editTabSelectedElement));
				pack();
			}
			editButton.setEnabled(true);
			deleteButton.setEnabled(true);
		}
		
	}
	
/*	@Override
	public void stateChanged(ChangeEvent e) {
		//Show Lexicon tab
		if(tabPane.getSelectedComponent()==tabPane.getComponentAt(0)){
			Element showTabSelectedElement=((EditLXRecordPanel)tabPane.getComponentAt(1)).getSelectedElement();
			if(showTabSelectedElement!=null){
				tabPane.setComponentAt(0, new ShowLXRecordPanel(lxViewer, showTabSelectedElement));
				pack();
			}
			editButton.setEnabled(false);
			deleteButton.setEnabled(false);
		}
		//Edit Lexicon tab
		else{
			Element editTabSelectedElement=((ShowLXRecordPanel)tabPane.getComponentAt(0)).getSelectedElement();
			if(editTabSelectedElement!=null){
				tabPane.setComponentAt(1, new EditLXRecordPanel(lxViewer, editTabSelectedElement));
				pack();
			}
			editButton.setEnabled(true);
			deleteButton.setEnabled(true);
		}
	}*/

}
