package mpi.eudico.client.annotator.lexica.record;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.lexica.LexiconDocumentData;
import mpi.eudico.client.annotator.viewer.AbstractViewer;
import mpi.eudico.client.annotator.viewer.LexiconViewer;

import org.jdom.Element;

public class ShowEditVariantDataDialog extends ShowEditLXDataDialog implements ActionListener, ChangeListener{
	
	public ShowEditVariantDataDialog(AbstractViewer viewer){
		super(viewer);
	}
	
	public ShowEditVariantDataDialog(AbstractViewer viewer, Element selectedElement){
		super(viewer, selectedElement);
	}
	
	@Override
	protected void updateLocale(){
		super.updateLocale();
		setTitle(ElanLocale.getString("ShowEditVariantDataDialog.Dialog.Title"));
		titleLabel.setText(ElanLocale.getString("ShowEditVariantDataDialog.Dialog.Title"));
	}

	@Override
	protected void initTabPane() {
		tabPane.addTab(ElanLocale.getString("ShowEditVariantDataDialog.Tab.Show"), new ShowVariantToRecordPanel(lxViewer, selectedElement));
		tabPane.addTab(ElanLocale.getString("ShowEditVariantDataDialog.Tab.Edit"), new EditVariantToRecordPanel(lxViewer, selectedElement));
		tabPane.addChangeListener(this);
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==cancelButton){
			this.setVisible(false);
			this.dispose();
		}
		else if(e.getSource()==editButton){
			EditVariantToRecordPanel editVariantPanel=(EditVariantToRecordPanel)tabPane.getSelectedComponent();
			String wordform=editVariantPanel.getWordform();
			Element variantToModify=editVariantPanel.getSelectedWordFormElement();
			Element elementToModify=editVariantPanel.getSelectedElement();
			if(wordform.isEmpty()){
				JOptionPane.showMessageDialog(this, "Variant name is empty", "Error", JOptionPane.ERROR_MESSAGE);
			}
			else{
				if(elementToModify!=null&&variantToModify!=null){
					Element lexicalEntryElmt=elementToModify.getParentElement();
					String id=lexicalEntryElmt.getAttributeValue(LexiconDocumentData.ID);
					List<LXSubPanel> subParsePaneList=editVariantPanel.getParsePane();
					String der=editVariantPanel.getDerInfo();
					LXRecordQueries query=new LXRecordQueries(lxViewer, id);
					boolean isModified=query.actionEditVariant(wordform, variantToModify, subParsePaneList, der);
					if(isModified){
						((LexiconViewer) lxViewer).updateLexiconChange();
						JOptionPane.showMessageDialog(this,"The variant '"+wordform+"' of the lexeme '"+elementToModify.getText()+"' was successfully modified","Record Modified",JOptionPane.INFORMATION_MESSAGE);
						LexiconDocumentData.clearAllLists();
						clearInformations();
						this.setVisible(false);
						this.dispose();
					} else{
						JOptionPane.showMessageDialog(this, "Variant edition failed\n Please check your datas", "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
				else{
					JOptionPane.showMessageDialog(this, "Select a morpheme to modify", "Error", JOptionPane.ERROR_MESSAGE);
				}
			} 
		}
		else if(e.getSource()==deleteButton){
			int option=JOptionPane.showConfirmDialog(null, "Are you sure you want to delete this variant ?", "Delete Variant", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
			if(option==JOptionPane.OK_OPTION){
				EditVariantToRecordPanel editVariantPanel=(EditVariantToRecordPanel)tabPane.getSelectedComponent();
				String wordform=editVariantPanel.getWordform();
				Element variantToDelete=editVariantPanel.getSelectedWordFormElement();
				Element elementToModify=editVariantPanel.getSelectedElement();
				if(elementToModify!=null&&variantToDelete!=null){
					Element lexicalEntryElmt=elementToModify.getParentElement();
					String id=lexicalEntryElmt.getAttributeValue(LexiconDocumentData.ID);
					LXRecordQueries query=new LXRecordQueries(lxViewer, id);
					boolean isDeleted=query.actionDeleteVariant(variantToDelete);
					if(isDeleted){
						((LexiconViewer) lxViewer).updateLexiconChange();
						JOptionPane.showMessageDialog(this,"The variant '"+wordform+"' of the lexeme '"+elementToModify.getText()+"' was successfully deleted","Variant Deleted",JOptionPane.INFORMATION_MESSAGE);
						LexiconDocumentData.clearAllLists();
						clearInformations();
						this.setVisible(false);
						this.dispose();
					} else{
						JOptionPane.showMessageDialog(this, "Delete variant failed\n Please check your datas", "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		}
		
	}
	
	@Override
	public void stateChanged(ChangeEvent e) {
		//Show
		if(tabPane.getSelectedComponent()==tabPane.getComponentAt(0)){
			Element showTabSelectedElement=((EditVariantToRecordPanel)tabPane.getComponentAt(1)).getSelectedWordFormElement();
			if(showTabSelectedElement!=null){
				tabPane.setComponentAt(0, new ShowVariantToRecordPanel(lxViewer, showTabSelectedElement));
				pack();
				//setLocationRelativeTo(getParent());
			}
			editButton.setEnabled(false);
			deleteButton.setEnabled(false);
		}
		//Edit
		else{			
			Element editTabSelectedElement=((ShowVariantToRecordPanel)tabPane.getComponentAt(0)).getSelectedWordFormElement();
			if(editTabSelectedElement!=null){
				tabPane.setComponentAt(1, new EditVariantToRecordPanel(lxViewer, editTabSelectedElement));
				pack();
				//setLocationRelativeTo(getParent());
			}
			editButton.setEnabled(true);
			deleteButton.setEnabled(true);
		}
	}

}
