package mpi.eudico.client.annotator.viewer;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Locale;
import java.util.Vector;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.SwingUtilities;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;

import org.jdom.*; 
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.lexica.InterlinearizationProcess;
import mpi.eudico.client.annotator.lexica.LXConfigurationDialog;
import mpi.eudico.client.annotator.lexica.LexiconDocumentData;
import mpi.eudico.client.annotator.lexica.LexiconFieldConfiguration;
import mpi.eudico.client.annotator.lexica.LexiconIdConnectionToTableContent;
import mpi.eudico.client.annotator.lexica.LexiconParser;
import mpi.eudico.client.annotator.lexica.LexiconRef;
import mpi.eudico.client.annotator.lexica.LexiconRenderer;
import mpi.eudico.client.annotator.lexica.LexiconTable;
import mpi.eudico.client.annotator.lexica.LexiconTableContent;
import mpi.eudico.client.annotator.lexica.LexiconTableModel;
import mpi.eudico.client.annotator.lexica.LexiconVariantContent;
import mpi.eudico.client.annotator.lexica.LexiconViewerPopUpMenu;
import mpi.eudico.client.annotator.lexica.ParseDocumentData;
import mpi.eudico.client.annotator.lexica.ParsingMorpheme;
import mpi.eudico.client.annotator.lexica.gui.LexiconRecordPanel;
import mpi.eudico.client.annotator.lexica.gui.LexiconSegmentationAnnotationPanel;
import mpi.eudico.client.annotator.lexica.gui.MorphemeBreakCharacterFrame;
import mpi.eudico.client.annotator.lexica.gui.TierAnnotationParameterFrame;
import mpi.eudico.client.annotator.lexica.gui.TierInterlinearizeParameterFrame;
import mpi.eudico.client.annotator.lexica.gui.TierRenameAnnotationFrame;
import mpi.eudico.client.annotator.lexica.gui.TierRenameInterlinearizeFrame;
import mpi.eudico.client.annotator.lexica.record.AnnotationTree;
import mpi.eudico.client.annotator.util.ClientLogger;
import mpi.eudico.client.annotator.util.ElanFileFilter;
import mpi.eudico.client.annotator.util.FileUtility;
import mpi.eudico.client.mediacontrol.ControllerEvent;
import mpi.eudico.server.corpora.clom.Annotation;
import mpi.eudico.server.corpora.clom.Tier;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.server.corpora.event.ACMEditEvent;
import mpi.eudico.server.corpora.event.ACMEditListener;

/**
 * Elements of the lexicon viewer panel
 * The interface contains :
 * - current JMenuBar
 * - splitPane divided in leftPanel and rightPanel
 * leftPanel contains the current JLexiconTable filled in by the  
 * @author Coralie
 * CC 31/03/2011 Try to give the lexicon the same font than the mot line
 */

public class LexiconViewer extends AbstractViewer implements SingleTierViewer, ACMEditListener, ActionListener, ItemListener { 
	private TranscriptionImpl transcription;  
	private String lexicalPath;
	private TierImpl tier;
	
	private JLabel lxPathLabel;
	private JSplitPane viewerContainer;
	private JPanel leftPanel;
	private JPanel rightPanel;
	
	//private LexiconSegmentationPanel lexiconSegmentationPanel;
	private LexiconSegmentationAnnotationPanel lexiconAnnotationProcessPanel;
	
	/** constant for lexicon source */
	static public final String LX_SOURCE="Interlinear.LexiconSource";
	static public final String PARSE_SOURCE="Interlinear.ParseSource";
	
	//interlinearize tier preferences
	static public final String INTERLINEARIZE_TIER_WORD="Interlinear.Tier.Word";
	static public final String INTERLINEARIZE_TIER_PARSE="Interlinear.Tier.Parse";
	static public final String INTERLINEARIZE_TIER_GLOSS="Interlinear.Tier.Gloss";
	static public final String INTERLINEARIZE_TIER_POS="Interlinear.Tier.Pos";
	private Font selectedFont;		//CC
	private Integer selectedSize;	
	private HashMap prefTierFonts;
	private HashMap prefTierSizes;
	
	//annotate tier preferences
	static public final String ANNOTATION_TIER_PARSE="Annotate.Tier.Parse";
	static public final String ANNOTATION_TIER_GLOSS="Annotate.Tier.Gloss";
	static public final String ANNOTATION_TIER_POS="Annotate.Tier.Pos";
	
	private LXConfigurationDialog configPanel;

	private LexiconTable lxTable;
	private LexiconTableModel lxTableModel;
	private LexiconViewerPopUpMenu popup;
	
	private JMenuBar menuBar;
	private JMenu menuLinkedFile;
	private JCheckBoxMenuItem parseLinkedFile;
	private JCheckBoxMenuItem lexiconLinkedFile;
	private JMenuItem exportLexiconItem;
	
	//action command values
	static private String CREATE_LEXICON="0";
	static private String OPEN_LEXICON="1";
	static private String IMPORT_LEXICON="2";
	static private String SAVE_LEXICON="3";
	static private String EXPORT_PARSE="4";
	static private String OPEN_PARSE="5";
	static private String CONFIGURE_INTERLINEARIZE_TIERS="6";
	static private String CONFIGURE_ANNOTATION_TIERS="7";
	//static private String CONFIGURE_TIERS="6";
	//static private String RENAME_TIERS="8";
	static private String RENAME_INTERLINEARIZE_TIERS="8";
	static private String RENAME_ANNOTATION_TIERS="9";
	static private String EXPORT_MERGE_PARSE="10";
	static private String EXPORT_LEXICON="11";
	static private String MORPHEME_BREAK_CHAR="12";
	
	//interlinearize default tier name
	static public String INTERLINEARIZE_WORD_TIER_NAME="mot";
	static public String INTERLINEARIZE_PARSE_TIER_NAME="mb";
	static public String INTERLINEARIZE_GLOSS_TIER_NAME="ge";
	static public String INTERLINEARIZE_POS_TIER_NAME="rx";
	
	//annotation default tier name
	static public String ANNOTATION_PARSE_TIER_NAME="mb";
	static public String ANNOTATION_GLOSS_TIER_NAME="ge";
	static public String ANNOTATION_POS_TIER_NAME="rx";
	
	static public int INTERLINEARIZE_MAP=0;
	static public int ANNOTATION_MAP=1;
	
	static private String TXT_EXT=".txt";
	static private String EAFL_EXT=".eafl";
	static private String EAFP_EXT=".eafp";
	
	//morpheme break characters preferences
	static public final String SUFFIX_CHAR="MorphemeBreakCharacter.Suffix";
	static public final String STEM_CHAR="MorphemeBreakCharacter.Stem";
	static public final String TRANS_BREAK_CHAR="MorphemeBreakCharacter.Transmit";
	
	static public final String DEFAULT_SUFFIX_VALUE="-=";
	static public final String DEFAULT_STEM_VALUE="_";
	static public final Integer DEFAULT_TRANSMIT_BREAK_CHAR=0;
	
	private InterlinearizationProcess interlinearizeProcess=null;
	private List<LexiconIdConnectionToTableContent> lexiconIdConnectToTableContentList;
	
	private List<LexiconTableContent> entries;
	private List<ParsingMorpheme> parsingMorphemeList;
	
	/**
	 * Creates a new LexiconViewer instance
	 *
	 * @param TranscriptionImpl
	 */
	public LexiconViewer(TranscriptionImpl transcription) {
		super();
		lexiconIdConnectToTableContentList=new ArrayList<LexiconIdConnectionToTableContent>();
		this.transcription = transcription;
		initComponents();
	}

	/**
	 * Components always visible 
	 */
	private void initComponents() {
		menuBar=new JMenuBar();
		JMenu lexiconMenu=new JMenu(ElanLocale.getString("LexiconViewer.JMenuBar.Lexicon"));
		JMenuItem createLexiconItem=new JMenuItem(ElanLocale.getString("LexiconViewer.JMenuBar.Lexicon.MenuItem.New"));
		createLexiconItem.setActionCommand(CREATE_LEXICON);
		createLexiconItem.addActionListener(this);
		JMenuItem openLexiconItem=new JMenuItem(ElanLocale.getString("LexiconViewer.JMenuBar.Lexicon.MenuItem.Open"));
		openLexiconItem.setActionCommand(OPEN_LEXICON);
		openLexiconItem.addActionListener(this);
		
		JMenuItem importLexiconItem=new JMenuItem(ElanLocale.getString("LexiconViewer.JMenuBar.Lexicon.MenuItem.Import"));
		importLexiconItem.setActionCommand(IMPORT_LEXICON);
		importLexiconItem.addActionListener(this);
		
		exportLexiconItem=new JMenuItem(ElanLocale.getString("LexiconViewer.JMenuBar.Lexicon.MenuItem.Export"));
		exportLexiconItem.setActionCommand(EXPORT_LEXICON);
		//exportLexiconItem.setEnabled(false);
		exportLexiconItem.addActionListener(this);
		
		JMenuItem saveLexiconItem=new JMenuItem(ElanLocale.getString("LexiconViewer.JMenuBar.Lexicon.MenuItem.Save"));
		saveLexiconItem.setActionCommand(SAVE_LEXICON);
		saveLexiconItem.addActionListener(this);
		
		lexiconMenu.add(createLexiconItem);
		lexiconMenu.add(openLexiconItem);
		lexiconMenu.add(importLexiconItem);
		lexiconMenu.add(exportLexiconItem);
		lexiconMenu.addSeparator();
		lexiconMenu.add(saveLexiconItem);
		menuBar.add(lexiconMenu);
		
		JMenu parseMenu=new JMenu(ElanLocale.getString("LexiconViewer.JMenuBar.Parse"));
		JMenuItem exportParseItem=new JMenuItem(ElanLocale.getString("LexiconViewer.JMenuBar.Parse.MenuItem.Export"));
		exportParseItem.setActionCommand(EXPORT_PARSE);
		exportParseItem.addActionListener(this);
		
		JMenuItem openParseItem=new JMenuItem(ElanLocale.getString("LexiconViewer.JMenuBar.Parse.MenuItem.Open"));
		openParseItem.setActionCommand(OPEN_PARSE);
		openParseItem.addActionListener(this);
		
		JMenuItem mergeParseItem=new JMenuItem(ElanLocale.getString("LexiconViewer.JMenuBar.Parse.MenuItem.ExportMerge"));
		mergeParseItem.setActionCommand(EXPORT_MERGE_PARSE);
		mergeParseItem.addActionListener(this);
		
		parseMenu.add(exportParseItem);
		parseMenu.add(openParseItem);
		parseMenu.add(mergeParseItem);
		menuBar.add(parseMenu);
		
		JMenu parameterMenu=new JMenu(ElanLocale.getString("LexiconViewer.JMenuBar.Parameters"));
		JMenu tierInterlinearizeParameters=new JMenu(ElanLocale.getString("LexiconViewer.JMenuBar.Parameters.Interlinearize.MenuItem.Tier"));
		JMenuItem renameInterlinearizeTiers=new JMenuItem(ElanLocale.getString("LexiconViewer.JMenuBar.Parameters.Interlinearize.MenuItem.Tier.Rename"));
		renameInterlinearizeTiers.setActionCommand(RENAME_INTERLINEARIZE_TIERS);
		renameInterlinearizeTiers.addActionListener(this);
		JMenuItem configureInterlinearizeTiers=new JMenuItem(ElanLocale.getString("LexiconViewer.JMenuBar.Parameters.Interlinearize.MenuItem.Tier.Config"));
		configureInterlinearizeTiers.setActionCommand(CONFIGURE_INTERLINEARIZE_TIERS);
		configureInterlinearizeTiers.addActionListener(this);
		
		JMenu tierAnnotationParameters=new JMenu(ElanLocale.getString("LexiconViewer.JMenuBar.Parameters.Annotation.MenuItem.Tier"));
		JMenuItem renameAnnotationTiers=new JMenuItem(ElanLocale.getString("LexiconViewer.JMenuBar.Parameters.Annotation.MenuItem.Tier.Rename"));
		renameAnnotationTiers.setActionCommand(RENAME_ANNOTATION_TIERS);
		renameAnnotationTiers.addActionListener(this);
		JMenuItem configureAnnotationTiers=new JMenuItem(ElanLocale.getString("LexiconViewer.JMenuBar.Parameters.Annotation.MenuItem.Tier.Config"));
		configureAnnotationTiers.setActionCommand(CONFIGURE_ANNOTATION_TIERS);
		configureAnnotationTiers.addActionListener(this);
		
		JMenuItem morphemeCharacter=new JMenuItem("Morpheme break character");
		morphemeCharacter.setActionCommand(MORPHEME_BREAK_CHAR);
		morphemeCharacter.addActionListener(this);
		
		parameterMenu.add(tierInterlinearizeParameters);
		tierInterlinearizeParameters.add(renameInterlinearizeTiers);
		tierInterlinearizeParameters.add(configureInterlinearizeTiers);
		
		parameterMenu.add(tierAnnotationParameters);
		tierAnnotationParameters.add(renameAnnotationTiers);
		tierAnnotationParameters.add(configureAnnotationTiers); 
		
		
		parameterMenu.add(morphemeCharacter);
		
		menuBar.add(parameterMenu);
		
		setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill=GridBagConstraints.BOTH;
		gbc.weighty =0.1; //0.3; CC
		gbc.weightx=1.0;
		gbc.gridx=0;
		gbc.gridy=0;
		add(menuBar, gbc);

		lxPathLabel = new JLabel();
		lxPathLabel.setFont(lxPathLabel.getFont().deriveFont(10f));

		leftPanel=new JPanel();
		rightPanel=new JPanel();

		viewerContainer=new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, leftPanel, rightPanel);
		viewerContainer.setOneTouchExpandable(true);
		gbc.gridx=0;
		gbc.gridy=1;
		gbc.weighty =0.7;
		add(viewerContainer, gbc);
		getPreferences();
		
		menuLinkedFile=new JMenu(ElanLocale.getString("LexiconViewer.JMenuBar.LinkedFile"));
		menuBar.add(menuLinkedFile);
		updateLinkedFile();
	}
	
	/**
	 * Load file for interlinearization and parameters if they are included in the pfx file
	 */
	private void getPreferences(){
		exportLexiconItem.setEnabled(false);
		ParseDocumentData.setDocument(null, null);
		LexiconDocumentData.setDocument(null,null);
		leftPanel.removeAll();
		rightPanel.removeAll();
		tierPreferences(); 		//CC 
		//selectedFont = new Font("Times New Roman", Font.PLAIN, 12);
		String parsePrefSource=(String)getPreference(PARSE_SOURCE, transcription);
		String lxPrefSource=(String)getPreference(LX_SOURCE, transcription);
		if(parsePrefSource!=null){
			File fParsePrefSource=new File(parsePrefSource);
			if(fParsePrefSource.exists()&&parsePrefSource.endsWith(EAFP_EXT)){
				SAXBuilder sxb=new SAXBuilder();
				try {
					ParseDocumentData.setDocument(sxb.build(fParsePrefSource), fParsePrefSource);
				}
				catch(Exception e){
					System.out.println("document build failed");
				}
			}
			if(lxPrefSource!=null){
				File fLxPrefSource=new File(lxPrefSource);
				if(fLxPrefSource.exists()&&lxPrefSource.endsWith(EAFL_EXT)){
					createLexiconGrid(lxPrefSource);
				}
				else{
					entries=new ArrayList<LexiconTableContent>();
					parsingMorphemeList=new ArrayList<ParsingMorpheme>();
					lxTableModel=new LexiconTableModel(entries);
					lxTable=new LexiconTable(lxTableModel);
					System.out.println("unable to open the file :"+lxPrefSource);
					initTable();
					placeTable();
				}
			}
			else if(lxPrefSource==null){
				entries=new ArrayList<LexiconTableContent>();
				parsingMorphemeList=new ArrayList<ParsingMorpheme>();
				lxTableModel=new LexiconTableModel(entries);
				lxTable=new LexiconTable(lxTableModel);
				initTable();
				placeTable();
			}
			lxTable.updateLocale();
			leftPanel.repaint();
			rightPanel.repaint();
		}
		else{
			if(lxPrefSource!=null){
				File f=new File(lxPrefSource);
				if(f.exists()&&lxPrefSource.endsWith(EAFL_EXT)){
					createLexiconGrid(lxPrefSource);
				}
				else{
					entries=new ArrayList<LexiconTableContent>();
					parsingMorphemeList=new ArrayList<ParsingMorpheme>();
					lxTableModel=new LexiconTableModel(entries);
					lxTable=new LexiconTable(lxTableModel);
					System.out.println("unable to open the file :"+lxPrefSource);
					initTable();
					placeTable();
				}
			}
			else if(lxPrefSource==null){
				entries=new ArrayList<LexiconTableContent>();
				parsingMorphemeList=new ArrayList<ParsingMorpheme>();
				lxTableModel=new LexiconTableModel(entries);
				lxTable=new LexiconTable(lxTableModel);
				initTable();
				placeTable();
			}
			lxTable.updateLocale();
			leftPanel.repaint();
			rightPanel.repaint();
		}
		
		//tierPreferences();
	}
	
	private void tierPreferences(){
		prefTierFonts = new HashMap();
		prefTierSizes = new HashMap();
		String interlinearizeWordTier=(String)getPreference(INTERLINEARIZE_TIER_WORD, transcription);
		String interlinearizeParseTier=(String)getPreference(INTERLINEARIZE_TIER_PARSE, transcription);
		String interlinearizeGlossTier=(String)getPreference(INTERLINEARIZE_TIER_GLOSS, transcription);
		String interlinearizePosTier=(String)getPreference(INTERLINEARIZE_TIER_POS, transcription);
		if(interlinearizeWordTier==null){
			setPreference(INTERLINEARIZE_TIER_WORD, INTERLINEARIZE_WORD_TIER_NAME, transcription);
		}
		if(interlinearizeParseTier==null){
			setPreference(INTERLINEARIZE_TIER_PARSE, INTERLINEARIZE_PARSE_TIER_NAME, transcription);
		}
		if(interlinearizeGlossTier==null){
			setPreference(INTERLINEARIZE_TIER_GLOSS, INTERLINEARIZE_GLOSS_TIER_NAME, transcription);
		}
		if(interlinearizePosTier==null){
			setPreference(INTERLINEARIZE_TIER_POS, INTERLINEARIZE_POS_TIER_NAME, transcription);
		}

		String annotateParseTier=(String)getPreference(ANNOTATION_TIER_PARSE, transcription);
		String annotateGlossTier=(String)getPreference(ANNOTATION_TIER_GLOSS, transcription);
		String annotatePosTier=(String)getPreference(ANNOTATION_TIER_POS, transcription);
		if(annotateParseTier==null){
			setPreference(ANNOTATION_TIER_PARSE, ANNOTATION_PARSE_TIER_NAME, transcription);
		}
		if(annotateGlossTier==null){
			setPreference(ANNOTATION_TIER_GLOSS, ANNOTATION_GLOSS_TIER_NAME, transcription);
		}
		if(annotatePosTier==null){
			setPreference(ANNOTATION_TIER_POS, ANNOTATION_POS_TIER_NAME, transcription);
		}
		selectedFont=Constants.DEFAULTFONT;
		selectedSize= 12;
		// CC try to catch the font of the interlinearizeWordTier if any to apply it to the Lexicon
		if (getPreference("Interlinear.Tier.Word", transcription)!=null){		//if interlinearize is set
			if (getPreference("TierFonts", transcription)!=null){;				
				prefTierFonts= (HashMap) getPreference("TierFonts", transcription);
				if (prefTierFonts.containsKey(interlinearizeWordTier)){
					selectedFont=(Font) prefTierFonts.get(interlinearizeWordTier);
				}
			}
			else {String x="Quoi!";};
			if (getPreference("Interlinear.FontSizes", transcription)!=null){
				prefTierSizes=(HashMap) getPreference("Interlinear.FontSizes", transcription);
				if (prefTierSizes.containsKey(interlinearizeWordTier)){
					selectedSize=(Integer) prefTierSizes.get(interlinearizeWordTier);
				}
			}
		}
	}

	
	private void updateLinkedFile(){
		menuLinkedFile.removeAll();
		if(LexiconDocumentData.fileReference!=null){
			lexiconLinkedFile=new JCheckBoxMenuItem(LexiconDocumentData.fileReference.getName());
			lexiconLinkedFile.setSelected(true);
			lexiconLinkedFile.addItemListener(this);
			menuLinkedFile.add(lexiconLinkedFile);
		}
		if(ParseDocumentData.fileReference!=null){
			parseLinkedFile=new JCheckBoxMenuItem(ParseDocumentData.fileReference.getName());
			parseLinkedFile.setSelected(true);
			parseLinkedFile.addItemListener(this);
			menuLinkedFile.add(parseLinkedFile);
		}
		menuLinkedFile.repaint();
	}
	
	public void exportLexicon(){
		JFileChooser chooser = new JFileChooser();
		chooser.setFileFilter(ElanFileFilter.createFileFilter(ElanFileFilter.TEXT_TYPE));
        chooser.setDialogTitle(ElanLocale.getString("LexiconViewer.JFileChooser.ExportLexicon.Title"));
        
        int option = chooser.showSaveDialog(ELANCommandFactory.getRootFrame(transcription));
        if (option == JFileChooser.APPROVE_OPTION) {
            File f=chooser.getSelectedFile();
            if (f != null) {
            	String pathName=f.getAbsolutePath();
                String lowerPathName=pathName.toLowerCase();
                if (!lowerPathName.endsWith(TXT_EXT)){
                	pathName+=TXT_EXT;
                }
                if ((new File(pathName)).exists()) {
                    int answer = JOptionPane.showConfirmDialog(ELANCommandFactory.getRootFrame(transcription),
                            ElanLocale.getString("Message.Overwrite"),
                            ElanLocale.getString("SaveDialog.Message.Title"),
                            JOptionPane.YES_NO_OPTION);

                    if (answer==JOptionPane.YES_OPTION) {
                    	writeAndExportLexicon(pathName);
                    	JOptionPane.showConfirmDialog(null, ElanLocale.getString("LexiconViewer.JFileChooser.ExportLexicon.Success"), ElanLocale.getString("LexiconViewer.JFileChooser.ExportLexicon.Title"),
                    JOptionPane.CLOSED_OPTION, JOptionPane.INFORMATION_MESSAGE);      
                    }
                }
                else{
                    writeAndExportLexicon(pathName);
                    JOptionPane.showConfirmDialog(null, ElanLocale.getString("LexiconViewer.JFileChooser.ExportLexicon.Success"), ElanLocale.getString("LexiconViewer.JFileChooser.ExportLexicon.Title"),
                            JOptionPane.CLOSED_OPTION, JOptionPane.INFORMATION_MESSAGE); 
                }
            }
           
        }
	}

	/**
	 * create a parsing file with the current data in the mot, mb and rx lines
	 * 
	 */
	private void exportParseData(){
		 JFileChooser chooser = new JFileChooser();
		 chooser.setFileFilter(ElanFileFilter.createFileFilter(ElanFileFilter.ELAN_PARSE_TYPE));
         chooser.setDialogTitle(ElanLocale.getString("LexiconViewer.JFileChooser.ExportParse.Title"));
        
         int option = chooser.showSaveDialog(ELANCommandFactory.getRootFrame(transcription));
         if (option == JFileChooser.APPROVE_OPTION) {
             File f=chooser.getSelectedFile();
             
             if (f != null) {
                 // make sure pathname finishes with .eafp
                 String pathName=f.getAbsolutePath();
                 String lowerPathName=pathName.toLowerCase();

                 if (!lowerPathName.endsWith(EAFP_EXT)) {
                	 pathName+=EAFP_EXT;
                 }
                 if ((new File(pathName)).exists()) {
                     int answer = JOptionPane.showConfirmDialog(ELANCommandFactory.getRootFrame(transcription),
                             ElanLocale.getString("Message.Overwrite"),
                             ElanLocale.getString("SaveDialog.Message.Title"),
                             JOptionPane.YES_NO_OPTION);

                     if (answer==JOptionPane.NO_OPTION) {
                         return;
                     }
                 }
                 printParseData(pathName);
                 initTable();
                 placeTable();
                 
                 lxTable.updateLocale();
                 updateLinkedFile();
                 rightPanel.repaint();
             }
             JOptionPane.showConfirmDialog(null, ElanLocale.getString("LexiconViewer.JFileChooser.ExportParse.Success"), ElanLocale.getString("LexiconViewer.JFileChooser.ExportParse.Title"),
	                    JOptionPane.CLOSED_OPTION, JOptionPane.INFORMATION_MESSAGE);
         }
	}
	
	/**
	 * Parse Data
	 */
	private void selectParseData() {
		JFileChooser chooser = new JFileChooser();
		chooser.setMultiSelectionEnabled(false);
		chooser.addChoosableFileFilter(ElanFileFilter.createFileFilter(ElanFileFilter.ELAN_PARSE_TYPE));
		
		String dir = (String) Preferences.get("LexiconFileDir", transcription);

		if (dir == null) {
			// start in the directory of the eaf file
			String eafPath = transcription.getFullPath();

			if ((eafPath != null) &&
					(eafPath.indexOf(TranscriptionImpl.UNDEFINED_FILE_NAME) == -1)) {
				if (eafPath.startsWith("file:")) {
					eafPath = eafPath.substring(5);
				}
				int lastSep = eafPath.lastIndexOf(File.separator);
				if (lastSep > 0) {
					eafPath = eafPath.substring(0, lastSep);
				}
				dir = eafPath;
			} else {
				// user.dir is probably a better choice than home.dir?
				dir = System.getProperty("user.dir");
			}
		}

		chooser.setCurrentDirectory(new File(dir));
		chooser.setDialogTitle(ElanLocale.getString("Button.Select"));
		chooser.setApproveButtonText(ElanLocale.getString("Button.Select"));

		int option = chooser.showOpenDialog(this);

		if (option == JFileChooser.APPROVE_OPTION) {
			File selected = chooser.getSelectedFile();
			if ((selected != null) && selected.exists()) {
				String selectedFilePath = selected.getAbsolutePath();
				String lxDir = selected.getParent();

				setPreference("LexiconFileDir", lxDir, transcription);
				lxPathLabel.setText(selectedFilePath);
				if(selectedFilePath.endsWith(EAFP_EXT)){
					SAXBuilder sxb=new SAXBuilder();
					try {
						File f=new File(selectedFilePath);
						ParseDocumentData.setDocument(sxb.build(f), f);
					}
					catch(Exception e){
						System.out.println("document build failed");
					}
					setPreference(PARSE_SOURCE, selectedFilePath, transcription);
					if(lxTable==null&&lxTableModel==null){
						LexiconDocumentData.setDocument(null, null);
						lxTableModel=new LexiconTableModel();
						lxTable=new LexiconTable(lxTableModel);
					}
					rightPanel.removeAll();
					initTable();
					placeTable();
					lxTable.updateLocale();
					updateLinkedFile();
					rightPanel.repaint();
				}
				else{
					JOptionPane.showMessageDialog(this, "invalide file format", "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}
	
	/**
	 * create a new Lexicon file
	 * 
	 */
	private void createLXSource(){
		JFileChooser chooser = new JFileChooser();
		chooser.setFileFilter(ElanFileFilter.createFileFilter(ElanFileFilter.ELAN_LEX_TYPE));
		chooser.setDialogTitle(ElanLocale.getString("LexiconViewer.JFileChooser.CreateLexicon.Title"));

		int option = chooser.showSaveDialog(ELANCommandFactory.getRootFrame(transcription));
		if (option == JFileChooser.APPROVE_OPTION) {
			File f=chooser.getSelectedFile();

			if (f != null) {
				// make sure pathname finishes with .eafl
				String pathName=f.getAbsolutePath();
				String lowerPathName=pathName.toLowerCase();

				if (!lowerPathName.endsWith(EAFL_EXT)) {
					pathName+=EAFL_EXT;
				}
				if ((new File(pathName)).exists()) {
					int answer = JOptionPane.showConfirmDialog(ELANCommandFactory.getRootFrame(transcription),
							ElanLocale.getString("Message.Overwrite"),
							ElanLocale.getString("SaveDialog.Message.Title"),
							JOptionPane.YES_NO_OPTION);

					if (answer==JOptionPane.NO_OPTION) {
						return;
					}
				}
				setPreference(LX_SOURCE, pathName, transcription);
				lxPathLabel.setText(pathName);
				lexicalPath=pathName;
				print(pathName);
				updateLinkedFile();
			}
		}
	}
	
	private void createElanLexicon(LexiconParser parser, Element root){
		JFileChooser chooser = new JFileChooser();
		chooser.setFileFilter(ElanFileFilter.createFileFilter(ElanFileFilter.ELAN_LEX_TYPE));
		chooser.setDialogTitle(ElanLocale.getString("LexiconViewer.DialogTitle.CreateLexicon"));
		int option = chooser.showSaveDialog(ELANCommandFactory.getRootFrame(transcription));
		if (option == JFileChooser.APPROVE_OPTION) {
			File f=chooser.getSelectedFile();

			if (f != null) {
				// make sure pathname finishes with .eafl
				String pathName=f.getAbsolutePath();
				String lowerPathName=pathName.toLowerCase();

				if (!lowerPathName.endsWith(EAFL_EXT)) {
					pathName+=EAFL_EXT;
				}
				if ((new File(pathName)).exists()) {
					int answer = JOptionPane.showConfirmDialog(ELANCommandFactory.getRootFrame(transcription),
							ElanLocale.getString("Message.Overwrite"),
							ElanLocale.getString("SaveDialog.Message.Title"),
							JOptionPane.YES_NO_OPTION);

					if (answer==JOptionPane.NO_OPTION) {
						return;
					}
				}
				LexiconDocumentData.setDocument(new Document(root), new File(pathName));
				parser.print(pathName);
				
				setPreference(LX_SOURCE, pathName, transcription);
				lxPathLabel.setText(pathName);
				lexicalPath=pathName;
				//print(pathName);
				updateLinkedFile();
			}
		}
	}

	/**
	 * Export & merge parse data
	 * 
	 */
	private void exportAndMergeParseData(){
		JFileChooser chooser = new JFileChooser();
		chooser.setFileFilter(ElanFileFilter.createFileFilter(ElanFileFilter.ELAN_PARSE_TYPE));
        chooser.setDialogTitle(ElanLocale.getString("LexiconViewer.JFileChooser.ExportAndMerge.Title"));
        
        String dir = (String) Preferences.get("LexiconFileDir", transcription);

		if (dir == null) {
			// start in the directory of the eaf file
			String eafPath = transcription.getFullPath();

			if ((eafPath != null) &&
					(eafPath.indexOf(TranscriptionImpl.UNDEFINED_FILE_NAME) == -1)) {
				if (eafPath.startsWith("file:")) {
					eafPath = eafPath.substring(5);
				}
				int lastSep = eafPath.lastIndexOf(File.separator);
				if (lastSep > 0) {
					eafPath = eafPath.substring(0, lastSep);
				}
				dir = eafPath;
			} else {
				// user.dir is probably a better choice than home.dir?
				dir = System.getProperty("user.dir");
			}
		}

		chooser.setCurrentDirectory(new File(dir));
		chooser.setDialogTitle(ElanLocale.getString("Button.Select"));
		chooser.setApproveButtonText(ElanLocale.getString("Button.Select"));

		int option = chooser.showOpenDialog(this);

		if (option == JFileChooser.APPROVE_OPTION) {
			File selected = chooser.getSelectedFile();
			if ((selected != null) && selected.exists()) {
				String selectedFilePath = selected.getAbsolutePath();
				String lxDir = selected.getParent();

				setPreference("LexiconFileDir", lxDir, transcription);
				lxPathLabel.setText(selectedFilePath);
				if(selectedFilePath.endsWith(EAFP_EXT)){
					//printParseData(selectedFilePath);
					SAXBuilder sxb=new SAXBuilder();
					try {
						File f=new File(selectedFilePath);
						ParseDocumentData.setDocument(sxb.build(f), f);
					}
					catch(Exception e){
						System.out.println("document build failed");
					}
					List<AnnotationTree> annTreeList=createAnnotationTree();
					if(!annTreeList.isEmpty()){
						Element exportedRootElmt=exportParseData(annTreeList);
						
						List<Element> exportedParses=exportedRootElmt.cloneContent();
						List<Element> currParses=ParseDocumentData.DOCUMENT.getRootElement().getChildren();
						ParseDocumentData.DOCUMENT.getRootElement().addContent(exportedParses);
						Element rootElmt=ParseDocumentData.DOCUMENT.getRootElement();
						sortContent(rootElmt, new Comparator() {
							@Override
							public int compare(Object o1, Object o2) {
								Element e1=(Element)o1;
								Element e2=(Element)o2;
								return e1.getChild(LexiconDocumentData.WRITTENFORM).getValue().compareTo(e2.getChild(LexiconDocumentData.WRITTENFORM).getValue());
							}
						});

						//delete doubles
						boolean added;
						Element lastParseElmt=null;
						List<Element> elementsToAdd=new ArrayList<Element>();
						List<Element> parses=ParseDocumentData.DOCUMENT.getRootElement().getChildren();
						for(Element currParseElmt:parses){
							added=true;
							if(lastParseElmt!=null){
								String currWFValue=currParseElmt.getChild(LexiconDocumentData.WRITTENFORM).getValue();
								String lastWFValue=lastParseElmt.getChild(LexiconDocumentData.WRITTENFORM).getValue();
								if(currWFValue.equals(lastWFValue)){
									Element currMorphElmt=currParseElmt.getChild(LexiconDocumentData.MORPHOLOGY);
									Element lastMorphElmt=lastParseElmt.getChild(LexiconDocumentData.MORPHOLOGY);
									if(currMorphElmt.getChildren(LexiconDocumentData.SEGMENT).size()==lastMorphElmt.getChildren(LexiconDocumentData.SEGMENT).size()){
										List<Element> currSegmentList=currMorphElmt.getChildren(LexiconDocumentData.SEGMENT);
										List<Element> lastSegmentList=lastMorphElmt.getChildren(LexiconDocumentData.SEGMENT);
										int idemSegment=0;
										for(int i=0; i<currSegmentList.size(); i++){
											Element currSegment=currSegmentList.get(i);
											Element lastSegment=lastSegmentList.get(i);
											if(currSegment.getValue().equals(lastSegment.getValue())){
												if(currSegment.getAttributeValue(ParseDocumentData.GE).equals(lastSegment.getAttributeValue(ParseDocumentData.GE))){
													if(currSegment.getAttributeValue(ParseDocumentData.RX).equals(lastSegment.getAttributeValue(ParseDocumentData.RX))){
														idemSegment++;
													}
												}
											}
										}
										if(idemSegment==currSegmentList.size()){
											added=false;
										}
									}
								}
							}
							if(added){
								elementsToAdd.add(currParseElmt);
							}
							lastParseElmt=currParseElmt;
						}
						
						ParseDocumentData.DOCUMENT.getRootElement().removeContent();
						ParseDocumentData.DOCUMENT.getRootElement().addContent(elementsToAdd);
						try {
							XMLOutputter output=new XMLOutputter(Format.getPrettyFormat());
							output.output(ParseDocumentData.DOCUMENT, new FileOutputStream(selectedFilePath));
						}
						catch (java.io.IOException e){
							
						}
					}
					setPreference(PARSE_SOURCE, selectedFilePath, transcription);
					if(lxTable==null&&lxTableModel==null){
						LexiconDocumentData.setDocument(null, null);
						lxTableModel=new LexiconTableModel();
						lxTable=new LexiconTable(lxTableModel);
					}
					rightPanel.removeAll();
					initTable();
					placeTable();
					lxTable.updateLocale();
					updateLinkedFile();
					rightPanel.repaint();
					JOptionPane.showConfirmDialog(null, ElanLocale.getString("LexiconViewer.JFileChooser.ExportAndMerge.Success"), ElanLocale.getString("LexiconViewer.JFileChooser.ExportAndMerge.Title"),
		                    JOptionPane.CLOSED_OPTION, JOptionPane.INFORMATION_MESSAGE);
				}
				else{
					JOptionPane.showMessageDialog(this, "invalide file format", "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}
	
	/**
	 * Sort Document Data
	 * @param element
	 * @param comparator
	 */
	public static void sortContent(Element element, Comparator comparator) { 
        // Create a new, static list of child elements, and sort it.
        List children = new ArrayList(element.getChildren());
        Collections.sort(children, comparator);
        ListIterator childrenIter = children.listIterator();

        // Create a new, static list of all content items.
        List content = new ArrayList(element.getContent());
        ListIterator contentIter = content.listIterator();

        // Loop through the content items, and whenever we find an Element,
        // we'll insert the next ordered Element in its place. Because the
        // content list is not live, it won't complain about an Element being
        // added twice.
        while(contentIter.hasNext())
        {
            Object obj = contentIter.next();
            if (obj instanceof Element)
                contentIter.set(childrenIter.next());
        }

        // Finally, we set the content list back into the parent Element.
        element.setContent((List)null);
        element.setContent(content);
    }

	
	//print an empty file
	private void print(String file){
		createEmptyDocument(file);
		   try {
		      XMLOutputter output=new XMLOutputter(Format.getPrettyFormat());
		      output.output(LexiconDocumentData.DOCUMENT, new FileOutputStream(file));
		      createLexiconGrid(file);
		   }
		   catch (java.io.IOException e){
			   
		   }
		}

	/**
	 * create an empty document
	 * @param file
	 */
	private void createEmptyDocument(String file){
		Element root=new Element(LexiconDocumentData.ROOT);
		LexiconDocumentData.setDocument(new Document(root), new File(file));
	}
	
	/**
	 * Provides the means to select a source for the lexicon.
	 */
	private void selectLXSource(int filterType) {
		JFileChooser chooser = new JFileChooser();
		chooser.setMultiSelectionEnabled(false);
		chooser.addChoosableFileFilter(ElanFileFilter.createFileFilter(filterType));
		
		String dir = (String) Preferences.get("LexiconFileDir", transcription);

		if (dir == null) {
			// start in the directory of the eaf file
			String eafPath = transcription.getFullPath();

			if ((eafPath != null) &&
					(eafPath.indexOf(TranscriptionImpl.UNDEFINED_FILE_NAME) == -1)) {
				if (eafPath.startsWith("file:")) {
					eafPath = eafPath.substring(5);
				}
				int lastSep = eafPath.lastIndexOf(File.separator);
				if (lastSep > 0) {
					eafPath = eafPath.substring(0, lastSep);
				}
				dir = eafPath;
			} else {
				dir = System.getProperty("user.dir");
			}
		}

		chooser.setCurrentDirectory(new File(dir));
		if (filterType==ElanFileFilter.ELAN_LEX_TYPE){
			chooser.setDialogTitle(ElanLocale.getString("LexiconViewer.JFileChooser.SelectLexicon.Title"));
					//ElanLocale.getString("Button.Select"));
		} else{
			chooser.setDialogTitle(ElanLocale.getString("LexiconViewer.JFileChooser.ImportLexicon.Title"));
		}
		
		chooser.setApproveButtonText(ElanLocale.getString("Button.Select"));

		int option = chooser.showOpenDialog(this);

		if (option == JFileChooser.APPROVE_OPTION) {
			File selected = chooser.getSelectedFile();
			if ((selected != null) && selected.exists()) {
				String selectedFilePath = selected.getAbsolutePath();
				String lxDir = selected.getParent();
				setPreference("LexiconFileDir", lxDir, transcription);
				lxPathLabel.setText(selectedFilePath);
				if (filterType==ElanFileFilter.ELAN_LEX_TYPE){
					if(selectedFilePath.endsWith(EAFL_EXT)){
						lexicalPath = selectedFilePath;
						setPreference(LX_SOURCE, lexicalPath, transcription);
						rightPanel.removeAll();
						createLexiconGrid(lexicalPath);
					}
					else{
						JOptionPane.showMessageDialog(this, "invalide file format", "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
				else{
					lexicalPath = selectedFilePath;
					configPanel=new LXConfigurationDialog(new File(lexicalPath));
					configPanel.setVisible(true);
					
					if(configPanel.isConfigOK()){
						rightPanel.removeAll();
						createLexiconGrid(lexicalPath, configPanel.getLexiconFieldConfig());
						setPreference(LX_SOURCE, lexicalPath, transcription);
					}
				}
			}
			updateLinkedFile();
		}
	}

	/**
	 * update Lexicon after an insertion or a modification 
	 */
	public void updateLexiconChange(){
		leftPanel.removeAll();
		rightPanel.removeAll();

		Collections.sort(entries);
		List<LexiconTableContent> copyTableContentEntries=new ArrayList<LexiconTableContent>(entries);

		List<ParsingMorpheme> copyMorphemeList=new ArrayList<ParsingMorpheme>(parsingMorphemeList);
		List<ParsingMorpheme> addedParsingMorphemeList=new ArrayList<ParsingMorpheme>();
		List<LexiconIdConnectionToTableContent> copyLexiconIdContentList=new ArrayList<LexiconIdConnectionToTableContent>(lexiconIdConnectToTableContentList);

		if(LexiconDocumentData.modifiedElementId>-1){
			LexiconDocumentData.setDeletedElementId(LexiconDocumentData.modifiedElementId);
			LexiconDocumentData.setAddedElementId(LexiconDocumentData.modifiedElementId);
			LexiconDocumentData.setModifiedElementId(-1);
		}
		
		if(LexiconDocumentData.deletedElementId>-1){
			int cmpt=0;
			for(LexiconTableContent tableContent:entries){
				tableContent.setGridIndex(cmpt);
				if(Integer.parseInt(tableContent.getAssociatedLexicalEntryId())==LexiconDocumentData.deletedElementId){
					copyTableContentEntries.remove(tableContent);
				}
				else{
					cmpt++;
				}
			}
			for(ParsingMorpheme parsingMorpheme:parsingMorphemeList){
				if(Integer.parseInt(parsingMorpheme.getAssociatedId())==LexiconDocumentData.deletedElementId){
					copyMorphemeList.remove(parsingMorpheme);
				}
			}
			
			for(LexiconIdConnectionToTableContent lexiconIdConnectToTableContent:lexiconIdConnectToTableContentList){
				if(Integer.parseInt(lexiconIdConnectToTableContent.getLexiconId())==LexiconDocumentData.deletedElementId){
					copyLexiconIdContentList.remove(lexiconIdConnectToTableContent);
				}
			}
			LexiconDocumentData.setDeletedElementId(-1);
			entries=copyTableContentEntries;
			parsingMorphemeList=copyMorphemeList;
			lexiconIdConnectToTableContentList=copyLexiconIdContentList;
		}
		
		if(LexiconDocumentData.addedElementId>-1){
			int id=0;
			if(entries.size()>0){
				id=entries.get(entries.size()-1).getGridIndex()+1;
			}
			Element lexemeElmt=LexiconDocumentData.getElement(Integer.toString(LexiconDocumentData.addedElementId));
			Element lexicalEntryElmt=lexemeElmt.getParentElement();
			LexiconIdConnectionToTableContent lexiconIdConnectToTableContent=new LexiconIdConnectionToTableContent(Integer.toString(LexiconDocumentData.addedElementId));
			Element formElmt=lexicalEntryElmt.getChild(LexiconDocumentData.FORM);
			List<Element> altFormElmtList=new ArrayList<Element>();
			List<Element>glossElmtList=new ArrayList<Element>();
			List<Element>segmentElmtList=new ArrayList<Element>();
			if(formElmt!=null){
				altFormElmtList=formElmt.getChildren(LexiconDocumentData.ALTERNATE_FORM);
			}
			Element senseElmt=lexicalEntryElmt.getChild(LexiconDocumentData.SENSE);
			if(senseElmt!=null){
				glossElmtList=senseElmt.getChildren(LexiconDocumentData.GLOSE);
			}
			Element lexMorphElmt=lexicalEntryElmt.getChild(LexiconDocumentData.MORPHOLOGY);
			if(lexMorphElmt!=null){
				segmentElmtList=lexMorphElmt.getChildren(LexiconDocumentData.SEGMENT);
			}

			if(!glossElmtList.isEmpty()){
				for(Element glossElmt:glossElmtList){
					LexiconTableContent currentEntry=new LexiconTableContent(id);
					currentEntry.setAssociatedLexicalEntryId(Integer.toString(LexiconDocumentData.addedElementId));
					currentEntry.setLexemeElmt(lexemeElmt);
					currentEntry.setType(lexemeElmt.getAttributeValue(LexiconDocumentData.TYPE));
					currentEntry.setLx(lexemeElmt.getText());
					String der=glossElmt.getAttributeValue(LexiconDocumentData.DERIVATION);
					if(der!=null){
						currentEntry.setDer(der);
					} 
					currentEntry.setGe(glossElmt.getText());
					currentEntry.setRx(glossElmt.getAttributeValue(LexiconDocumentData.RX));
					lexiconIdConnectToTableContent.addTableContent(currentEntry);
					copyTableContentEntries.add(currentEntry);
					id++;
					
					if(altFormElmtList!=null){
						for(Element altFormElmt:altFormElmtList){
							Element wFormElmt=altFormElmt.getChild(LexiconDocumentData.WRITTENFORM);
							LexiconVariantContent variantContent=new LexiconVariantContent(wFormElmt.getText());
							if(wFormElmt.getAttribute(LexiconDocumentData.DERIVATION)!=null){
    							variantContent.setDer(wFormElmt.getAttribute(LexiconDocumentData.DERIVATION).getValue());
    						}
							if(altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY)!=null){
								//LexiconRef lexiconRef=new LexiconRef(wFormElmt.getText());
								Element altMorphElmt=altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY);
								List<Element> altSegmentElmtList=altMorphElmt.getChildren(LexiconDocumentData.SEGMENT);
								//copyMorphemeList.remove(index)
								ParsingMorpheme parsingMorpheme=new ParsingMorpheme(wFormElmt.getText(), Integer.toString(LexiconDocumentData.addedElementId));
								for(Element altSegmentElmt:altSegmentElmtList){
									variantContent.addElement(altSegmentElmt.getText());
									parsingMorpheme.addMorpheme(altSegmentElmt.getText());
									if(altSegmentElmt.getAttribute(LexiconDocumentData.REFERENCE)!=null){
										String refId=altSegmentElmt.getAttributeValue(LexiconDocumentData.REFERENCE);
										parsingMorpheme.addLexiconRefId(refId);
									}
									else{
										parsingMorpheme.addLexiconRefId(null);
									}
								}
								copyMorphemeList.add(parsingMorpheme);
								addedParsingMorphemeList.add(parsingMorpheme);
								
							}
							currentEntry.addLexiconVariantContent(variantContent);
						}
					}
				}
			} else if(!segmentElmtList.isEmpty()){
				LexiconTableContent currentEntry=new LexiconTableContent(id);
				currentEntry.setAssociatedLexicalEntryId(Integer.toString(LexiconDocumentData.addedElementId));
				currentEntry.setLexemeElmt(lexemeElmt);
				currentEntry.setType(lexemeElmt.getAttributeValue(LexiconDocumentData.TYPE));
				currentEntry.setLx(lexemeElmt.getText());
				lexiconIdConnectToTableContent.addTableContent(currentEntry);
				copyTableContentEntries.add(currentEntry);
				id++;
				
				ParsingMorpheme parsingMorpheme=new ParsingMorpheme(lexemeElmt.getValue(), Integer.toString(LexiconDocumentData.addedElementId));
					for(Element segmentElmt:segmentElmtList){
						currentEntry.addSegment(segmentElmt.getText());
						parsingMorpheme.addMorpheme(segmentElmt.getText());
						if(segmentElmt.getAttribute(LexiconDocumentData.REFERENCE)!=null){
							String refId=segmentElmt.getAttributeValue(LexiconDocumentData.REFERENCE);
							parsingMorpheme.addLexiconRefId(refId);
						}
						else{
							parsingMorpheme.addLexiconRefId(null);
						}
					}
					copyMorphemeList.add(parsingMorpheme);
					addedParsingMorphemeList.add(parsingMorpheme);

					for(Element altFormElmt:altFormElmtList){
						Element wFormElmt=altFormElmt.getChild(LexiconDocumentData.WRITTENFORM);
						if(altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY)!=null){
							LexiconRef lexiconRef=new LexiconRef(wFormElmt.getText());
							Element altMorphElmt=altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY);
							List<Element> altSegmentElmtList=altMorphElmt.getChildren(LexiconDocumentData.SEGMENT);
							ParsingMorpheme parsingMorpheme2=new ParsingMorpheme(wFormElmt.getText(), Integer.toString(LexiconDocumentData.addedElementId));
							for(Element altSegmentElmt:altSegmentElmtList){
								parsingMorpheme2.addMorpheme(altSegmentElmt.getText());
								if(altSegmentElmt.getAttribute(LexiconDocumentData.REFERENCE)!=null){
									String refId=altSegmentElmt.getAttributeValue(LexiconDocumentData.REFERENCE);
									parsingMorpheme2.addLexiconRefId(refId);
								}
								else{
									parsingMorpheme2.addLexiconRefId(null);
								}
							}
							copyMorphemeList.add(parsingMorpheme2);
							addedParsingMorphemeList.add(parsingMorpheme2);

						}
						else{
							ParsingMorpheme variantParsingMorpheme=new ParsingMorpheme(wFormElmt.getText(), id+"");
							variantParsingMorpheme.setLexiconRefids(parsingMorpheme.getLexiconRefIds());
							copyMorphemeList.add(variantParsingMorpheme);
							addedParsingMorphemeList.add(variantParsingMorpheme);
						}
					}
					id++;
				}
			lexiconIdConnectToTableContentList.add(lexiconIdConnectToTableContent);
			LexiconDocumentData.setAddedElementId(-1);
			entries=copyTableContentEntries;
			parsingMorphemeList=copyMorphemeList;
		}
		
		if(!addedParsingMorphemeList.isEmpty()){
			for(ParsingMorpheme parsingMorpheme:addedParsingMorphemeList){
				parsingMorpheme.generateLexiconRefList(lexiconIdConnectToTableContentList);    	
			}
		}
		lxTableModel=new LexiconTableModel(entries);
		lxTable=new LexiconTable(lxTableModel);
		initTable();
		placeTable();
		lxTable.updateLocale();
	}

	/**
	 * create a JTable with an elan xml file
	 * @param lexicalPath (.eafl)
	 */
	private void createLexiconGrid(String lexicalPath){
		LexiconParser.parseXmlLexicon(lexicalPath);
		if(LexiconDocumentData.DOCUMENT!=null){
			exportLexiconItem.setEnabled(true);
			createLexiconTableContent(LexiconDocumentData.DOCUMENT.getRootElement().getChildren());			
		}
		lxTableModel=new LexiconTableModel(entries);
		lxTable=new LexiconTable(lxTableModel);
		initTable();
		placeTable();
		lxTable.updateLocale();
	}
	
	/**
	 * create a the lexiconGrid with the toolbox lexicon file selected
	 * @param lexicalPath (.txt)
	 * @param configPanel
	 */
	private void createLexiconGrid(String lexicalPath, LexiconFieldConfiguration configPanel){
		LexiconParser parser=new LexiconParser(configPanel);
		//if(lexicalPath.endsWith(TXT_EXT)){		// CC 13/07/12
			Element root=parser.parseShoeboxDictionary(lexicalPath);
			createElanLexicon(parser, root);
		//}
		if(LexiconDocumentData.DOCUMENT!=null){
			createLexiconTableContent(LexiconDocumentData.DOCUMENT.getRootElement().getChildren());			
		}
		lxTableModel=new LexiconTableModel(entries);
		lxTable=new LexiconTable(lxTableModel);
		initTable();
		placeTable();

		lxTable.updateLocale();
	}
	
	/**
	 * init the grid lexicon table
	 */
	protected void initTable() {
		lxTable.setFont(selectedFont);  //CC
	    lxTable.setFont(selectedFont.deriveFont((float) selectedSize));
		popup=new LexiconViewerPopUpMenu(lxTable, this);
		lxTable.setAutoCreateRowSorter(true);
		lxTable.setDefaultRenderer(Object.class, createTableCellRenderer(lxTableModel));
		lxTable.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if (SwingUtilities.isRightMouseButton(e) || e.isPopupTrigger()) {
					popup.show(lxTable, e.getPoint().x, e.getPoint().y);
				}
			}
		});
	       
		//lxTable.setFont(Constants.DEFAULTFONT);
		lxTable.setBackground(Constants.DEFAULTBACKGROUNDCOLOR);
	}

	protected TableCellRenderer createTableCellRenderer(TableModel model) {
		return new LexiconRenderer(this, model);
	}

	private void clearElements(){
		entries.clear();
		parsingMorphemeList.clear();
		lexiconIdConnectToTableContentList.clear();
	}
	
	/**
	 * place the grid on the JPanel
	 */
	private void placeTable() {
		leftPanel.removeAll();
		rightPanel.removeAll();
		
		JScrollPane scrollPane=new JScrollPane(lxTable);
		scrollPane.setBorder(new javax.swing.border.EmptyBorder(0, 0, 0, 0));
		GridBagConstraints gbc;	

		leftPanel.setLayout(new GridBagLayout());
		gbc=new GridBagConstraints();
		gbc.fill=GridBagConstraints.BOTH;
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.weighty=0.9;
		gbc.weightx=1;		
		leftPanel.add(scrollPane,gbc);
		
		//Lexicon Panel with Edit/Record buttons
		gbc.gridx=0;
		gbc.gridy=1;
		gbc.weightx=0.0; //1;	
		gbc.weighty=0.0; //0.1;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		JPanel lexiconRecordPanel=new LexiconRecordPanel(this);
		leftPanel.add(lexiconRecordPanel,gbc);
		leftPanel.repaint();
		
		//
		rightPanel.setLayout(new GridBagLayout());
		gbc=new GridBagConstraints();
		gbc.fill=GridBagConstraints.BOTH;
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.weighty=0.9;
		gbc.weightx=1;
		interlinearizeProcess=new InterlinearizationProcess(lxTableModel, parsingMorphemeList, this);		
		// CC 310811
		lexiconAnnotationProcessPanel=new LexiconSegmentationAnnotationPanel(this, lxTableModel, lxTable, interlinearizeProcess);	
		rightPanel.add(lexiconAnnotationProcessPanel,gbc);
		rightPanel.repaint();
		
		//viewerContainer.setLeftComponent(scrollPane);
		viewerContainer.setDividerLocation(0.5);
		viewerContainer.repaint();
	}
	
	
	/**create LexiconTableContent and ParsingMorphemes
	 * */
	private void createLexiconTableContent(List<Element> lexicalEntries){
		entries=new ArrayList<LexiconTableContent>();
		parsingMorphemeList=new ArrayList<ParsingMorpheme>();
		clearElements();
		
		LexiconTableContent currentEntry=null; 
		int k=0;
		for(Element lexicalEntryElmt:lexicalEntries){
			List<Element> altFormElmtList=null;
			List<Element> glossElmtList=null;
			List<Element> segmentElmtList=null;

			String id=lexicalEntryElmt.getAttributeValue(LexiconDocumentData.ID);
			LexiconIdConnectionToTableContent lexiconIdConnectToTableContent=new LexiconIdConnectionToTableContent(id);
			Element lexemeElmt=lexicalEntryElmt.getChild(LexiconDocumentData.LEXEME);
			Element formElmt=lexicalEntryElmt.getChild(LexiconDocumentData.FORM);
			if(formElmt!=null){
				altFormElmtList=formElmt.getChildren(LexiconDocumentData.ALTERNATE_FORM);
			}
			Element senseElmt=lexicalEntryElmt.getChild(LexiconDocumentData.SENSE);
			if(senseElmt!=null){
				glossElmtList=senseElmt.getChildren(LexiconDocumentData.GLOSE);
			}
			Element lexMorphElmt=lexicalEntryElmt.getChild(LexiconDocumentData.MORPHOLOGY);
			if(lexMorphElmt!=null){
				segmentElmtList=lexMorphElmt.getChildren(LexiconDocumentData.SEGMENT);
			}

			if(glossElmtList!=null){
				//check the variant of an element
				List<LexiconVariantContent> lexiconVariantList=new ArrayList<LexiconVariantContent>();
				if(altFormElmtList!=null){
					for(Element altFormElmt:altFormElmtList){
						List<Element> altSegmentElmtList=null;
						Element wFormElmt=altFormElmt.getChild(LexiconDocumentData.WRITTENFORM);
						LexiconVariantContent variantContent=new LexiconVariantContent(wFormElmt.getText());
						if(wFormElmt.getAttribute(LexiconDocumentData.DERIVATION)!=null){
							variantContent.setDer(wFormElmt.getAttribute(LexiconDocumentData.DERIVATION).getValue());
						}
						if(altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY)!=null){
							Element altMorphElmt=altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY);
							//List<Element> 
							altSegmentElmtList=altMorphElmt.getChildren(LexiconDocumentData.SEGMENT);
							ParsingMorpheme parsingMorpheme=new ParsingMorpheme(wFormElmt.getText(), id);
							for(Element altSegmentElmt:altSegmentElmtList){
								parsingMorpheme.addMorpheme(altSegmentElmt.getText());
								variantContent.addElement(altSegmentElmt.getText());
								if(altSegmentElmt.getAttribute(LexiconDocumentData.REFERENCE)!=null){
									String refId=altSegmentElmt.getAttributeValue(LexiconDocumentData.REFERENCE);
									parsingMorpheme.addLexiconRefId(refId);
								}
								else{
									parsingMorpheme.addLexiconRefId(null);
								}
							}
							parsingMorphemeList.add(parsingMorpheme);

						}
						lexiconVariantList.add(variantContent);
					}
				}
				//check gloss values
				for(Element glossElmt:glossElmtList){
					currentEntry=new LexiconTableContent(k);
					currentEntry.setAssociatedLexicalEntryId(id);
					currentEntry.setLexemeElmt(lexemeElmt);
					currentEntry.setType(lexemeElmt.getAttributeValue(LexiconDocumentData.TYPE));
					currentEntry.setLx(lexemeElmt.getText());
					if(!lexiconVariantList.isEmpty()){
						currentEntry.setLexiconVariantList(lexiconVariantList);						
					}
					String der=glossElmt.getAttributeValue(LexiconDocumentData.DERIVATION);
					if(der!=null){
						currentEntry.setDer(der);
					} 
					currentEntry.setGe(glossElmt.getText());
					currentEntry.setRx(glossElmt.getAttributeValue(LexiconDocumentData.RX));

					lexiconIdConnectToTableContent.addTableContent(currentEntry);
					entries.add(currentEntry);
					k++;
				}
			}
    		
			if(segmentElmtList!=null){
				currentEntry=new LexiconTableContent(k);
				currentEntry.setAssociatedLexicalEntryId(id);
				currentEntry.setLexemeElmt(lexemeElmt);
				currentEntry.setType(lexemeElmt.getAttributeValue(LexiconDocumentData.TYPE));
				currentEntry.setLx(lexemeElmt.getText());
				currentEntry.setGe("");
				currentEntry.setRx("");
				for(Element segmentElmt:segmentElmtList){
					currentEntry.addSegment(segmentElmt.getText());
				}
				if(altFormElmtList!=null){
					for(Element altFormElmt:altFormElmtList){
						Element wFormElmt=altFormElmt.getChild(LexiconDocumentData.WRITTENFORM);
						LexiconVariantContent variantContent=new LexiconVariantContent(wFormElmt.getText());
						if(wFormElmt.getAttribute(LexiconDocumentData.DERIVATION)!=null){
							variantContent.setDer(wFormElmt.getAttribute(LexiconDocumentData.DERIVATION).getValue());
						}
						if(altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY)!=null){
							Element variantMorphElmt=altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY);
							List<Element> variantSegmentList=variantMorphElmt.getChildren(LexiconDocumentData.SEGMENT);
							for(Element variantSegment:variantSegmentList){
								variantContent.addElement(variantSegment.getText());
							}
						}
						currentEntry.addLexiconVariantContent(variantContent);
					}
				}
				lexiconIdConnectToTableContent.addTableContent(currentEntry);
				entries.add(currentEntry);
				k++;
				
				ParsingMorpheme parsingMorpheme=new ParsingMorpheme(lexemeElmt.getText(), id);	    		
				for(Element segmentElmt:segmentElmtList){
					parsingMorpheme.addMorpheme(segmentElmt.getText());
					if(segmentElmt.getAttribute(LexiconDocumentData.REFERENCE)!=null){
						String refId=segmentElmt.getAttributeValue(LexiconDocumentData.REFERENCE);
						parsingMorpheme.addLexiconRefId(refId);
					}
					else{
						parsingMorpheme.addLexiconRefId(null);
					}
				}
				parsingMorphemeList.add(parsingMorpheme);

				for(Element altFormElmt:altFormElmtList){
					Element wFormElmt=altFormElmt.getChild(LexiconDocumentData.WRITTENFORM);
					if(altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY)!=null){
						Element altMorphElmt=altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY);
						List<Element> altSegmentElmtList=altMorphElmt.getChildren(LexiconDocumentData.SEGMENT);
						ParsingMorpheme parsingMorpheme2=new ParsingMorpheme(wFormElmt.getText(), id);
						for(Element altSegmentElmt:altSegmentElmtList){
							parsingMorpheme2.addMorpheme(altSegmentElmt.getText());
							if(altSegmentElmt.getAttribute(LexiconDocumentData.REFERENCE)!=null){
								String refId=altSegmentElmt.getAttributeValue(LexiconDocumentData.REFERENCE);
								parsingMorpheme2.addLexiconRefId(refId);
							}
							else{
								parsingMorpheme2.addLexiconRefId(null);
							}
						}
						parsingMorphemeList.add(parsingMorpheme2);

					}
					else{
						ParsingMorpheme variantParsingMorpheme=new ParsingMorpheme(wFormElmt.getText(), id);
						variantParsingMorpheme.setLexiconRefids(parsingMorpheme.getLexiconRefIds());
						parsingMorphemeList.add(variantParsingMorpheme);
					}
				}
			}
			lexiconIdConnectToTableContentList.add(lexiconIdConnectToTableContent);

		}
		for(ParsingMorpheme parsingMorpheme:parsingMorphemeList){
			parsingMorpheme.generateLexiconRefList(lexiconIdConnectToTableContentList);    	
		}
	}

	/**
	 * 
	 * @param path
	 */
	private void printParseData(String path){
		exportParseFile(path);
		if(lxTable==null&&lxTableModel==null){
			LexiconDocumentData.setDocument(new Document(new Element("x")), null);
			lxTableModel=new LexiconTableModel();
			lxTable=new LexiconTable(lxTableModel);
			initTable();
			placeTable();
			lxTable.updateLocale();
		}
		rightPanel.removeAll();
	}
	
	/**
	 * Export  the current .eafl lexicon to a toolbox file
	 * @param path
	 */
	private void writeAndExportLexicon(String path){
		Element rootElement=LexiconDocumentData.DOCUMENT.getRootElement();
		List<Element> lexicalEntries=rootElement.getChildren(LexiconDocumentData.LEXICAL_ENTRY);
		File f=new File(path);
		if(!f.exists()){
			try {
				f.createNewFile();

				FileOutputStream out = new FileOutputStream(f);
				BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(out, "UTF-8"));

				writer.write("\\_sh v3.0  231  MDF 4.0\n\\_DateStampHasFourDigitYear\n\n");
				for(Element lexicalEntry:lexicalEntries){
					writer.write("\\lx "+lexicalEntry.getChildText(LexiconDocumentData.LEXEME));
					Element morphElmt=lexicalEntry.getChild(LexiconDocumentData.MORPHOLOGY);
					if(morphElmt!=null){
						writer.write("\n\\u ");
						List<Element> segmentElmtList=morphElmt.getChildren(LexiconDocumentData.SEGMENT);
						for(int i=0; i<segmentElmtList.size(); i++){
							Element segmentElmt=segmentElmtList.get(i);
							writer.write(segmentElmt.getText());
							if(i<segmentElmtList.size()-1){
								writer.write(" ");
							}
						}
					}
					Element formElmt=lexicalEntry.getChild(LexiconDocumentData.FORM);
					if(formElmt.getChildren(LexiconDocumentData.ALTERNATE_FORM).size()>0){
						List<Element> altFormList=formElmt.getChildren(LexiconDocumentData.ALTERNATE_FORM);
						for(Element altFormElmt:altFormList){
							writer.write("\n\\a "+altFormElmt.getChildText(LexiconDocumentData.WRITTENFORM));
							Element altFormMorphElmt=altFormElmt.getChild(LexiconDocumentData.MORPHOLOGY);
							if(altFormMorphElmt!=null){
								writer.write("\n\\u ");
								List<Element> segmentAltElmtList=altFormMorphElmt.getChildren(LexiconDocumentData.SEGMENT);
								for(int i=0; i<segmentAltElmtList.size(); i++){
									Element segmentAltElmt=segmentAltElmtList.get(i);
									writer.write(segmentAltElmt.getText());
									if(i<segmentAltElmtList.size()-1){
										writer.write(" ");
									}
								}
							}
						}
					}

					Element senseElmt=lexicalEntry.getChild(LexiconDocumentData.SENSE);
					if(senseElmt!=null){
						List<Element> glossElmtList=senseElmt.getChildren(LexiconDocumentData.GLOSE);
						for(Element glossElmt:glossElmtList){
							writer.write("\n\\ge "+glossElmt.getText());
							String rx="";
							rx+=glossElmt.getAttributeValue(LexiconDocumentData.RX);
							writer.write("\n\\rx "+rx);
						}
					}

					if(lexicalEntry.getAttribute(LexiconDocumentData.DATE)!=null){
						writer.write("\n\\dt "+lexicalEntry.getAttributeValue(LexiconDocumentData.DATE));
					}

					writer.write("\n\n");

				}
				writer.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}


	/**
	 * create an annotation tree with tier mot, mb, ge, rx
	 */
	private List<AnnotationTree> createAnnotationTree(){
		Vector<TierImpl> tierVector=transcription.getTiers();
		List<AnnotationTree> annTreeList=new ArrayList<AnnotationTree>();
		for(TierImpl currentTier:tierVector){
			//if(currentTier.getName().startsWith(INTERLINEARIZE_WORD_TIER_NAME)){		// CC 26042012 BUG n'enregistrait pas le parse si noms des tiers ne contenaient pas mot, mb,...
			if(currentTier.getName().equals((String)getPreference(INTERLINEARIZE_TIER_WORD, transcription))){ // prend en compte le nom actuel des tiers
				TierImpl motTier=currentTier;
				AnnotationTree currAnnTree=new AnnotationTree(motTier);
				annTreeList.add(currAnnTree);
				//motTierList.add(motTier);
				Vector<TierImpl> motTierchildren= currentTier.getChildTiers();
				for(TierImpl childTier:motTierchildren){
					//if(childTier.getName().startsWith(INTERLINEARIZE_PARSE_TIER_NAME)){		// CC
					if(childTier.getName().startsWith((String)getPreference(INTERLINEARIZE_TIER_PARSE, transcription))){
						TierImpl mbTier=childTier;
						currAnnTree.setSegmentTier(mbTier);
						Vector<TierImpl> mbTierChildren=childTier.getChildTiers();
						for(TierImpl gerxTier:mbTierChildren){
							//if(gerxTier.getName().startsWith(INTERLINEARIZE_GLOSS_TIER_NAME)){	// CC
							if(gerxTier.getName().startsWith((String)getPreference(INTERLINEARIZE_TIER_GLOSS, transcription))){
								TierImpl geTier=gerxTier;
								currAnnTree.setGlossTier(geTier);
							//} else if(gerxTier.getName().startsWith(INTERLINEARIZE_POS_TIER_NAME)){	// CC
							} else if(gerxTier.getName().startsWith((String)getPreference(INTERLINEARIZE_TIER_POS, transcription))){
								TierImpl rxTier=gerxTier;
								currAnnTree.setPosTier(rxTier);
							}
						}
					}
				}
			}
		}
		return annTreeList;
	}
	
	/**export parseFile with annotation data
	 * */
	private void exportParseFile(String xmlPath){
		List<AnnotationTree> annTreeList=createAnnotationTree();
		if(!annTreeList.isEmpty()){
			Element parseRootElmt=exportParseData(annTreeList);
			ParseDocumentData.setDocument(new Document(parseRootElmt), new File(xmlPath));
			try {
				XMLOutputter output=new XMLOutputter(Format.getPrettyFormat());
				output.output(ParseDocumentData.DOCUMENT, new FileOutputStream(xmlPath));
			}
			catch (java.io.IOException e){
				
			}
		}
	}
	
	private Element exportParseData(List<AnnotationTree> annTreeList){
		Element parseRootElmt=new Element(ParseDocumentData.ROOT);
		
		SimpleDateFormat dateFormat=new SimpleDateFormat("dd/MMM/yyyy", Locale.ENGLISH);
		Attribute dateAttribute=new Attribute(ParseDocumentData.DT,dateFormat.format(new Date()));
		parseRootElmt.setAttribute(dateAttribute);
		Vector<Annotation> annotations=new Vector<Annotation>();
		for(AnnotationTree annTree:annTreeList){
			if(annTree.getWordTier()!=null){
				annotations.addAll(annTree.getWordTier().getAnnotations());
			}
		}
		Collections.sort(annotations, new Comparator<Annotation>() {	
			@Override
			public int compare(Annotation o1, Annotation o2) {
			return o1.getValue().compareTo(o2.getValue());
			}
		});
		List<Element> parseElementList=new ArrayList<Element>();
		for(Annotation motAnn:annotations){
			for(AnnotationTree annTree:annTreeList){
				if (motAnn.getTier().equals(annTree.getWordTier())){
					if(motAnn.getChildrenOnTier(annTree.getSegmentTier()).size()>0){
						Element parseElmt=new Element(ParseDocumentData.PARSE);
						Element wFormElmt=new Element(LexiconDocumentData.WRITTENFORM);
						wFormElmt.setText(motAnn.getValue());
						Vector<Annotation>motAnnChildren=motAnn.getChildrenOnTier(annTree.getSegmentTier());
						Collections.sort(motAnnChildren);
						Element morphElmt=new Element(LexiconDocumentData.MORPHOLOGY);
						for(Annotation mbAnn:motAnnChildren){
							Element segmentElmt=new Element(LexiconDocumentData.SEGMENT);
							morphElmt.addContent(segmentElmt);
							if(mbAnn!=null){
								segmentElmt.setText(mbAnn.getValue());
								//}
								Vector<Annotation> geAnnVect=mbAnn.getChildrenOnTier(annTree.getGlossTier());
								if(!geAnnVect.isEmpty()){
									Attribute glossAtt=new Attribute(ParseDocumentData.GE, geAnnVect.get(0).getValue());
									segmentElmt.setAttribute(glossAtt);
								}
								else{
									segmentElmt.setAttribute(new Attribute(ParseDocumentData.GE, ""));
								}
								Vector<Annotation> rxAnnVect=mbAnn.getChildrenOnTier(annTree.getPosTier());
								if(!rxAnnVect.isEmpty()){
									Attribute rxAtt=new Attribute(ParseDocumentData.RX, rxAnnVect.get(0).getValue());
									segmentElmt.setAttribute(rxAtt);	
								}
								else{
									segmentElmt.setAttribute(new Attribute(ParseDocumentData.RX, ""));
								}
							}
						}

						boolean insertElement=true;
						if(!parseElementList.isEmpty()){
							for(Element currentParseElmt:parseElementList){
								Element currentWForm=currentParseElmt.getChild(LexiconDocumentData.WRITTENFORM);
								if(currentWForm.getText().equals(wFormElmt.getText())){
									Element currentMorphElmt=currentParseElmt.getChild(LexiconDocumentData.MORPHOLOGY);
									List<Element> currentSegmentList=currentMorphElmt.getChildren(LexiconDocumentData.SEGMENT);
									List<Element> segmentList=morphElmt.getChildren(LexiconDocumentData.SEGMENT);
									if(segmentList.size()==currentSegmentList.size()){
										int cmpt=0;
										for(int i=0; i<currentSegmentList.size(); i++){
											Element currentSegmentElmt=currentSegmentList.get(i);
											Element segElmt=segmentList.get(i);
											if(currentSegmentElmt.getText().equals(segElmt.getText())){
												if(currentSegmentElmt.getAttributeValue(ParseDocumentData.GE).equals(segElmt.getAttributeValue(ParseDocumentData.GE))){
													if(currentSegmentElmt.getAttributeValue(ParseDocumentData.RX).equals(segElmt.getAttributeValue(ParseDocumentData.RX))){
														cmpt++;
													}
												}
											}
										}
										if(cmpt==currentSegmentList.size()){
											insertElement=false;
											break;
										}
									}
								}
							}
						}
						if(insertElement){
							parseElementList.add(parseElmt);
							parseElmt.addContent(wFormElmt);
							parseElmt.addContent(morphElmt);
							parseRootElmt.addContent(parseElmt);
						}
					}
				}
			}
		}
		return parseRootElmt;
	}
	
	private void saveLexicon(){
		try {
			XMLOutputter output=new XMLOutputter(Format.getPrettyFormat());
			output.output(LexiconDocumentData.DOCUMENT, new FileOutputStream(LexiconDocumentData.fileReference));
		}
		catch (java.io.IOException ex){
			System.out.println("Can't save the lexicon");
		}
	}
	
	/**
	 * Notification of a change in the preferences. Also called after opening a
	 * file to restore state.
	 */
	public void preferencesChanged() {
		Object pref = Preferences.get(LX_SOURCE, transcription);

		if (pref != null) {
			lexicalPath = (String) pref;
			if (lexicalPath.startsWith(".")) {
				lexicalPath = lexicalPath.replace("\\", "/");
				String eafPath = transcription.getFullPath();
				if (eafPath.startsWith("file:")) {
					eafPath = eafPath.substring(5);
				}
				eafPath = eafPath.replace("\\", "/");
				ClientLogger.LOG.info(eafPath);
				ClientLogger.LOG.info(lexicalPath);
				lexicalPath = FileUtility.getAbsolutePath(eafPath, lexicalPath);
			}
			lxPathLabel.setText(lexicalPath);
		}
	}

	//create preferences map
	private HashMap<String, String> createMap(int type){
		HashMap<String, String> map=new HashMap<String, String>();
		if(type==INTERLINEARIZE_MAP){
			map.put(INTERLINEARIZE_WORD_TIER_NAME, (String)getPreference(INTERLINEARIZE_TIER_WORD, transcription));
			map.put(INTERLINEARIZE_PARSE_TIER_NAME, (String)getPreference(INTERLINEARIZE_TIER_PARSE, transcription));
			map.put(INTERLINEARIZE_GLOSS_TIER_NAME, (String)getPreference(INTERLINEARIZE_TIER_GLOSS, transcription));
			map.put(INTERLINEARIZE_POS_TIER_NAME, (String)getPreference(INTERLINEARIZE_TIER_POS, transcription));
		}
		else if(type==ANNOTATION_MAP){
			map.put(ANNOTATION_PARSE_TIER_NAME, (String)getPreference(ANNOTATION_TIER_PARSE, transcription));
			map.put(ANNOTATION_GLOSS_TIER_NAME, (String)getPreference(ANNOTATION_TIER_GLOSS, transcription));
			map.put(ANNOTATION_POS_TIER_NAME, (String)getPreference(ANNOTATION_TIER_POS, transcription));
		}
		return map;
	}

	/**
	 * Returns the viewer manager.
	 *
	 * @return the viewer manager
	 */
	public Transcription getTranscription() {
		return transcription;
	}

	/**
	 * Sets the viewer manager.
	 *
	 * @param viewerManager the viewer manager
	 */
	public void setTranscription(TranscriptionImpl transcription) {
		this.transcription = transcription;
	}

	/**
	 * Each ActionCommand represents a JMenuItem from the menuBar of the class
	 * 
	 * @param e actionEvent
	 */
	public void actionPerformed(ActionEvent e) {
		HashMap<String, String> interlinearMap=createMap(INTERLINEARIZE_MAP);
		HashMap<String, String> annotateMap=createMap(ANNOTATION_MAP);
		
		if (e.getActionCommand()==OPEN_LEXICON) {
			selectLXSource(ElanFileFilter.ELAN_LEX_TYPE);
		}
		else if(e.getActionCommand()==CREATE_LEXICON){
			createLXSource();
		}
		else if (e.getActionCommand()==IMPORT_LEXICON){
			selectLXSource(ElanFileFilter.TEXT_TYPE);
		}
		else if(e.getActionCommand()==SAVE_LEXICON){
			if(LexiconDocumentData.documentModified){
				saveLexicon();
				LexiconDocumentData.isModified(false);
			}
		}
		else if(e.getActionCommand()==EXPORT_PARSE){
			exportParseData();
		} 
		else if(e.getActionCommand()==OPEN_PARSE){
			selectParseData();
		}
		else if(e.getActionCommand()==EXPORT_MERGE_PARSE){
			exportAndMergeParseData();
		}
		else if(e.getActionCommand()==EXPORT_LEXICON){
			exportLexicon();
		}
		else if(e.getActionCommand()==CONFIGURE_INTERLINEARIZE_TIERS){
			TierInterlinearizeParameterFrame frame=new TierInterlinearizeParameterFrame(this, transcription, interlinearMap);
			frame.setLocationRelativeTo(getParent());
		}
		else if(e.getActionCommand()==CONFIGURE_ANNOTATION_TIERS){
			TierAnnotationParameterFrame frame=new TierAnnotationParameterFrame(this, transcription, annotateMap);
			frame.setLocationRelativeTo(getParent());
		}
		else if(e.getActionCommand()==RENAME_INTERLINEARIZE_TIERS){
			TierRenameInterlinearizeFrame renameFrame=new TierRenameInterlinearizeFrame(this, transcription, interlinearMap);
			renameFrame.setLocationRelativeTo(getParent());
		}
		else if(e.getActionCommand()==RENAME_ANNOTATION_TIERS){
			TierRenameAnnotationFrame renameFrame=new TierRenameAnnotationFrame(this, transcription, annotateMap);
			renameFrame.setLocationRelativeTo(getParent());
		}
		else if(e.getActionCommand()==MORPHEME_BREAK_CHAR){
			MorphemeBreakCharacterFrame morphemeFrame=new MorphemeBreakCharacterFrame(this, transcription);
			morphemeFrame.setLocationRelativeTo(getParent());
		}
	}
	
	public void setTierPreferences(String key, String value){
		setPreference(key, value, transcription);
	}
	
	public Object getTierPreferences(String key){
		return getPreference(key, transcription);
	}
	
	public Object getMorphemeBreakPreferences(String key){
		Object preference=getPreference(key, transcription);
		if(preference==null){
			if(key.equals(SUFFIX_CHAR)){
				preference=DEFAULT_SUFFIX_VALUE;
				setPreference(SUFFIX_CHAR, preference, transcription);
			} else if(key.equals(STEM_CHAR)){
				preference=DEFAULT_STEM_VALUE;
				setPreference(STEM_CHAR, preference, transcription);				
			} else if(key.equals(TRANS_BREAK_CHAR)){
				preference=DEFAULT_TRANSMIT_BREAK_CHAR;
				setPreference(TRANS_BREAK_CHAR, preference, transcription);
			}			
		}
		return preference;
	}
	
	public void setMorphemeBreakPreferences(String key, Object value){
		setPreference(key, value, transcription);
	}

	@Override
	public void controllerUpdate(ControllerEvent event) {
		// TODO Auto-generated method stub

	}

	@Override
	public void updateActiveAnnotation() {
	}

	@Override
	public void updateSelection() {
		// TODO Auto-generated method stub

	}

	@Override
	public Tier getTier() {
		return tier;
	}

	@Override
	public void setTier(Tier tier) {
		this.tier=(TierImpl) tier;
		if (tier == null) {
			setPreference("LexiconViewer.TierName", tier, getViewerManager().getTranscription());
		}
		else{
			setPreference("LexiconViewer.TierName", tier.getName(), getViewerManager().getTranscription());
			preferencesChanged();
		}
		updateSelection();
	}

	public void initializeInterlinearizeProcess(){
		interlinearizeProcess=new InterlinearizationProcess(lxTableModel, parsingMorphemeList, this);
		leftPanel.removeAll();
		rightPanel.removeAll();
		initTable();
		placeTable();
		lxTable.updateLocale();
		leftPanel.repaint();
		rightPanel.repaint();
	}
	
	@Override
	public void ACMEdited(ACMEditEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void updateLocale() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void itemStateChanged(ItemEvent e) {
		boolean dataChanged=true;
		if(e.getStateChange()==ItemEvent.DESELECTED){
			if(e.getItemSelectable()==lexiconLinkedFile){
				if(!lexiconLinkedFile.isSelected()){
					if(LexiconDocumentData.documentModified){
						int option = JOptionPane.showConfirmDialog(null, "The current dictionary has been modified, do you want to save it ?", ElanLocale.getString("Menu.File.Save"),
			                    JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);

			            if (option==JOptionPane.YES_OPTION) {
			            	saveLexicon();
			            	setPreference(LX_SOURCE, null, transcription);
			            	LexiconDocumentData.isModified(false);
			            	LexiconDocumentData.setDocument(null, null);
			            }
			            else if(option==JOptionPane.NO_OPTION){
			            	setPreference(LX_SOURCE, null, transcription);
			            	LexiconDocumentData.isModified(false);
			            	LexiconDocumentData.setDocument(null, null);
			            }
			            else if(option==JOptionPane.CANCEL_OPTION){
			            	lexiconLinkedFile.setSelected(true);
			            	dataChanged=false;
			            }
					}
					else{
						setPreference(LX_SOURCE, null, transcription);
		            	LexiconDocumentData.isModified(false);
		            	LexiconDocumentData.setDocument(null, null);
					}
				}
			}
			if(e.getItemSelectable()==parseLinkedFile){
				if(!parseLinkedFile.isSelected()){
					setPreference(PARSE_SOURCE, null, transcription);
				}
			}
			if(dataChanged){
				getPreferences();
				updateLinkedFile();
			}
		}
	}
}